@@ -144,9 +144,9 @@ def cmp_by_dirs_py(path1, path2):
 
     :param path1: first path
     :param path2: second path
-    :return: positive number if ``path1`` comes first,
+    :return: negative number if ``path1`` comes first,
         0 if paths are equal,
-        and negative number if ``path2`` sorts first
+        and positive number if ``path2`` sorts first
     """
     if not isinstance(path1, str):
         raise TypeError("'path1' must be a plain string, not %s: %r"
@@ -166,9 +166,9 @@ def _cmp_path_by_dirblock_py(path1, path2):
 
     :param path1: first path
     :param path2: the second path
-    :return: positive number if ``path1`` comes first,
+    :return: negative number if ``path1`` comes first,
         0 if paths are equal
-        and a negative number if ``path2`` sorts first
+        and a positive number if ``path2`` sorts first
     """
     if not isinstance(path1, str):
         raise TypeError("'path1' must be a plain string, not %s: %r"
@@ -200,7 +200,9 @@ def _read_dirblocks_py(state):
     fields = text.split('\0')
     # Remove the last blank entry
     trailing = fields.pop()
-    assert trailing == ''
+    if trailing != '':
+        raise AssertionError("dirstate line %r has trailing garbage: %r" 
+            % (trailing,))
     # consider turning fields into a tuple.
 
     # skip the first field which is the trailing null from the header.
@@ -217,11 +219,12 @@ def _read_dirblocks_py(state):
     expected_field_count = entry_size * state._num_entries
     field_count = len(fields)
     # this checks our adjustment, and also catches file too short.
-    assert field_count - cur == expected_field_count, \
-        'field count incorrect %s != %s, entry_size=%s, '\
-        'num_entries=%s fields=%r' % (
+    if field_count - cur != expected_field_count:
+        raise AssertionError(
+            'field count incorrect %s != %s, entry_size=%s, '\
+            'num_entries=%s fields=%r' % (
             field_count - cur, expected_field_count, entry_size,
-            state._num_entries, fields)
+            state._num_entries, fields))
 
     if num_present_parents == 1:
         # Bind external functions to local names
@@ -269,7 +272,8 @@ def _read_dirblocks_py(state):
                      ),
                      ])
             trailing = next()
-            assert trailing == '\n'
+            if trailing != '\n':
+                raise ValueError("trailing garbage in dirstate: %r" % trailing)
             # append the entry to the current block
             append_entry(entry)
         state._split_root_dirblock_into_contents()
