@@ -14,14 +14,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Tests for bzr add performance."""
+"""Tests for `bzrlib.inventory.Inventory` performance."""
 
 
-import bzrlib
+import bzrlib.inventory
 from bzrlib.benchmarks import Benchmark
 
 
 class InvBenchmark(Benchmark):
+    """Benchmarks for building large inventories."""
 
     def test_make_10824_inv_entries(self):
         """Making 10824 inv entries should be quick."""
