@@ -195,7 +195,6 @@ class KernelLikeInventoryCreator(TreeCreator):
             returns only an in-memory Inventory, so it should always be None.
         :return: An Inventory object.
         """
-        assert root is None, "Cannot create a memory inventory in a on disk."
         cache_dir = self._get_cache_dir()
         if cache_dir is None:
             return self._create_and_return()
@@ -210,7 +209,12 @@ class KernelLikeInventoryCreator(TreeCreator):
                                                  link_bzr=True,
                                                  hot_cache=False)
         tree = creator.create('.')
-        return tree.basis_tree().inventory
+        basis = tree.basis_tree()
+        basis.lock_read()
+        try:
+            return basis.inventory
+        finally:
+            basis.unlock()
 
     def _open_cached(self, cache_dir):
         f = open(cache_dir + '/inventory', 'rb')
