@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from StringIO import StringIO
+
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
@@ -30,26 +32,35 @@ from bzrlib.trace import note
 
 
 def read_bundle_from_url(url):
-    """Read a bundle from a given URL.
+    return read_mergeable_from_url(url, _do_directive=False)
 
-    :return: A BundleReader, may raise NotABundle if the target 
-            is not a proper bundle.
+
+def read_mergeable_from_url(url, _do_directive=True, possible_transports=None):
+    """Read mergable object from a given URL.
+
+    :return: An object supporting get_target_revision.  Raises NotABundle if
+        the target is not a mergeable type.
     """
-    url = urlutils.normalize_url(url)
-    url, filename = urlutils.split(url, exclude_trailing_slash=False)
-    if not filename:
+    child_transport = get_transport(url,
+        possible_transports=possible_transports)
+    transport = child_transport.clone('..')
+    filename = transport.relpath(child_transport.base)
+    if filename.endswith('/'):
         # A path to a directory was passed in
         # definitely not a bundle
         raise errors.NotABundle('A directory cannot be a bundle')
+    mergeable, transport = read_mergeable_from_transport(transport, filename,
+                                                         _do_directive)
+    return mergeable
 
+
+def read_mergeable_from_transport(transport, filename, _do_directive=True):
     # All of this must be in the try/except
     # Some transports cannot detect that we are trying to read a
     # directory until we actually issue read() on the handle.
     try:
-        transport = get_transport(url)
-
         def get_bundle(transport):
-            return transport.get(filename)
+            return transport.get(filename), transport
 
         def redirected_transport(transport, exception, redirection_notice):
             note(redirection_notice)
@@ -60,12 +71,17 @@ def read_bundle_from_url(url):
             return get_transport(url)
 
         try:
-            f = do_catching_redirections(get_bundle, transport,
-                                         redirected_transport)
+            f, transport = do_catching_redirections(get_bundle, transport,
+                                                    redirected_transport)
         except errors.TooManyRedirections:
             raise errors.NotABundle(str(url))
 
-        return _serializer.read_bundle(f)
+        if _do_directive:
+            from bzrlib.merge_directive import MergeDirective
+            directive = MergeDirective.from_lines(f.readlines())
+            return directive, transport
+        else:
+            return _serializer.read_bundle(f), transport
     except (errors.TransportError, errors.PathError), e:
         raise errors.NotABundle(str(e))
     except (IOError,), e:
@@ -77,4 +93,6 @@ def read_bundle_from_url(url):
         # StubSFTPServer does fail during get() (because of prefetch) 
         # so it has an opportunity to translate the error.
         raise errors.NotABundle(str(e))
-
+    except errors.NotAMergeDirective:
+        f.seek(0)
+        return _serializer.read_bundle(f), transport
