@@ -107,17 +107,20 @@ class BundleWriter(object):
     @staticmethod
     def encode_name(content_kind, revision_id, file_id=None):
         """Encode semantic ids as a container name"""
-        assert content_kind in ('revision', 'file', 'inventory', 'signature',
-                                'info')
-
+        if content_kind not in ('revision', 'file', 'inventory', 'signature',
+                'info'):
+            raise ValueError(content_kind)
         if content_kind == 'file':
-            assert file_id is not None
+            if file_id is None:
+                raise AssertionError()
         else:
-            assert file_id is None
+            if file_id is not None:
+                raise AssertionError()
         if content_kind == 'info':
-            assert revision_id is None
-        else:
-            assert revision_id is not None
+            if revision_id is not None:
+                raise AssertionError()
+        elif revision_id is None:
+            raise AssertionError()
         names = [n.replace('/', '//') for n in
                  (content_kind, revision_id, file_id) if n is not None]
         return '/'.join(names)
@@ -144,20 +147,33 @@ class BundleReader(object):
     body
     """
 
-    def __init__(self, fileobj):
+    def __init__(self, fileobj, stream_input=True):
+        """Constructor
+
+        :param fileobj: a file containing a bzip-encoded container
+        :param stream_input: If True, the BundleReader stream input rather than
+            reading it all into memory at once.  Reading it into memory all at
+            once is (currently) faster.
+        """
         line = fileobj.readline()
         if line != '\n':
             fileobj.readline()
         self.patch_lines = []
-        self._container = pack.ContainerReader(
-            iterablefile.IterableFile(self.iter_decode(fileobj)))
+        if stream_input:
+            source_file = iterablefile.IterableFile(self.iter_decode(fileobj))
+        else:
+            source_file = StringIO(bz2.decompress(fileobj.read()))
+        self._container_file = source_file
 
     @staticmethod
     def iter_decode(fileobj):
         """Iterate through decoded fragments of the file"""
         decompressor = bz2.BZ2Decompressor()
         for line in fileobj:
-            yield decompressor.decompress(line)
+            try:
+                yield decompressor.decompress(line)
+            except EOFError:
+                return
 
     @staticmethod
     def decode_name(name):
@@ -189,17 +205,16 @@ class BundleReader(object):
         :return: a generator of (bytes, metadata, content_kind, revision_id,
             file_id)
         """
-        iterator = self._container.iter_records()
-        for names, meta_bytes in iterator:
+        iterator = pack.iter_records_from_file(self._container_file)
+        for names, bytes in iterator:
             if len(names) != 1:
                 raise errors.BadBundle('Record has %d names instead of 1'
                                        % len(names))
-            metadata = bencode.bdecode(meta_bytes(None))
+            metadata = bencode.bdecode(bytes)
             if metadata['storage_kind'] == 'header':
                 bytes = None
             else:
                 _unused, bytes = iterator.next()
-                bytes = bytes(None)
             yield (bytes, metadata) + self.decode_name(names[0][0])
 
 
@@ -315,12 +330,12 @@ class BundleWriteOperation(object):
                 if revision_id in self.base_ancestry:
                     continue
                 new_revision_ids.add(revision_id)
-                pending.extend(vf.get_parents(revision_id))
+                pending.extend(vf.get_parent_map([revision_id])[revision_id])
             yield vf, file_id, new_revision_ids
 
     def write_files(self):
         """Write bundle records for all revisions of all files"""
-        for vf, file_id, revision_ids in self.iter_file_revisions_aggressive():
+        for vf, file_id, revision_ids in self.iter_file_revisions():
             self.add_mp_records('file', file_id, vf, revision_ids)
 
     def write_revisions(self):
@@ -331,8 +346,9 @@ class BundleWriteOperation(object):
             revision_order.remove(self.target)
             revision_order.append(self.target)
         self.add_mp_records('inventory', None, inv_vf, revision_order)
-        parents_list = self.repository.get_parents(revision_order)
-        for parents, revision_id in zip(parents_list, revision_order):
+        parent_map = self.repository.get_parent_map(revision_order)
+        for revision_id in revision_order:
+            parents = parent_map.get(revision_id, None)
             revision_text = self.repository.get_revision_xml(revision_id)
             self.bundle.add_fulltext_record(revision_text, parents,
                                        'revision', revision_id)
@@ -363,8 +379,9 @@ class BundleWriteOperation(object):
         revision_ids = list(multiparent.topo_iter(vf, revision_ids))
         mpdiffs = vf.make_mpdiffs(revision_ids)
         sha1s = vf.get_sha1s(revision_ids)
+        parent_map = vf.get_parent_map(revision_ids)
         for mpdiff, revision_id, sha1, in zip(mpdiffs, revision_ids, sha1s):
-            parents = vf.get_parents(revision_id)
+            parents = parent_map[revision_id]
             text = ''.join(mpdiff.to_patch())
             self.bundle.add_multiparent_record(text, sha1, parents, repo_kind,
                                                revision_id, file_id)
@@ -382,11 +399,17 @@ class BundleInfoV4(object):
     def install(self, repository):
         return self.install_revisions(repository)
 
-    def install_revisions(self, repository):
-        """Install this bundle's revisions into the specified repository"""
+    def install_revisions(self, repository, stream_input=True):
+        """Install this bundle's revisions into the specified repository
+
+        :param target_repo: The repository to install into
+        :param stream_input: If True, will stream input rather than reading it
+            all into memory at once.  Reading it into memory all at once is
+            (currently) faster.
+        """
         repository.lock_write()
         try:
-            ri = RevisionInstaller(self.get_bundle_reader(),
+            ri = RevisionInstaller(self.get_bundle_reader(stream_input),
                                    self._serializer, repository)
             return ri.install()
         finally:
@@ -399,9 +422,15 @@ class BundleInfoV4(object):
         """
         return None, self.target, 'inapplicable'
 
-    def get_bundle_reader(self):
+    def get_bundle_reader(self, stream_input=True):
+        """Return a new BundleReader for the associated bundle
+
+        :param stream_input: If True, the BundleReader stream input rather than
+            reading it all into memory at once.  Reading it into memory all at
+            once is (currently) faster.
+        """
         self._fileobj.seek(0)
-        return BundleReader(self._fileobj)
+        return BundleReader(self._fileobj, stream_input)
 
     def _get_real_revisions(self):
         if self.__real_revisions is None:
@@ -444,35 +473,58 @@ class RevisionInstaller(object):
         self._info = None
 
     def install(self):
-        """Perform the installation"""
+        """Perform the installation.
+        
+        Must be called with the Repository locked.
+        """
+        self._repository.start_write_group()
+        try:
+            result = self._install_in_write_group()
+        except:
+            self._repository.abort_write_group()
+            raise
+        self._repository.commit_write_group()
+        return result
+
+    def _install_in_write_group(self):
         current_file = None
         current_versionedfile = None
         pending_file_records = []
+        inventory_vf = None
+        pending_inventory_records = []
         added_inv = set()
         target_revision = None
         for bytes, metadata, repo_kind, revision_id, file_id in\
             self._container.iter_records():
             if repo_kind == 'info':
-                assert self._info is None
+                if self._info is not None:
+                    raise AssertionError()
                 self._handle_info(metadata)
-            if repo_kind != 'file':
-                self._install_mp_records(current_versionedfile,
-                    pending_file_records)
+            if (repo_kind, file_id) != ('file', current_file):
+                if len(pending_file_records) > 0:
+                    self._install_mp_records(current_versionedfile,
+                                             pending_file_records)
                 current_file = None
                 current_versionedfile = None
                 pending_file_records = []
-                if repo_kind == 'inventory':
-                    self._install_inventory(revision_id, metadata, bytes)
-                if repo_kind == 'revision':
-                    target_revision = revision_id
-                    self._install_revision(revision_id, metadata, bytes)
-                if repo_kind == 'signature':
-                    self._install_signature(revision_id, metadata, bytes)
+            if len(pending_inventory_records) > 0 and repo_kind != 'inventory':
+                self._install_inventory_records(inventory_vf,
+                                                pending_inventory_records)
+                pending_inventory_records = []
+            if repo_kind == 'inventory':
+                if inventory_vf is None:
+                    inventory_vf = self._repository.get_inventory_weave()
+                if revision_id not in inventory_vf:
+                    pending_inventory_records.append((revision_id, metadata,
+                                                      bytes))
+            if repo_kind == 'revision':
+                target_revision = revision_id
+                self._install_revision(revision_id, metadata, bytes)
+            if repo_kind == 'signature':
+                self._install_signature(revision_id, metadata, bytes)
             if repo_kind == 'file':
-                if file_id != current_file:
-                    self._install_mp_records(current_versionedfile,
-                        pending_file_records)
-                    current_file = file_id
+                current_file = file_id
+                if current_versionedfile is None:
                     current_versionedfile = \
                         self._repository.weave_store.get_weave_or_empty(
                         file_id, self._repository.get_transaction())
@@ -501,30 +553,28 @@ class RevisionInstaller(object):
                       records if r not in versionedfile]
         versionedfile.add_mpdiffs(vf_records)
 
-    def _install_inventory(self, revision_id, metadata, text):
-        vf = self._repository.get_inventory_weave()
-        if revision_id in vf:
-            return
-        parent_ids = metadata['parents']
+    def _install_inventory_records(self, vf, records):
         if self._info['serializer'] == self._repository._serializer.format_num:
-            return self._install_mp_records(vf, [(revision_id, metadata,
-                                                  text)])
-        parents = [self._repository.get_inventory(p)
-                   for p in parent_ids]
-        parent_texts = [self._source_serializer.write_inventory_to_string(p)
-                        for p in parents]
-        target_lines = multiparent.MultiParent.from_patch(text).to_lines(
-            parent_texts)
-        sha1 = osutils.sha_strings(target_lines)
-        if sha1 != metadata['sha1']:
-            raise errors.BadBundle("Can't convert to target format")
-        target_inv = self._source_serializer.read_inventory_from_string(
-            ''.join(target_lines))
-        self._handle_root(target_inv, parent_ids)
-        try:
-            self._repository.add_inventory(revision_id, target_inv, parent_ids)
-        except errors.UnsupportedInventoryKind:
-            raise errors.IncompatibleRevision(repr(self._repository))
+            return self._install_mp_records(vf, records)
+        for revision_id, metadata, bytes in records:
+            parent_ids = metadata['parents']
+            parents = [self._repository.get_inventory(p)
+                       for p in parent_ids]
+            p_texts = [self._source_serializer.write_inventory_to_string(p)
+                       for p in parents]
+            target_lines = multiparent.MultiParent.from_patch(bytes).to_lines(
+                p_texts)
+            sha1 = osutils.sha_strings(target_lines)
+            if sha1 != metadata['sha1']:
+                raise errors.BadBundle("Can't convert to target format")
+            target_inv = self._source_serializer.read_inventory_from_string(
+                ''.join(target_lines))
+            self._handle_root(target_inv, parent_ids)
+            try:
+                self._repository.add_inventory(revision_id, target_inv,
+                                               parent_ids)
+            except errors.UnsupportedInventoryKind:
+                raise errors.IncompatibleRevision(repr(self._repository))
 
     def _handle_root(self, target_inv, parent_ids):
         revision_id = target_inv.revision_id
@@ -542,7 +592,11 @@ class RevisionInstaller(object):
     def _install_revision(self, revision_id, metadata, text):
         if self._repository.has_revision(revision_id):
             return
-        self._repository._add_revision_text(revision_id, text)
+        if self._info['serializer'] == self._repository._serializer.format_num:
+            self._repository._add_revision_text(revision_id, text)
+        else:
+            revision = self._source_serializer.read_revision_from_string(text)
+            self._repository.add_revision(revision.revision_id, revision)
 
     def _install_signature(self, revision_id, metadata, text):
         transaction = self._repository.get_transaction()
