@@ -185,6 +185,13 @@ class AlreadyBuilding(BzrError):
     _fmt = "The tree builder is already building a tree."
 
 
+class BranchError(BzrError):
+    """Base class for concrete 'errors about a branch'."""
+
+    def __init__(self, branch):
+        BzrError.__init__(self, branch=branch)
+
+
 class BzrCheckError(InternalBzrError):
     
     _fmt = "Internal check failed: %(message)s"
@@ -305,6 +312,11 @@ class NoSuchIdInRepository(NoSuchId):
         BzrError.__init__(self, repository=repository, file_id=file_id)
 
 
+class NotStacked(BranchError):
+
+    _fmt = "The branch '%(branch)s' is not stacked."
+
+
 class InventoryModified(InternalBzrError):
 
     _fmt = ("The current inventory for the tree %(tree)r has been modified,"
@@ -541,11 +553,13 @@ class InvalidURL(PathError):
 
 class InvalidURLJoin(PathError):
 
-    _fmt = 'Invalid URL join request: "%(args)s"%(extra)s'
+    _fmt = "Invalid URL join request: %(reason)s: %(base)r + %(join_args)r"
 
-    def __init__(self, msg, base, args):
-        PathError.__init__(self, base, msg)
-        self.args = [base] + list(args)
+    def __init__(self, reason, base, join_args):
+        self.reason = reason
+        self.base = base
+        self.join_args = join_args
+        PathError.__init__(self, base, reason)
 
 
 class UnknownHook(BzrError):
@@ -566,6 +580,28 @@ class UnsupportedProtocol(PathError):
         PathError.__init__(self, url, extra=extra)
 
 
+class UnstackableBranchFormat(BzrError):
+
+    _fmt = ("The branch '%(url)s'(%(format)s) is not a stackable format. "
+        "You will need to upgrade the branch to permit branch stacking.")
+
+    def __init__(self, format, url):
+        BzrError.__init__(self)
+        self.format = format
+        self.url = url
+
+
+class UnstackableRepositoryFormat(BzrError):
+
+    _fmt = ("The repository '%(url)s'(%(format)s) is not a stackable format. "
+        "You will need to upgrade the repository to permit branch stacking.")
+
+    def __init__(self, format, url):
+        BzrError.__init__(self)
+        self.format = format
+        self.url = url
+
+
 class ReadError(PathError):
     
     _fmt = """Error reading from %(path)r."""
@@ -858,17 +894,6 @@ class ReadOnlyError(LockError):
         self.obj = obj
 
 
-class ReadOnlyLockError(LockError):
-
-    _fmt = "Cannot acquire write lock on %(fname)s. %(msg)s"
-
-    @symbol_versioning.deprecated_method(symbol_versioning.zero_ninetytwo)
-    def __init__(self, fname, msg):
-        LockError.__init__(self, '')
-        self.fname = fname
-        self.msg = msg
-
-
 class LockFailed(LockError):
 
     internal_error = False
@@ -1030,18 +1055,6 @@ class NoSuchRevision(InternalBzrError):
         BzrError.__init__(self, branch=branch, revision=revision)
 
 
-# zero_ninetyone: this exception is no longer raised and should be removed
-class NotLeftParentDescendant(InternalBzrError):
-
-    _fmt = ("Revision %(old_revision)s is not the left parent of"
-            " %(new_revision)s, but branch %(branch_location)s expects this")
-
-    def __init__(self, branch, old_revision, new_revision):
-        BzrError.__init__(self, branch_location=branch.base,
-                          old_revision=old_revision,
-                          new_revision=new_revision)
-
-
 class RangeInChangeOption(BzrError):
 
     _fmt = "Option --change does not accept revision ranges"
@@ -1173,13 +1186,10 @@ class AmbiguousBase(BzrError):
         self.bases = bases
 
 
-class NoCommits(BzrError):
+class NoCommits(BranchError):
 
     _fmt = "Branch %(branch)s has no commits."
 
-    def __init__(self, branch):
-        BzrError.__init__(self, branch=branch)
-
 
 class UnlistableStore(BzrError):
 
@@ -1443,6 +1453,14 @@ class SmartProtocolError(TransportError):
         self.details = details
 
 
+class UnknownSmartMethod(InternalBzrError):
+
+    _fmt = "The server does not recognise the '%(verb)s' request."
+
+    def __init__(self, verb):
+        self.verb = verb
+
+
 # A set of semi-meaningful errors which can be thrown
 class TransportNotPossible(TransportError):
 
@@ -2033,6 +2051,11 @@ class UpgradeRequired(BzrError):
         self.path = path
 
 
+class RepositoryUpgradeRequired(UpgradeRequired):
+
+    _fmt = "To use this feature you must upgrade your repository at %(path)s."
+
+
 class LocalRequiresBoundBranch(BzrError):
 
     _fmt = "Cannot perform local-only commits on unbound branches."
@@ -2182,6 +2205,7 @@ class NoSmartServer(NotBranchError):
 
     _fmt = "No smart server available at %(url)s"
 
+    @symbol_versioning.deprecated_method(symbol_versioning.one_four)
     def __init__(self, url):
         self.url = url
 
@@ -2504,6 +2528,18 @@ class BzrDirError(BzrError):
         BzrError.__init__(self, bzrdir=bzrdir, display_url=display_url)
 
 
+class UnsyncedBranches(BzrDirError):
+
+    _fmt = ("'%(display_url)s' is not in sync with %(target_url)s.  See"
+            " bzr help sync-for-reconfigure.")
+
+    def __init__(self, bzrdir, target_branch):
+        BzrDirError.__init__(self, bzrdir)
+        import bzrlib.urlutils as urlutils
+        self.target_url = urlutils.unescape_for_display(target_branch.base,
+                                                        'ascii')
+
+
 class AlreadyBranch(BzrDirError):
 
     _fmt = "'%(display_url)s' is already a branch."
@@ -2524,6 +2560,16 @@ class AlreadyLightweightCheckout(BzrDirError):
     _fmt = "'%(display_url)s' is already a lightweight checkout."
 
 
+class AlreadyUsingShared(BzrDirError):
+
+    _fmt = "'%(display_url)s' is already using a shared repository."
+
+
+class AlreadyStandalone(BzrDirError):
+
+    _fmt = "'%(display_url)s' is already standalone."
+
+
 class ReconfigurationNotSupported(BzrDirError):
 
     _fmt = "Requested reconfiguration of '%(display_url)s' is not supported."
@@ -2582,6 +2628,36 @@ class UnsupportedTimezoneFormat(BzrError):
         self.timezone = timezone
 
 
+class CommandAvailableInPlugin(StandardError):
+    
+    internal_error = False
+
+    def __init__(self, cmd_name, plugin_metadata, provider):
+        
+        self.plugin_metadata = plugin_metadata
+        self.cmd_name = cmd_name
+        self.provider = provider
+
+    def __str__(self):
+
+        _fmt = ('"%s" is not a standard bzr command. \n' 
+                'However, the following official plugin provides this command: %s\n'
+                'You can install it by going to: %s'
+                % (self.cmd_name, self.plugin_metadata['name'], 
+                    self.plugin_metadata['url']))
+
+        return _fmt
+
+
+class NoPluginAvailable(BzrError):
+    pass    
+
+
+class NotATerminal(BzrError):
+
+    _fmt = 'Unable to ask for a password without real terminal.'
+
+
 class UnableEncodePath(BzrError):
 
     _fmt = ('Unable to encode %(kind)s path %(path)r in '
@@ -2591,3 +2667,12 @@ class UnableEncodePath(BzrError):
         self.path = path
         self.kind = kind
         self.user_encoding = osutils.get_user_encoding()
+
+
+class CannotBindAddress(BzrError):
+
+    _fmt = 'Cannot bind address "%(host)s:%(port)i": %(orig_error)s.'
+
+    def __init__(self, host, port, orig_error):
+        BzrError.__init__(self, host=host, port=port,
+            orig_error=orig_error[1])
