@@ -1,15 +1,15 @@
 # Copyright (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -71,7 +71,7 @@ class IntSet(Exception):
             If specified, an initial collection of values.
         """
         self._val = bitmask
-        if values != None:
+        if values is not None:
             self.update(values)
 
 
@@ -149,7 +149,6 @@ class IntSet(Exception):
 
 
     def __contains__(self, i):
-        assert i >= 0
         return self._val & (1L << i)
 
 
@@ -177,12 +176,10 @@ class IntSet(Exception):
             self._val |= to_add._val
         else:
             for i in to_add:
-                assert i >= 0
                 self._val |= (1L << i)
 
 
     def add(self, to_add):
-        assert 0 <= to_add
         self._val |= (1L << to_add)
 
 
@@ -203,7 +200,6 @@ class IntSet(Exception):
         >>> not a
         True
         """
-        assert 0 <= to_remove
         m = 1L << to_remove
         if not self._val & m:
             raise KeyError(to_remove)
