@@ -170,9 +170,8 @@ class ImportReplacer(ScopeReplacer):
             from foo import bar, baz would get translated into 2 import
             requests. On for 'name=bar' and one for 'name=baz'
         """
-        if member is not None:
-            assert not children, \
-                'Cannot supply both a member and children'
+        if (member is not None) and children:
+            raise ValueError('Cannot supply both a member and children')
 
         object.__setattr__(self, '_import_replacer_children', children)
         object.__setattr__(self, '_member', member)
@@ -260,7 +259,8 @@ class ImportProcessor(object):
 
         :param import_str: The import string to process
         """
-        assert import_str.startswith('import ')
+        if not import_str.startswith('import '):
+            raise ValueError('bad import string %r' % (import_str,))
         import_str = import_str[len('import '):]
 
         for path in import_str.split(','):
@@ -305,7 +305,8 @@ class ImportProcessor(object):
 
         :param from_str: The import string to process
         """
-        assert from_str.startswith('from ')
+        if not from_str.startswith('from '):
+            raise ValueError('bad from/import %r' % from_str)
         from_str = from_str[len('from '):]
 
         from_module, import_list = from_str.split(' import ')
