@@ -57,7 +57,7 @@ class _OSLock(object):
             return self.f
         except IOError, e:
             if e.errno in (errno.EACCES, errno.EPERM):
-                raise errors.ReadOnlyLockError(self.filename, str(e))
+                raise errors.LockFailed(self.filename, str(e))
             if e.errno != errno.ENOENT:
                 raise
 
@@ -187,7 +187,9 @@ if have_fcntl:
 
             :return: A token which can be used to switch back to a read lock.
             """
-            assert self.filename not in _fcntl_WriteLock._open_locks
+            if self.filename in _fcntl_WriteLock._open_locks:
+                raise AssertionError('file already locked: %r'
+                    % (self.filename,))
             try:
                 wlock = _fcntl_TemporaryWriteLock(self)
             except errors.LockError:
@@ -213,7 +215,9 @@ if have_fcntl:
                 # write lock.
                 raise errors.LockContention(self.filename)
 
-            assert self.filename not in _fcntl_WriteLock._open_locks
+            if self.filename in _fcntl_WriteLock._open_locks:
+                raise AssertionError('file already locked: %r'
+                    % (self.filename,))
 
             # See if we can open the file for writing. Another process might
             # have a read lock. We don't use self._open() because we don't want
@@ -223,7 +227,7 @@ if have_fcntl:
                 new_f = open(self.filename, 'rb+')
             except IOError, e:
                 if e.errno in (errno.EACCES, errno.EPERM):
-                    raise errors.ReadOnlyLockError(self.filename, str(e))
+                    raise errors.LockFailed(self.filename, str(e))
                 raise
             try:
                 # LOCK_NB will cause IOError to be raised if we can't grab a
