@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 from cStringIO import StringIO
 import codecs
 #import traceback
+from warnings import warn
 
 import bzrlib
 from bzrlib.decorators import (needs_read_lock,
@@ -24,8 +25,10 @@ from bzrlib.decorators import (needs_read_lock,
 import bzrlib.errors as errors
 from bzrlib.errors import BzrError
 from bzrlib.osutils import file_iterator, safe_unicode
-from bzrlib.symbol_versioning import (deprecated_method, 
-        zero_eight)
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    deprecated_method,
+    )
 from bzrlib.trace import mutter, note
 import bzrlib.transactions as transactions
 import bzrlib.urlutils as urlutils
@@ -104,11 +107,9 @@ class LockableFiles(object):
 
     def __del__(self):
         if self.is_locked():
-            # XXX: This should show something every time, and be suitable for
-            # headless operation and embedding
-            from warnings import warn
-            warn("file group %r was not explicitly unlocked" % self)
-            self._lock.unlock()
+            # do not automatically unlock; there should have been a
+            # try/finally to unlock this.
+            warn("%r was gc'd while locked" % self)
 
     def break_lock(self):
         """Break the lock of this lockable files group if it is held.
@@ -132,7 +133,11 @@ class LockableFiles(object):
             self._dir_mode = 0755
             self._file_mode = 0644
         else:
-            self._dir_mode = st.st_mode & 07777
+            # Check the directory mode, but also make sure the created
+            # directories and files are read-write for this user. This is
+            # mostly a workaround for filesystems which lie about being able to
+            # write to a directory (cygwin & win32)
+            self._dir_mode = (st.st_mode & 07777) | 00700
             # Remove the sticky and execute bits for files
             self._file_mode = self._dir_mode & ~07111
         if not self._set_dir_mode:
@@ -144,34 +149,6 @@ class LockableFiles(object):
         """Return location relative to branch."""
         return self._transport.abspath(self._escape(file_or_path))
 
-    @deprecated_method(zero_eight)
-    def controlfile(self, file_or_path, mode='r'):
-        """Open a control file for this branch.
-
-        There are two classes of file in a lockable directory: text
-        and binary.  binary files are untranslated byte streams.  Text
-        control files are stored with Unix newlines and in UTF-8, even
-        if the platform or locale defaults are different.
-
-        Such files are not openable in write mode : they are managed via
-        put and put_utf8 which atomically replace old versions using
-        atomicfile.
-        """
-
-        relpath = self._escape(file_or_path)
-        # TODO: codecs.open() buffers linewise, so it was overloaded with
-        # a much larger buffer, do we need to do the same for getreader/getwriter?
-        if mode == 'rb': 
-            return self.get(relpath)
-        elif mode == 'wb':
-            raise BzrError("Branch.controlfile(mode='wb') is not supported, use put[_utf8]")
-        elif mode == 'r':
-            return self.get_utf8(relpath)
-        elif mode == 'w':
-            raise BzrError("Branch.controlfile(mode='w') is not supported, use put[_utf8]")
-        else:
-            raise BzrError("invalid controlfile mode %r" % mode)
-
     @needs_read_lock
     def get(self, relpath):
         """Get a file as a bytestream."""
@@ -179,6 +156,7 @@ class LockableFiles(object):
         return self._transport.get(relpath)
 
     @needs_read_lock
+    @deprecated_method(deprecated_in((1, 5, 0)))
     def get_utf8(self, relpath):
         """Get a file as a unicode stream."""
         relpath = self._escape(relpath)
@@ -267,8 +245,8 @@ class LockableFiles(object):
     def lock_read(self):
         # mutter("lock read: %s (%s)", self, self._lock_count)
         if self._lock_mode:
-            assert self._lock_mode in ('r', 'w'), \
-                   "invalid lock mode %r" % self._lock_mode
+            if self._lock_mode not in ('r', 'w'):
+                raise ValueError("invalid lock mode %r" % (self._lock_mode,))
             self._lock_count += 1
         else:
             self._lock.lock_read()
