@@ -63,7 +63,6 @@ from bzrlib import (
     config,
     lazy_regex,
     registry,
-    symbol_versioning,
     )
 from bzrlib.errors import (
     BzrCommandError,
@@ -73,16 +72,13 @@ from bzrlib.osutils import (
     get_terminal_encoding,
     terminal_width,
     )
+from bzrlib.repository import _strip_NULL_ghosts
 from bzrlib.revision import (
     NULL_REVISION,
     )
 from bzrlib.revisionspec import (
     RevisionInfo,
     )
-from bzrlib.symbol_versioning import (
-    deprecated_method,
-    zero_seventeen,
-    )
 from bzrlib.trace import mutter
 from bzrlib.tsort import (
     merge_sort,
@@ -193,6 +189,7 @@ def show_log(branch,
     finally:
         branch.unlock()
 
+
 def _show_log(branch,
              lf,
              specific_fileid=None,
@@ -208,46 +205,65 @@ def _show_log(branch,
 
     if specific_fileid:
         mutter('get log for file_id %r', specific_fileid)
-
+    generate_merge_revisions = getattr(lf, 'supports_merge_revisions', False)
+    allow_single_merge_revision = getattr(lf,
+        'supports_single_merge_revision', False)
+    view_revisions = calculate_view_revisions(branch, start_revision,
+                                              end_revision, direction,
+                                              specific_fileid,
+                                              generate_merge_revisions,
+                                              allow_single_merge_revision)
     if search is not None:
         searchRE = re.compile(search, re.IGNORECASE)
     else:
         searchRE = None
 
+    rev_tag_dict = {}
+    generate_tags = getattr(lf, 'supports_tags', False)
+    if generate_tags:
+        if branch.supports_tags():
+            rev_tag_dict = branch.tags.get_reverse_tag_dict()
+
+    generate_delta = verbose and getattr(lf, 'supports_delta', False)
+
+    # now we just print all the revisions
+    log_count = 0
+    for (rev_id, revno, merge_depth), rev, delta in _iter_revisions(
+        branch.repository, view_revisions, generate_delta):
+        if searchRE:
+            if not searchRE.search(rev.message):
+                continue
+
+        lr = LogRevision(rev, revno, merge_depth, delta,
+                         rev_tag_dict.get(rev_id))
+        lf.log_revision(lr)
+        if limit:
+            log_count += 1
+            if log_count >= limit:
+                break
+
+
+def calculate_view_revisions(branch, start_revision, end_revision, direction,
+                             specific_fileid, generate_merge_revisions,
+                             allow_single_merge_revision):
+    if (not generate_merge_revisions and start_revision is end_revision is
+        None and direction == 'reverse' and specific_fileid is None):
+        return _linear_view_revisions(branch)
+
     mainline_revs, rev_nos, start_rev_id, end_rev_id = \
         _get_mainline_revs(branch, start_revision, end_revision)
     if not mainline_revs:
-        return
+        return []
 
     if direction == 'reverse':
         start_rev_id, end_rev_id = end_rev_id, start_rev_id
-        
-    legacy_lf = getattr(lf, 'log_revision', None) is None
-    if legacy_lf:
-        # pre-0.17 formatters use show for mainline revisions.
-        # how should we show merged revisions ?
-        #   pre-0.11 api: show_merge
-        #   0.11-0.16 api: show_merge_revno
-        show_merge_revno = getattr(lf, 'show_merge_revno', None)
-        show_merge = getattr(lf, 'show_merge', None)
-        if show_merge is None and show_merge_revno is None:
-            # no merged-revno support
-            generate_merge_revisions = False
-        else:
-            generate_merge_revisions = True
-        # tell developers to update their code
-        symbol_versioning.warn('LogFormatters should provide log_revision '
-            'instead of show and show_merge_revno since bzr 0.17.',
-            DeprecationWarning, stacklevel=3)
-    else:
-        generate_merge_revisions = getattr(lf, 'supports_merge_revisions', 
-                                           False)
+
     generate_single_revision = False
     if ((not generate_merge_revisions)
         and ((start_rev_id and (start_rev_id not in rev_nos))
             or (end_rev_id and (end_rev_id not in rev_nos)))):
         generate_single_revision = ((start_rev_id == end_rev_id)
-            and getattr(lf, 'supports_single_merge_revision', False))
+            and allow_single_merge_revision)
         if not generate_single_revision:
             raise BzrCommandError('Selected log formatter only supports '
                 'mainline revisions.')
@@ -271,65 +287,35 @@ def _show_log(branch,
         min_depth = min([d for r,n,d in view_revisions])
         if min_depth != 0:
             view_revisions = [(r,n,d-min_depth) for r,n,d in view_revisions]
-        
-    rev_tag_dict = {}
-    generate_tags = getattr(lf, 'supports_tags', False)
-    if generate_tags:
-        if branch.supports_tags():
-            rev_tag_dict = branch.tags.get_reverse_tag_dict()
+    return view_revisions
 
-    generate_delta = verbose and getattr(lf, 'supports_delta', False)
 
-    def iter_revisions():
+def _linear_view_revisions(branch):
+    start_revno, start_revision_id = branch.last_revision_info()
+    repo = branch.repository
+    revision_ids = repo.iter_reverse_revision_history(start_revision_id)
+    for num, revision_id in enumerate(revision_ids):
+        yield revision_id, str(start_revno - num), 0
+
+
+def _iter_revisions(repository, view_revisions, generate_delta):
+    num = 9
+    view_revisions = iter(view_revisions)
+    while True:
+        cur_view_revisions = [d for x, d in zip(range(num), view_revisions)]
+        if len(cur_view_revisions) == 0:
+            break
+        cur_deltas = {}
         # r = revision, n = revno, d = merge depth
-        revision_ids = [r for r, n, d in view_revisions]
-        num = 9
-        repository = branch.repository
-        while revision_ids:
-            cur_deltas = {}
-            revisions = repository.get_revisions(revision_ids[:num])
-            if generate_delta:
-                deltas = repository.get_deltas_for_revisions(revisions)
-                cur_deltas = dict(izip((r.revision_id for r in revisions),
-                                       deltas))
-            for revision in revisions:
-                yield revision, cur_deltas.get(revision.revision_id)
-            revision_ids  = revision_ids[num:]
-            num = min(int(num * 1.5), 200)
-
-    # now we just print all the revisions
-    log_count = 0
-    for ((rev_id, revno, merge_depth), (rev, delta)) in \
-         izip(view_revisions, iter_revisions()):
-
-        if searchRE:
-            if not searchRE.search(rev.message):
-                continue
-
-        if not legacy_lf:
-            lr = LogRevision(rev, revno, merge_depth, delta,
-                             rev_tag_dict.get(rev_id))
-            lf.log_revision(lr)
-        else:
-            # support for legacy (pre-0.17) LogFormatters
-            if merge_depth == 0:
-                if generate_tags:
-                    lf.show(revno, rev, delta, rev_tag_dict.get(rev_id))
-                else:
-                    lf.show(revno, rev, delta)
-            else:
-                if show_merge_revno is None:
-                    lf.show_merge(rev, merge_depth)
-                else:
-                    if generate_tags:
-                        lf.show_merge_revno(rev, merge_depth, revno,
-                                            rev_tag_dict.get(rev_id))
-                    else:
-                        lf.show_merge_revno(rev, merge_depth, revno)
-        if limit:
-            log_count += 1
-            if log_count >= limit:
-                break
+        revision_ids = [r for (r, n, d) in cur_view_revisions]
+        revisions = repository.get_revisions(revision_ids)
+        if generate_delta:
+            deltas = repository.get_deltas_for_revisions(revisions)
+            cur_deltas = dict(izip((r.revision_id for r in revisions),
+                                   deltas))
+        for view_data, revision in izip(cur_view_revisions, revisions):
+            yield view_data, revision, cur_deltas.get(revision.revision_id)
+        num = min(int(num * 1.5), 200)
 
 
 def _get_mainline_revs(branch, start_revision, end_revision):
@@ -474,11 +460,18 @@ def _filter_revisions_touching_file_id(branch, file_id, mainline_revisions,
     weave_modifed_revisions = set(file_weave.versions())
     # build the ancestry of each revision in the graph
     # - only listing the ancestors that change the specific file.
-    rev_graph = branch.repository.get_revision_graph(mainline_revisions[-1])
-    sorted_rev_list = topo_sort(rev_graph)
+    graph = branch.repository.get_graph()
+    # This asks for all mainline revisions, which means we only have to spider
+    # sideways, rather than depth history. That said, its still size-of-history
+    # and should be addressed.
+    # mainline_revisions always includes an extra revision at the beginning, so
+    # don't request it.
+    parent_map = dict(((key, value) for key, value in
+        graph.iter_ancestry(mainline_revisions[1:]) if value is not None))
+    sorted_rev_list = topo_sort(parent_map.items())
     ancestry = {}
     for rev in sorted_rev_list:
-        parents = rev_graph[rev]
+        parents = parent_map[rev]
         if rev not in weave_modifed_revisions and len(parents) == 1:
             # We will not be adding anything new, so just use a reference to
             # the parent ancestry.
@@ -488,11 +481,16 @@ def _filter_revisions_touching_file_id(branch, file_id, mainline_revisions,
             if rev in weave_modifed_revisions:
                 rev_ancestry.add(rev)
             for parent in parents:
+                if parent not in ancestry:
+                    # parent is a Ghost, which won't be present in
+                    # sorted_rev_list, but we may access it later, so create an
+                    # empty node for it
+                    ancestry[parent] = set()
                 rev_ancestry = rev_ancestry.union(ancestry[parent])
         ancestry[rev] = rev_ancestry
 
     def is_merging_rev(r):
-        parents = rev_graph[r]
+        parents = parent_map[r]
         if len(parents) > 1:
             leftparent = parents[0]
             for rightparent in parents[1:]:
@@ -520,8 +518,16 @@ def get_view_revisions(mainline_revs, rev_nos, branch, direction,
         for revision_id in revision_ids:
             yield revision_id, str(rev_nos[revision_id]), 0
         return
+    graph = branch.repository.get_graph()
+    # This asks for all mainline revisions, which means we only have to spider
+    # sideways, rather than depth history. That said, its still size-of-history
+    # and should be addressed.
+    parent_map = dict(((key, value) for key, value in
+        graph.iter_ancestry(mainline_revs) if value is not None))
+    # filter out ghosts; merge_sort errors on ghosts.
+    rev_graph = _strip_NULL_ghosts(parent_map)
     merge_sorted_revisions = merge_sort(
-        branch.repository.get_revision_graph(mainline_revs[-1]),
+        rev_graph,
         mainline_revs[-1],
         mainline_revs,
         generate_revno=True)
@@ -548,7 +554,6 @@ def reverse_by_depth(merge_sorted_revisions, _depth=0):
         if val[2] == _depth:
             zd_revisions.append([val])
         else:
-            assert val[2] > _depth
             zd_revisions[-1].append(val)
     for revisions in zd_revisions:
         if len(revisions) > 1:
@@ -616,10 +621,6 @@ class LogFormatter(object):
 #        """
 #        raise NotImplementedError('not implemented in abstract base')
 
-    @deprecated_method(zero_seventeen)
-    def show(self, revno, rev, delta):
-        raise NotImplementedError('not implemented in abstract base')
-
     def short_committer(self, rev):
         name, address = config.parse_username(rev.committer)
         if name:
@@ -639,17 +640,6 @@ class LongLogFormatter(LogFormatter):
     supports_delta = True
     supports_tags = True
 
-    @deprecated_method(zero_seventeen)
-    def show(self, revno, rev, delta, tags=None):
-        lr = LogRevision(rev, revno, 0, delta, tags)
-        return self.log_revision(lr)
-
-    @deprecated_method(zero_seventeen)
-    def show_merge_revno(self, rev, merge_depth, revno, tags=None):
-        """Show a merged revision rev, with merge_depth and a revno."""
-        lr = LogRevision(rev, revno, merge_depth, tags=tags)
-        return self.log_revision(lr)
-
     def log_revision(self, revision):
         """Log a revision, either merged or not."""
         indent = '    ' * revision.merge_depth
@@ -695,11 +685,6 @@ class ShortLogFormatter(LogFormatter):
     supports_delta = True
     supports_single_merge_revision = True
 
-    @deprecated_method(zero_seventeen)
-    def show(self, revno, rev, delta):
-        lr = LogRevision(rev, revno, 0, delta)
-        return self.log_revision(lr)
-
     def log_revision(self, revision):
         to_file = self.to_file
         date_str = format_date(revision.rev.timestamp,
@@ -755,11 +740,6 @@ class LineLogFormatter(LogFormatter):
         else:
             return rev.message
 
-    @deprecated_method(zero_seventeen)
-    def show(self, revno, rev, delta):
-        self.to_file.write(self.log_string(revno, rev, terminal_width()-1))
-        self.to_file.write('\n')
-
     def log_revision(self, revision):
         self.to_file.write(self.log_string(revision.revno, revision.rev,
                                               self._max_chars))
