@@ -20,6 +20,7 @@ import errno
 import warnings
 
 from bzrlib import (
+    debug,
     errors,
     osutils,
     patiencediff,
@@ -44,10 +45,11 @@ from bzrlib.errors import (BzrCommandError,
 from bzrlib.merge3 import Merge3
 from bzrlib.osutils import rename, pathjoin
 from progress import DummyProgress, ProgressPhase
-from bzrlib.revision import (is_ancestor, NULL_REVISION, ensure_null)
+from bzrlib.revision import (NULL_REVISION, ensure_null)
 from bzrlib.textfile import check_text_lines
-from bzrlib.trace import mutter, warning, note
-from bzrlib.transform import (TreeTransform, resolve_conflicts, cook_conflicts,
+from bzrlib.trace import mutter, warning, note, is_quiet
+from bzrlib.transform import (TransformPreview, TreeTransform,
+                              resolve_conflicts, cook_conflicts,
                               conflict_pass, FinalPaths, create_by_entry,
                               unique_add, ROOT_PARENT)
 from bzrlib.versionedfile import PlanWeaveMerge
@@ -64,9 +66,8 @@ def transform_tree(from_tree, to_tree, interesting_ids=None):
 class Merger(object):
     def __init__(self, this_branch, other_tree=None, base_tree=None,
                  this_tree=None, pb=DummyProgress(), change_reporter=None,
-                 recurse='down'):
+                 recurse='down', revision_graph=None):
         object.__init__(self)
-        assert this_tree is not None, "this_tree is required"
         self.this_branch = this_branch
         self.this_basis = _mod_revision.ensure_null(
             this_branch.last_revision())
@@ -88,6 +89,40 @@ class Merger(object):
         self.recurse = recurse
         self.change_reporter = change_reporter
         self._cached_trees = {}
+        self._revision_graph = revision_graph
+        self._base_is_ancestor = None
+        self._base_is_other_ancestor = None
+
+    @property
+    def revision_graph(self):
+        if self._revision_graph is None:
+            self._revision_graph = self.this_branch.repository.get_graph()
+        return self._revision_graph
+
+    def _set_base_is_ancestor(self, value):
+        self._base_is_ancestor = value
+
+    def _get_base_is_ancestor(self):
+        if self._base_is_ancestor is None:
+            self._base_is_ancestor = self.revision_graph.is_ancestor(
+                self.base_rev_id, self.this_basis)
+        return self._base_is_ancestor
+
+    base_is_ancestor = property(_get_base_is_ancestor, _set_base_is_ancestor)
+
+    def _set_base_is_other_ancestor(self, value):
+        self._base_is_other_ancestor = value
+
+    def _get_base_is_other_ancestor(self):
+        if self._base_is_other_ancestor is None:
+            if self.other_basis is None:
+                return True
+            self._base_is_other_ancestor = self.revision_graph.is_ancestor(
+                self.base_rev_id, self.other_basis)
+        return self._base_is_other_ancestor
+
+    base_is_other_ancestor = property(_get_base_is_other_ancestor,
+                                      _set_base_is_other_ancestor)
 
     @staticmethod
     def from_uncommitted(tree, other_tree, pb):
@@ -101,6 +136,7 @@ class Merger(object):
                         pb)
         merger.base_rev_id = merger.base_tree.get_revision_id()
         merger.other_rev_id = None
+        merger.other_basis = merger.base_rev_id
         return merger
 
     @classmethod
@@ -114,17 +150,21 @@ class Merger(object):
         mergeable.install_revisions(tree.branch.repository)
         base_revision_id, other_revision_id, verified =\
             mergeable.get_merge_request(tree.branch.repository)
+        revision_graph = tree.branch.repository.get_graph()
         if (base_revision_id != _mod_revision.NULL_REVISION and
-            tree.branch.repository.get_graph().is_ancestor(
+            revision_graph.is_ancestor(
             base_revision_id, tree.branch.last_revision())):
             base_revision_id = None
+        else:
+            warning('Performing cherrypick')
         merger = klass.from_revision_ids(pb, tree, other_revision_id,
-                                         base_revision_id)
+                                         base_revision_id, revision_graph=
+                                         revision_graph)
         return merger, verified
 
     @staticmethod
-    def from_revision_ids(pb, this, other, base=None, other_branch=None,
-                          base_branch=None):
+    def from_revision_ids(pb, tree, other, base=None, other_branch=None,
+                          base_branch=None, revision_graph=None):
         """Return a Merger for revision-ids.
 
         :param tree: The tree to merge changes into
@@ -132,14 +172,17 @@ class Merger(object):
         :param base: The revision-id to use as BASE.  If not specified, will
             be auto-selected.
         :param other_branch: A branch containing the other revision-id.  If
-            not supplied, this.branch is used.
+            not supplied, tree.branch is used.
         :param base_branch: A branch containing the base revision-id.  If
-            not supplied, other_branch or this.branch will be used.
+            not supplied, other_branch or tree.branch will be used.
+        :param revision_graph: If you have a revision_graph precomputed, pass
+            it in, otherwise it will be created for you.
         :param pb: A progress indicator
         """
-        merger = Merger(this.branch, this_tree=this, pb=pb)
+        merger = Merger(tree.branch, this_tree=tree, pb=pb,
+                        revision_graph=revision_graph)
         if other_branch is None:
-            other_branch = this.branch
+            other_branch = tree.branch
         merger.set_other_revision(other, other_branch)
         if base is None:
             merger.find_base()
@@ -192,7 +235,6 @@ class Merger(object):
         self.ensure_revision_trees()
         def get_id(tree, file_id):
             revision_id = tree.inventory[file_id].revision
-            assert revision_id is not None
             return revision_id
         if self.this_rev_id is None:
             if self.this_basis_tree.get_file_sha1(file_id) != \
@@ -209,7 +251,7 @@ class Merger(object):
         if check_clean:
             self.compare_basis()
             if self.this_basis != self.this_rev_id:
-                raise BzrCommandError("Working tree has uncommitted changes.")
+                raise errors.UncommittedChanges(self.this_tree)
 
     def compare_basis(self):
         try:
@@ -296,29 +338,24 @@ class Merger(object):
         self.base_branch = branch
         self._maybe_fetch(branch, self.this_branch, revision_id)
         self.base_tree = self.revision_tree(revision_id)
-        self.base_is_ancestor = is_ancestor(self.this_basis,
-                                            self.base_rev_id,
-                                            self.this_branch)
-        self.base_is_other_ancestor = is_ancestor(self.other_basis,
-                                                  self.base_rev_id,
-                                                  self.this_branch)
 
     def _maybe_fetch(self, source, target, revision_id):
-        if (source.repository.bzrdir.root_transport.base !=
-            target.repository.bzrdir.root_transport.base):
+        if not source.repository.has_same_location(target.repository):
             target.fetch(source, revision_id)
 
     def find_base(self):
-        this_repo = self.this_branch.repository
-        graph = this_repo.get_graph()
         revisions = [ensure_null(self.this_basis),
                      ensure_null(self.other_basis)]
         if NULL_REVISION in revisions:
             self.base_rev_id = NULL_REVISION
         else:
-            self.base_rev_id = graph.find_unique_lca(*revisions)
+            self.base_rev_id, steps = self.revision_graph.find_unique_lca(
+                revisions[0], revisions[1], count_steps=True)
             if self.base_rev_id == NULL_REVISION:
                 raise UnrelatedBranches()
+            if steps > 1:
+                warning('Warning: criss-cross merge encountered.  See bzr'
+                        ' help criss-cross.')
         self.base_tree = self.revision_tree(self.base_rev_id)
         self.base_is_ancestor = True
         self.base_is_other_ancestor = True
@@ -341,19 +378,14 @@ class Merger(object):
                 self.base_rev_id = _mod_revision.ensure_null(
                     base_branch.get_rev_id(base_revision[1]))
             self._maybe_fetch(base_branch, self.this_branch, self.base_rev_id)
-            self.base_is_ancestor = is_ancestor(self.this_basis, 
-                                                self.base_rev_id,
-                                                self.this_branch)
-            self.base_is_other_ancestor = is_ancestor(self.other_basis,
-                                                      self.base_rev_id,
-                                                      self.this_branch)
 
-    def do_merge(self):
+    def make_merger(self):
         kwargs = {'working_tree':self.this_tree, 'this_tree': self.this_tree,
                   'other_tree': self.other_tree,
                   'interesting_ids': self.interesting_ids,
                   'interesting_files': self.interesting_files,
-                  'pp': self.pp}
+                  'pp': self.pp,
+                  'do_merge': False}
         if self.merge_type.requires_base:
             kwargs['base_tree'] = self.base_tree
         if self.merge_type.supports_reprocess:
@@ -365,16 +397,26 @@ class Merger(object):
             kwargs['show_base'] = self.show_base
         elif self.show_base:
             raise BzrError("Showing base is not supported for this"
-                                  " merge type. %s" % self.merge_type)
+                           " merge type. %s" % self.merge_type)
+        if (not getattr(self.merge_type, 'supports_reverse_cherrypick', True)
+            and not self.base_is_other_ancestor):
+            raise errors.CannotReverseCherrypick()
+        if self.merge_type.supports_cherrypick:
+            kwargs['cherrypick'] = (not self.base_is_ancestor or
+                                    not self.base_is_other_ancestor)
+        return self.merge_type(pb=self._pb,
+                               change_reporter=self.change_reporter,
+                               **kwargs)
+
+    def do_merge(self):
         self.this_tree.lock_tree_write()
         if self.base_tree is not None:
             self.base_tree.lock_read()
         if self.other_tree is not None:
             self.other_tree.lock_read()
         try:
-            merge = self.merge_type(pb=self._pb,
-                                    change_reporter=self.change_reporter,
-                                    **kwargs)
+            merge = self.make_merger()
+            merge.do_merge()
             if self.recurse == 'down':
                 for path, file_id in self.this_tree.iter_references():
                     sub_tree = self.this_tree.get_nested_tree(file_id, path)
@@ -390,6 +432,7 @@ class Merger(object):
                     base_revision = self.base_tree.get_reference_revision(file_id)
                     sub_merge.base_tree = \
                         sub_tree.branch.repository.revision_tree(base_revision)
+                    sub_merge.base_rev_id = base_revision
                     sub_merge.do_merge()
 
         finally:
@@ -399,7 +442,7 @@ class Merger(object):
                 self.base_tree.unlock()
             self.this_tree.unlock()
         if len(merge.cooked_conflicts) == 0:
-            if not self.ignore_zero:
+            if not self.ignore_zero and not is_quiet():
                 note("All changes applied successfully.")
         else:
             note("%d conflicts encountered." % len(merge.cooked_conflicts))
@@ -413,12 +456,15 @@ class Merge3Merger(object):
     supports_reprocess = True
     supports_show_base = True
     history_based = False
+    supports_cherrypick = True
+    supports_reverse_cherrypick = True
     winner_idx = {"this": 2, "other": 1, "conflict": 1}
 
     def __init__(self, working_tree, this_tree, base_tree, other_tree, 
                  interesting_ids=None, reprocess=False, show_base=False,
                  pb=DummyProgress(), pp=None, change_reporter=None,
-                 interesting_files=None):
+                 interesting_files=None, do_merge=True,
+                 cherrypick=False):
         """Initialize the merger object and perform the merge.
 
         :param working_tree: The working tree to apply the merge to
@@ -442,16 +488,14 @@ class Merge3Merger(object):
             merge.
         """
         object.__init__(self)
-        if interesting_files is not None:
-            assert interesting_ids is None
+        if interesting_files is not None and interesting_ids is not None:
+            raise ValueError(
+                'specify either interesting_ids or interesting_files')
         self.interesting_ids = interesting_ids
         self.interesting_files = interesting_files
         self.this_tree = working_tree
-        self.this_tree.lock_tree_write()
         self.base_tree = base_tree
-        self.base_tree.lock_read()
         self.other_tree = other_tree
-        self.other_tree.lock_read()
         self._raw_conflicts = []
         self.cooked_conflicts = []
         self.reprocess = reprocess
@@ -459,46 +503,25 @@ class Merge3Merger(object):
         self.pb = pb
         self.pp = pp
         self.change_reporter = change_reporter
+        self.cherrypick = cherrypick
         if self.pp is None:
             self.pp = ProgressPhase("Merge phase", 3, self.pb)
+        if do_merge:
+            self.do_merge()
 
-        self.tt = TreeTransform(working_tree, self.pb)
+    def do_merge(self):
+        self.this_tree.lock_tree_write()
+        self.base_tree.lock_read()
+        self.other_tree.lock_read()
+        self.tt = TreeTransform(self.this_tree, self.pb)
         try:
             self.pp.next_phase()
-            entries = self._entries3()
-            child_pb = ui.ui_factory.nested_progress_bar()
-            try:
-                for num, (file_id, changed, parents3, names3,
-                          executable3) in enumerate(entries):
-                    child_pb.update('Preparing file merge', num, len(entries))
-                    self._merge_names(file_id, parents3, names3)
-                    if changed:
-                        file_status = self.merge_contents(file_id)
-                    else:
-                        file_status = 'unmodified'
-                    self._merge_executable(file_id,
-                        executable3, file_status)
-            finally:
-                child_pb.finished()
-            self.fix_root()
-            self.pp.next_phase()
-            child_pb = ui.ui_factory.nested_progress_bar()
-            try:
-                fs_conflicts = resolve_conflicts(self.tt, child_pb,
-                    lambda t, c: conflict_pass(t, c, self.other_tree))
-            finally:
-                child_pb.finished()
-            if change_reporter is not None:
-                from bzrlib import delta
-                delta.report_changes(self.tt._iter_changes(), change_reporter)
-            self.cook_conflicts(fs_conflicts)
-            for conflict in self.cooked_conflicts:
-                warning(conflict)
+            self._compute_transform()
             self.pp.next_phase()
             results = self.tt.apply(no_conflicts=True)
             self.write_modified(results)
             try:
-                working_tree.add_conflicts(self.cooked_conflicts)
+                self.this_tree.add_conflicts(self.cooked_conflicts)
             except UnsupportedOperation:
                 pass
         finally:
@@ -508,6 +531,52 @@ class Merge3Merger(object):
             self.this_tree.unlock()
             self.pb.clear()
 
+    def make_preview_transform(self):
+        self.base_tree.lock_read()
+        self.other_tree.lock_read()
+        self.tt = TransformPreview(self.this_tree)
+        try:
+            self.pp.next_phase()
+            self._compute_transform()
+            self.pp.next_phase()
+        finally:
+            self.other_tree.unlock()
+            self.base_tree.unlock()
+            self.pb.clear()
+        return self.tt
+
+    def _compute_transform(self):
+        entries = self._entries3()
+        child_pb = ui.ui_factory.nested_progress_bar()
+        try:
+            for num, (file_id, changed, parents3, names3,
+                      executable3) in enumerate(entries):
+                child_pb.update('Preparing file merge', num, len(entries))
+                self._merge_names(file_id, parents3, names3)
+                if changed:
+                    file_status = self.merge_contents(file_id)
+                else:
+                    file_status = 'unmodified'
+                self._merge_executable(file_id,
+                    executable3, file_status)
+        finally:
+            child_pb.finished()
+        self.fix_root()
+        self.pp.next_phase()
+        child_pb = ui.ui_factory.nested_progress_bar()
+        try:
+            fs_conflicts = resolve_conflicts(self.tt, child_pb,
+                lambda t, c: conflict_pass(t, c, self.other_tree))
+        finally:
+            child_pb.finished()
+        if self.change_reporter is not None:
+            from bzrlib import delta
+            delta.report_changes(
+                self.tt.iter_changes(), self.change_reporter)
+        self.cook_conflicts(fs_conflicts)
+        for conflict in self.cooked_conflicts:
+            warning(conflict)
+
     def _entries3(self):
         """Gather data about files modified between three trees.
 
@@ -518,7 +587,7 @@ class Merge3Merger(object):
         executable3 is a tuple of execute-bit values for base, other and this.
         """
         result = []
-        iterator = self.other_tree._iter_changes(self.base_tree,
+        iterator = self.other_tree.iter_changes(self.base_tree,
                 include_unchanged=True, specific_files=self.interesting_files,
                 extra_trees=[self.this_tree])
         for (file_id, paths, changed, versioned, parents, names, kind,
@@ -551,7 +620,7 @@ class Merge3Merger(object):
                                  self.tt.root)
         if self.other_tree.inventory.root is None:
             return
-        other_root_file_id = self.other_tree.inventory.root.file_id
+        other_root_file_id = self.other_tree.get_root_id()
         other_root = self.tt.trans_id_file_id(other_root_file_id)
         if other_root == self.tt.root:
             return
@@ -642,14 +711,13 @@ class Merge3Merger(object):
         if key_base == key_other:
             return "this"
         key_this = key(this_tree, file_id)
-        if key_this not in (key_base, key_other):
-            return "conflict"
         # "Ambiguous clean merge"
-        elif key_this == key_other:
+        if key_this == key_other:
             return "this"
-        else:
-            assert key_this == key_base
+        elif key_this == key_base:
             return "other"
+        else:
+            return "conflict"
 
     def merge_names(self, file_id):
         def get_entry(tree):
@@ -803,7 +871,8 @@ class Merge3Merger(object):
             base_lines = []
         other_lines = self.get_lines(self.other_tree, file_id)
         this_lines = self.get_lines(self.this_tree, file_id)
-        m3 = Merge3(base_lines, this_lines, other_lines)
+        m3 = Merge3(base_lines, this_lines, other_lines,
+                    is_cherrypick=self.cherrypick)
         start_marker = "!START OF MERGE CONFLICT!" + "I HOPE THIS IS UNIQUE"
         if self.show_base is True:
             base_marker = '|' * 7
@@ -879,12 +948,6 @@ class Merge3Merger(object):
         base_executable, other_executable, this_executable = executable
         if file_status == "deleted":
             return
-        trans_id = self.tt.trans_id_file_id(file_id)
-        try:
-            if self.tt.final_kind(trans_id) != "file":
-                return
-        except NoSuchFile:
-            return
         winner = self._three_way(*executable)
         if winner == "conflict":
         # There must be a None in here, if we have a conflict, but we
@@ -893,23 +956,26 @@ class Merge3Merger(object):
                 winner = "this"
             else:
                 winner = "other"
+        if winner == 'this' and file_status != "modified":
+            return
+        trans_id = self.tt.trans_id_file_id(file_id)
+        try:
+            if self.tt.final_kind(trans_id) != "file":
+                return
+        except NoSuchFile:
+            return
         if winner == "this":
-            if file_status == "modified":
-                executability = this_executable
-                if executability is not None:
-                    trans_id = self.tt.trans_id_file_id(file_id)
-                    self.tt.set_executability(executability, trans_id)
+            executability = this_executable
         else:
-            assert winner == "other"
             if file_id in self.other_tree:
                 executability = other_executable
             elif file_id in self.this_tree:
                 executability = this_executable
             elif file_id in self.base_tree:
                 executability = base_executable
-            if executability is not None:
-                trans_id = self.tt.trans_id_file_id(file_id)
-                self.tt.set_executability(executability, trans_id)
+        if executability is not None:
+            trans_id = self.tt.trans_id_file_id(file_id)
+            self.tt.set_executability(executability, trans_id)
 
     def cook_conflicts(self, fs_conflicts):
         """Convert all conflicts into a form that doesn't depend on trans_id"""
@@ -948,13 +1014,15 @@ class Merge3Merger(object):
         for trans_id, conflicts in name_conflicts.iteritems():
             try:
                 this_parent, other_parent = conflicts['parent conflict']
-                assert this_parent != other_parent
+                if this_parent == other_parent:
+                    raise AssertionError()
             except KeyError:
                 this_parent = other_parent = \
                     self.tt.final_file_id(self.tt.final_parent(trans_id))
             try:
                 this_name, other_name = conflicts['name conflict']
-                assert this_name != other_name
+                if this_name == other_name:
+                    raise AssertionError()
             except KeyError:
                 this_name = other_name = self.tt.final_name(trans_id)
             other_path = fp.get_path(trans_id)
@@ -975,23 +1043,26 @@ class WeaveMerger(Merge3Merger):
     """Three-way tree merger, text weave merger."""
     supports_reprocess = True
     supports_show_base = False
-
-    def __init__(self, working_tree, this_tree, base_tree, other_tree, 
-                 interesting_ids=None, pb=DummyProgress(), pp=None,
-                 reprocess=False, change_reporter=None,
-                 interesting_files=None):
-        super(WeaveMerger, self).__init__(working_tree, this_tree, 
-                                          base_tree, other_tree, 
-                                          interesting_ids=interesting_ids, 
-                                          pb=pb, pp=pp, reprocess=reprocess,
-                                          change_reporter=change_reporter)
+    supports_reverse_cherrypick = False
+    history_based = True
 
     def _merged_lines(self, file_id):
         """Generate the merged lines.
         There is no distinction between lines that are meant to contain <<<<<<<
         and conflicts.
         """
-        plan = self.this_tree.plan_file_merge(file_id, self.other_tree)
+        if self.cherrypick:
+            base = self.base_tree
+        else:
+            base = None
+        plan = self.this_tree.plan_file_merge(file_id, self.other_tree,
+                                              base=base)
+        if 'merge' in debug.debug_flags:
+            plan = list(plan)
+            trans_id = self.tt.trans_id_file_id(file_id)
+            name = self.tt.final_name(trans_id) + '.plan'
+            contents = ('%10s|%s' % l for l in plan)
+            self.tt.new_file(name, self.tt.final_parent(trans_id), contents)
         textmerge = PlanWeaveMerge(plan, '<<<<<<< TREE\n',
             '>>>>>>> MERGE-SOURCE\n')
         return textmerge.merge_lines(self.reprocess)
@@ -1016,6 +1087,30 @@ class WeaveMerger(Merge3Merger):
             file_group.append(trans_id)
 
 
+class LCAMerger(WeaveMerger):
+
+    def _merged_lines(self, file_id):
+        """Generate the merged lines.
+        There is no distinction between lines that are meant to contain <<<<<<<
+        and conflicts.
+        """
+        if self.cherrypick:
+            base = self.base_tree
+        else:
+            base = None
+        plan = self.this_tree.plan_file_lca_merge(file_id, self.other_tree,
+                                                  base=base)
+        if 'merge' in debug.debug_flags:
+            plan = list(plan)
+            trans_id = self.tt.trans_id_file_id(file_id)
+            name = self.tt.final_name(trans_id) + '.plan'
+            contents = ('%10s|%s' % l for l in plan)
+            self.tt.new_file(name, self.tt.final_parent(trans_id), contents)
+        textmerge = PlanWeaveMerge(plan, '<<<<<<< TREE\n',
+            '>>>>>>> MERGE-SOURCE\n')
+        return textmerge.merge_lines(self.reprocess)
+
+
 class Diff3Merger(Merge3Merger):
     """Three-way merger using external diff3 for text merging"""
 
@@ -1086,13 +1181,18 @@ def merge_inner(this_branch, other_tree, base_tree, ignore_zero=False,
     merger.interesting_ids = interesting_ids
     merger.ignore_zero = ignore_zero
     if interesting_files:
-        assert not interesting_ids, ('Only supply interesting_ids'
-                                     ' or interesting_files')
+        if interesting_ids:
+            raise ValueError('Only supply interesting_ids'
+                             ' or interesting_files')
         merger.interesting_files = interesting_files
     merger.show_base = show_base
     merger.reprocess = reprocess
     merger.other_rev_id = other_rev_id
     merger.other_basis = other_rev_id
+    get_revision_id = getattr(base_tree, 'get_revision_id', None)
+    if get_revision_id is None:
+        get_revision_id = base_tree.last_revision
+    merger.set_base_revision(get_revision_id(), this_branch)
     return merger.do_merge()
 
 def get_merge_type_registry():
@@ -1131,10 +1231,244 @@ def _plan_annotate_merge(annotated_a, annotated_b, ancestors_a, ancestors_b):
             yield status_a(revision, text)
         for revision, text in annotated_b[b_cur:bi]:
             yield status_b(revision, text)
-
         # and now the matched section
         a_cur = ai + l
         b_cur = bi + l
-        for text_a, text_b in zip(plain_a[ai:a_cur], plain_b[bi:b_cur]):
-            assert text_a == text_b
+        for text_a in plain_a[ai:a_cur]:
             yield "unchanged", text_a
+
+
+class _PlanMergeBase(object):
+
+    def __init__(self, a_rev, b_rev, vf):
+        """Contructor.
+
+        :param a_rev: Revision-id of one revision to merge
+        :param b_rev: Revision-id of the other revision to merge
+        :param vf: A versionedfile containing both revisions
+        """
+        self.a_rev = a_rev
+        self.b_rev = b_rev
+        self.lines_a = vf.get_lines(a_rev)
+        self.lines_b = vf.get_lines(b_rev)
+        self.vf = vf
+        self._last_lines = None
+        self._last_lines_revision_id = None
+        self._cached_matching_blocks = {}
+
+    def plan_merge(self):
+        """Generate a 'plan' for merging the two revisions.
+
+        This involves comparing their texts and determining the cause of
+        differences.  If text A has a line and text B does not, then either the
+        line was added to text A, or it was deleted from B.  Once the causes
+        are combined, they are written out in the format described in
+        VersionedFile.plan_merge
+        """
+        blocks = self._get_matching_blocks(self.a_rev, self.b_rev)
+        unique_a, unique_b = self._unique_lines(blocks)
+        new_a, killed_b = self._determine_status(self.a_rev, unique_a)
+        new_b, killed_a = self._determine_status(self.b_rev, unique_b)
+        return self._iter_plan(blocks, new_a, killed_b, new_b, killed_a)
+
+    def _iter_plan(self, blocks, new_a, killed_b, new_b, killed_a):
+        last_i = 0
+        last_j = 0
+        for i, j, n in blocks:
+            for a_index in range(last_i, i):
+                if a_index in new_a:
+                    if a_index in killed_b:
+                        yield 'conflicted-a', self.lines_a[a_index]
+                    else:
+                        yield 'new-a', self.lines_a[a_index]
+                else:
+                    yield 'killed-b', self.lines_a[a_index]
+            for b_index in range(last_j, j):
+                if b_index in new_b:
+                    if b_index in killed_a:
+                        yield 'conflicted-b', self.lines_b[b_index]
+                    else:
+                        yield 'new-b', self.lines_b[b_index]
+                else:
+                    yield 'killed-a', self.lines_b[b_index]
+            # handle common lines
+            for a_index in range(i, i+n):
+                yield 'unchanged', self.lines_a[a_index]
+            last_i = i+n
+            last_j = j+n
+
+    def _get_matching_blocks(self, left_revision, right_revision):
+        """Return a description of which sections of two revisions match.
+
+        See SequenceMatcher.get_matching_blocks
+        """
+        cached = self._cached_matching_blocks.get((left_revision,
+                                                   right_revision))
+        if cached is not None:
+            return cached
+        if self._last_lines_revision_id == left_revision:
+            left_lines = self._last_lines
+        else:
+            left_lines = self.vf.get_lines(left_revision)
+        right_lines = self.vf.get_lines(right_revision)
+        self._last_lines = right_lines
+        self._last_lines_revision_id = right_revision
+        matcher = patiencediff.PatienceSequenceMatcher(None, left_lines,
+                                                       right_lines)
+        return matcher.get_matching_blocks()
+
+    def _unique_lines(self, matching_blocks):
+        """Analyse matching_blocks to determine which lines are unique
+
+        :return: a tuple of (unique_left, unique_right), where the values are
+            sets of line numbers of unique lines.
+        """
+        last_i = 0
+        last_j = 0
+        unique_left = []
+        unique_right = []
+        for i, j, n in matching_blocks:
+            unique_left.extend(range(last_i, i))
+            unique_right.extend(range(last_j, j))
+            last_i = i + n
+            last_j = j + n
+        return unique_left, unique_right
+
+    @staticmethod
+    def _subtract_plans(old_plan, new_plan):
+        """Remove changes from new_plan that came from old_plan.
+
+        It is assumed that the difference between the old_plan and new_plan
+        is their choice of 'b' text.
+
+        All lines from new_plan that differ from old_plan are emitted
+        verbatim.  All lines from new_plan that match old_plan but are
+        not about the 'b' revision are emitted verbatim.
+
+        Lines that match and are about the 'b' revision are the lines we
+        don't want, so we convert 'killed-b' -> 'unchanged', and 'new-b'
+        is skipped entirely.
+        """
+        matcher = patiencediff.PatienceSequenceMatcher(None, old_plan,
+                                                       new_plan)
+        last_j = 0
+        for i, j, n in matcher.get_matching_blocks():
+            for jj in range(last_j, j):
+                yield new_plan[jj]
+            for jj in range(j, j+n):
+                plan_line = new_plan[jj]
+                if plan_line[0] == 'new-b':
+                    pass
+                elif plan_line[0] == 'killed-b':
+                    yield 'unchanged', plan_line[1]
+                else:
+                    yield plan_line
+            last_j = j + n
+
+
+class _PlanMerge(_PlanMergeBase):
+    """Plan an annotate merge using on-the-fly annotation"""
+
+    def __init__(self, a_rev, b_rev, vf):
+       _PlanMergeBase.__init__(self, a_rev, b_rev, vf)
+       a_ancestry = set(vf.get_ancestry(a_rev, topo_sorted=False))
+       b_ancestry = set(vf.get_ancestry(b_rev, topo_sorted=False))
+       self.uncommon = a_ancestry.symmetric_difference(b_ancestry)
+
+    def _determine_status(self, revision_id, unique_line_numbers):
+        """Determines the status unique lines versus all lcas.
+
+        Basically, determines why the line is unique to this revision.
+
+        A line may be determined new or killed, but not both.
+
+        :param revision_id: The id of the revision in which the lines are
+            unique
+        :param unique_line_numbers: The line numbers of unique lines.
+        :return a tuple of (new_this, killed_other):
+        """
+        new = self._find_new(revision_id)
+        killed = set(unique_line_numbers).difference(new)
+        return new, killed
+
+    def _find_new(self, version_id):
+        """Determine which lines are new in the ancestry of this version.
+
+        If a lines is present in this version, and not present in any
+        common ancestor, it is considered new.
+        """
+        if version_id not in self.uncommon:
+            return set()
+        parents = self.vf.get_parent_map([version_id])[version_id]
+        if len(parents) == 0:
+            return set(range(len(self.vf.get_lines(version_id))))
+        new = None
+        for parent in parents:
+            blocks = self._get_matching_blocks(version_id, parent)
+            result, unused = self._unique_lines(blocks)
+            parent_new = self._find_new(parent)
+            for i, j, n in blocks:
+                for ii, jj in [(i+r, j+r) for r in range(n)]:
+                    if jj in parent_new:
+                        result.append(ii)
+            if new is None:
+                new = set(result)
+            else:
+                new.intersection_update(result)
+        return new
+
+
+class _PlanLCAMerge(_PlanMergeBase):
+    """
+    This merge algorithm differs from _PlanMerge in that:
+    1. comparisons are done against LCAs only
+    2. cases where a contested line is new versus one LCA but old versus
+       another are marked as conflicts, by emitting the line as conflicted-a
+       or conflicted-b.
+
+    This is faster, and hopefully produces more useful output.
+    """
+
+    def __init__(self, a_rev, b_rev, vf, graph):
+        _PlanMergeBase.__init__(self, a_rev, b_rev, vf)
+        self.lcas = graph.find_lca(a_rev, b_rev)
+        for lca in self.lcas:
+            lca_lines = self.vf.get_lines(lca)
+            matcher = patiencediff.PatienceSequenceMatcher(None, self.lines_a,
+                                                           lca_lines)
+            blocks = list(matcher.get_matching_blocks())
+            self._cached_matching_blocks[(a_rev, lca)] = blocks
+            matcher = patiencediff.PatienceSequenceMatcher(None, self.lines_b,
+                                                           lca_lines)
+            blocks = list(matcher.get_matching_blocks())
+            self._cached_matching_blocks[(b_rev, lca)] = blocks
+
+    def _determine_status(self, revision_id, unique_line_numbers):
+        """Determines the status unique lines versus all lcas.
+
+        Basically, determines why the line is unique to this revision.
+
+        A line may be determined new, killed, or both.
+
+        If a line is determined new, that means it was not present in at least
+        one LCA, and is not present in the other merge revision.
+
+        If a line is determined killed, that means the line was present in
+        at least one LCA.
+
+        If a line is killed and new, this indicates that the two merge
+        revisions contain differing conflict resolutions.
+        :param revision_id: The id of the revision in which the lines are
+            unique
+        :param unique_line_numbers: The line numbers of unique lines.
+        :return a tuple of (new_this, killed_other):
+        """
+        new = set()
+        killed = set()
+        unique_line_numbers = set(unique_line_numbers)
+        for lca in self.lcas:
+            blocks = self._get_matching_blocks(revision_id, lca)
+            unique_vs_lca, _ignored = self._unique_lines(blocks)
+            new.update(unique_line_numbers.intersection(unique_vs_lca))
+            killed.update(unique_line_numbers.difference(unique_vs_lca))
+        return new, killed
