@@ -25,7 +25,8 @@ import sys
 
 import bzrlib
 import bzrlib.config as config
-from bzrlib.errors import BzrError
+from bzrlib import osutils
+from bzrlib.errors import BzrError, BadCommitMessageEncoding
 from bzrlib.trace import warning, mutter
 
 
@@ -60,10 +61,10 @@ def _run_editor(filename):
             ## mutter("trying editor: %r", (edargs +[filename]))
             x = call(edargs + [filename])
         except OSError, e:
-           # We're searching for an editor, so catch safe errors and continue
-           if e.errno in (errno.ENOENT, ):
-               continue
-           raise
+            # We're searching for an editor, so catch safe errors and continue
+            if e.errno in (errno.ENOENT, ):
+                continue
+            raise
         if x == 0:
             return True
         elif x == 127:
@@ -97,6 +98,34 @@ def edit_commit_message(infotext, ignoreline=DEFAULT_IGNORE_LINE,
                             This will not be removed from the message
                             after the user has edited it.
 
+    :return:    commit message or None.
+    """
+
+    if not start_message is None:
+        start_message = start_message.encode(bzrlib.user_encoding)
+    infotext = infotext.encode(bzrlib.user_encoding, 'replace')
+    return edit_commit_message_encoded(infotext, ignoreline, start_message)
+
+
+def edit_commit_message_encoded(infotext, ignoreline=DEFAULT_IGNORE_LINE,
+                                start_message=None):
+    """Let the user edit a commit message in a temp file.
+
+    This is run if they don't give a message or
+    message-containing file on the command line.
+
+    :param infotext:    Text to be displayed at bottom of message
+                        for the user's reference;
+                        currently similar to 'bzr status'.
+                        The string is already encoded
+
+    :param ignoreline:  The separator to use above the infotext.
+
+    :param start_message:   The text to place above the separator, if any.
+                            This will not be removed from the message
+                            after the user has edited it.
+                            The string is already encoded
+
     :return:    commit message or None.
     """
     msgfilename = None
@@ -113,24 +142,29 @@ def edit_commit_message(infotext, ignoreline=DEFAULT_IGNORE_LINE,
         # codecs.open() ALWAYS opens file in binary mode but we need text mode
         # 'rU' mode useful when bzr.exe used on Cygwin (bialix 20070430)
         f = file(msgfilename, 'rU')
-        for line in codecs.getreader(bzrlib.user_encoding)(f):
-            stripped_line = line.strip()
-            # strip empty line before the log message starts
-            if not started:
-                if stripped_line != "":
-                    started = True
-                else:
-                    continue
-            # check for the ignore line only if there
-            # is additional information at the end
-            if hasinfo and stripped_line == ignoreline:
-                break
-            nlines += 1
-            # keep track of the last line that had some content
-            if stripped_line != "":
-                lastline = nlines
-            msg.append(line)
-        f.close()
+        try:
+            try:
+                for line in codecs.getreader(bzrlib.user_encoding)(f):
+                    stripped_line = line.strip()
+                    # strip empty line before the log message starts
+                    if not started:
+                        if stripped_line != "":
+                            started = True
+                        else:
+                            continue
+                    # check for the ignore line only if there
+                    # is additional information at the end
+                    if hasinfo and stripped_line == ignoreline:
+                        break
+                    nlines += 1
+                    # keep track of the last line that had some content
+                    if stripped_line != "":
+                        lastline = nlines
+                    msg.append(line)
+            except UnicodeDecodeError:
+                raise BadCommitMessageEncoding()
+        finally:
+            f.close()
 
         if len(msg) == 0:
             return ""
@@ -158,30 +192,30 @@ def _create_temp_file_with_commit_template(infotext,
     :param infotext:    Text to be displayed at bottom of message
                         for the user's reference;
                         currently similar to 'bzr status'.
+                        The text is already encoded.
 
     :param ignoreline:  The separator to use above the infotext.
 
     :param start_message:   The text to place above the separator, if any.
                             This will not be removed from the message
                             after the user has edited it.
+                            The string is already encoded
 
     :return:    2-tuple (temp file name, hasinfo)
     """
     import tempfile
     tmp_fileno, msgfilename = tempfile.mkstemp(prefix='bzr_log.',
-                                               dir=u'.',
+                                               dir='.',
                                                text=True)
+    msgfilename = osutils.basename(msgfilename)
     msgfile = os.fdopen(tmp_fileno, 'w')
     try:
         if start_message is not None:
-            msgfile.write("%s\n" % start_message.encode(
-                                       bzrlib.user_encoding, 'replace'))
+            msgfile.write("%s\n" % start_message)
 
         if infotext is not None and infotext != "":
             hasinfo = True
-            msgfile.write("\n\n%s\n\n%s" % (ignoreline,
-                          infotext.encode(bzrlib.user_encoding,
-                                                'replace')))
+            msgfile.write("\n\n%s\n\n%s" %(ignoreline, infotext))
         else:
             hasinfo = False
     finally:
@@ -195,8 +229,6 @@ def make_commit_message_template(working_tree, specific_files):
 
     Returns a unicode string containing the template.
     """
-    # TODO: Should probably be given the WorkingTree not the branch
-    #
     # TODO: make provision for this to be overridden or modified by a hook
     #
     # TODO: Rather than running the status command, should prepare a draft of
@@ -208,3 +240,30 @@ def make_commit_message_template(working_tree, specific_files):
     show_tree_status(working_tree, specific_files=specific_files, 
                      to_file=status_tmp)
     return status_tmp.getvalue()
+
+
+def make_commit_message_template_encoded(working_tree, specific_files,
+                                         diff=None, output_encoding='utf-8'):
+    """Prepare a template file for a commit into a branch.
+
+    Returns an encoded string.
+    """
+    # TODO: make provision for this to be overridden or modified by a hook
+    #
+    # TODO: Rather than running the status command, should prepare a draft of
+    # the revision to be committed, then pause and ask the user to
+    # confirm/write a message.
+    from StringIO import StringIO       # must be unicode-safe
+    from bzrlib.diff import show_diff_trees
+
+    template = make_commit_message_template(working_tree, specific_files)
+    template = template.encode(output_encoding, "replace")
+
+    if diff:
+        stream = StringIO()
+        show_diff_trees(working_tree.basis_tree(),
+                        working_tree, stream, specific_files,
+                        path_encoding=output_encoding)
+        template = template + '\n' + stream.getvalue()
+
+    return template
