@@ -23,6 +23,7 @@ import os
 from StringIO import StringIO
 
 from bzrlib import (
+    errors,
     patiencediff,
     trace,
     ui,
@@ -37,11 +38,12 @@ def topo_iter(vf, versions=None):
     descendants = {}
     if versions is None:
         versions = vf.versions()
+    parents = vf.get_parent_map(versions)
     def pending_parents(version):
-        return [v for v in vf.get_parents(version) if v in versions and
+        return [v for v in parents[version] if v in versions and
                 v not in seen]
     for version_id in versions:
-        for parent_id in vf.get_parents(version_id):
+        for parent_id in parents[version_id]:
             descendants.setdefault(parent_id, []).append(version_id)
     cur = [v for v in versions if len(pending_parents(v)) == 0]
     while len(cur) > 0:
@@ -55,7 +57,6 @@ def topo_iter(vf, versions=None):
             yield version_id
             seen.add(version_id)
         cur = next
-    assert len(seen) == len(versions)
 
 
 class MultiParent(object):
@@ -106,7 +107,7 @@ class MultiParent(object):
                 if block is None:
                     continue
                 i, j, n = block
-                while j + n < cur_line:
+                while j + n <= cur_line:
                     block = cur_block[p] = next_block(p)
                     if block is None:
                         break
@@ -136,6 +137,13 @@ class MultiParent(object):
             diff.hunks.append(new_text)
         return diff
 
+    def get_matching_blocks(self, parent, parent_len):
+        for hunk in self.hunks:
+            if not isinstance(hunk, ParentText) or hunk.parent != parent:
+                continue
+            yield (hunk.parent_pos, hunk.child_pos, hunk.num_lines)
+        yield parent_len, self.num_lines(), 0
+
     def to_lines(self, parents=()):
         """Contruct a fulltext from this diff and its parents"""
         mpvf = MultiMemoryVersionedFile()
@@ -186,7 +194,8 @@ class MultiParent(object):
             elif cur_line[0] == '\n':
                 hunks[-1].lines[-1] += '\n'
             else:
-                assert cur_line[0] == 'c', cur_line[0]
+                if not (cur_line[0] == 'c'):
+                    raise AssertionError(cur_line[0])
                 parent, parent_pos, child_pos, num_lines =\
                     [int(v) for v in cur_line.split(' ')[1:]]
                 hunks.append(ParentText(parent, parent_pos, child_pos,
@@ -361,7 +370,8 @@ class BaseVersionedFile(object):
         :param single_parent: If true, omit all but one parent text, (but
             retain parent metadata).
         """
-        assert no_cache or not verify
+        if not (no_cache or not verify):
+            raise ValueError()
         revisions = set(vf.versions())
         total = len(revisions)
         pb = ui.ui_factory.nested_progress_bar()
@@ -373,7 +383,7 @@ class BaseVersionedFile(object):
                     if [p for p in parents if p not in self._parents] != []:
                         continue
                     lines = [a + ' ' + l for a, l in
-                             vf.annotate_iter(revision)]
+                             vf.annotate(revision)]
                     if snapshots is None:
                         force_snapshot = None
                     else:
@@ -385,7 +395,8 @@ class BaseVersionedFile(object):
                         self.clear_cache()
                         vf.clear_cache()
                         if verify:
-                            assert lines == self.get_line_list([revision])[0]
+                            if not (lines == self.get_line_list([revision])[0]):
+                                raise AssertionError()
                             self.clear_cache()
                     pb.update('Importing revisions',
                               (total - len(revisions)) + len(added), total)
@@ -483,8 +494,7 @@ class BaseVersionedFile(object):
             pass
         diff = self.get_diff(version_id)
         lines = []
-        reconstructor = _Reconstructor(self, self._lines,
-                                       self._parents)
+        reconstructor = _Reconstructor(self, self._lines, self._parents)
         reconstructor.reconstruct_version(lines, version_id)
         self._lines[version_id] = lines
         return lines
@@ -502,7 +512,10 @@ class MultiMemoryVersionedFile(BaseVersionedFile):
         self._parents[version_id] = parent_ids
 
     def get_diff(self, version_id):
-        return self._diffs[version_id]
+        try:
+            return self._diffs[version_id]
+        except KeyError:
+            raise errors.RevisionNotPresent(version_id, self)
 
     def destroy(self):
         self._diffs = {}
@@ -534,6 +547,9 @@ class MultiVersionedFile(BaseVersionedFile):
     def add_diff(self, diff, version_id, parent_ids):
         outfile = open(self._filename + '.mpknit', 'ab')
         try:
+            outfile.seek(0, 2)      # workaround for windows bug:
+                                    # .tell() for files opened in 'ab' mode
+                                    # before any write returns 0
             start = outfile.tell()
             try:
                 zipfile = GzipFile(None, mode='ab', fileobj=outfile)
@@ -594,6 +610,9 @@ class _Reconstructor(object):
         while len(pending_reqs) > 0:
             req_version_id, req_start, req_end = pending_reqs.pop()
             # lazily allocate cursors for versions
+            if req_version_id in self.lines:
+                lines.extend(self.lines[req_version_id][req_start:req_end])
+                continue
             try:
                 start, end, kind, data, iterator = self.cursor[req_version_id]
             except KeyError:
