@@ -27,8 +27,10 @@ import os
 from bzrlib import (
     add,
     bzrdir,
+    hooks,
     )
 from bzrlib.osutils import dirname
+from bzrlib.revisiontree import RevisionTree
 from bzrlib.trace import mutter, warning
 """)
 
@@ -94,8 +96,12 @@ class MutableTree(tree.Tree):
         TODO: Perhaps callback with the ids and paths as they're added.
         """
         if isinstance(files, basestring):
-            assert(ids is None or isinstance(ids, basestring))
-            assert(kinds is None or isinstance(kinds, basestring))
+            # XXX: Passing a single string is inconsistent and should be
+            # deprecated.
+            if not (ids is None or isinstance(ids, basestring)):
+                raise AssertionError()
+            if not (kinds is None or isinstance(kinds, basestring)):
+                raise AssertionError()
             files = [files]
             if ids is not None:
                 ids = [ids]
@@ -107,13 +113,12 @@ class MutableTree(tree.Tree):
         if ids is None:
             ids = [None] * len(files)
         else:
-            assert(len(ids) == len(files))
-            ids = [osutils.safe_file_id(file_id) for file_id in ids]
-
+            if not (len(ids) == len(files)):
+                raise AssertionError()
         if kinds is None:
             kinds = [None] * len(files)
-        else:
-            assert(len(kinds) == len(files))
+        elif not len(kinds) == len(files):
+            raise AssertionError()
         for f in files:
             # generic constraint checks:
             if self.is_control_filename(f):
@@ -159,37 +164,14 @@ class MutableTree(tree.Tree):
     def apply_inventory_delta(self, changes):
         """Apply changes to the inventory as an atomic operation.
 
-        The argument is a set of changes to apply.  It must describe a
-        valid result, but the order is not important.  Specifically,
-        intermediate stages *may* be invalid, such as when two files
-        swap names.
-
-        The changes should be structured as a list of tuples, of the form
-        (old_path, new_path, file_id, new_entry).  For creation, old_path
-        must be None.  For deletion, new_path and new_entry must be None.
-        file_id is always non-None.  For renames and other mutations, all
-        values must be non-None.
-
-        If the new_entry is a directory, its children should be an empty
-        dict.  Children are handled by apply_inventory_delta itself.
-
-        :param changes: A list of tuples for the change to apply:
-            [(old_path, new_path, file_id, new_inventory_entry), ...]
+        :param changes: An inventory delta to apply to the working tree's
+            inventory.
+        :return None:
+        :seealso Inventory.apply_delta: For details on the changes parameter.
         """
         self.flush()
         inv = self.inventory
-        children = {}
-        for old_path, file_id in sorted(((op, f) for op, np, f, e in changes
-                                        if op is not None), reverse=True):
-            if file_id not in inv:
-                continue
-            children[file_id] = getattr(inv[file_id], 'children', {})
-            inv.remove_recursive_id(file_id)
-        for new_path, new_entry in sorted((np, e) for op, np, f, e in
-                                          changes if np is not None):
-            if getattr(new_entry, 'children', None) is not None:
-                new_entry.children = children.get(new_entry.file_id, {})
-            inv.add(new_entry)
+        inv.apply_delta(changes)
         self._write_inventory(inv)
 
     @needs_write_lock
@@ -201,8 +183,16 @@ class MutableTree(tree.Tree):
             revprops = {}
         if not 'branch-nick' in revprops:
             revprops['branch-nick'] = self.branch.nick
+        author = kwargs.pop('author', None)
+        if author is not None:
+            if 'author' in revprops:
+                # XXX: maybe we should just accept one of them?
+                raise AssertionError('author property given twice')
+            revprops['author'] = author
         # args for wt.commit start at message from the Commit.commit method,
         args = (message, ) + args
+        for hook in MutableTree.hooks['start_commit']:
+            hook(self)
         committed_id = commit.Commit().commit(working_tree=self,
             revprops=revprops, *args, **kwargs)
         return committed_id
@@ -257,6 +247,19 @@ class MutableTree(tree.Tree):
         """
         raise NotImplementedError(self.mkdir)
 
+    @needs_write_lock
+    def put_file_bytes_non_atomic(self, file_id, bytes):
+        """Update the content of a file in the tree.
+        
+        Note that the file is written in-place rather than being
+        written to a temporary location and renamed. As a consequence,
+        readers can potentially see the file half-written.
+
+        :param file_id: file-id of the file
+        :param bytes: the new file contents
+        """
+        raise NotImplementedError(self.put_file_bytes_non_atomic)
+
     def set_parent_ids(self, revision_ids, allow_leftmost_as_ghost=False):
         """Set the parents ids of the working tree.
 
@@ -293,7 +296,6 @@ class MutableTree(tree.Tree):
         # not in an inner loop; and we want to remove direct use of this,
         # so here as a reminder for now. RBC 20070703
         from bzrlib.inventory import InventoryEntry
-        assert isinstance(recurse, bool)
         if action is None:
             action = add.AddAction()
         
@@ -449,6 +451,60 @@ class MutableTree(tree.Tree):
                 self.read_working_inventory()
         return added, ignored
 
+    def update_basis_by_delta(self, new_revid, delta):
+        """Update the parents of this tree after a commit.
+
+        This gives the tree one parent, with revision id new_revid. The
+        inventory delta is applied to the current basis tree to generate the
+        inventory for the parent new_revid, and all other parent trees are
+        discarded.
+
+        All the changes in the delta should be changes synchronising the basis
+        tree with some or all of the working tree, with a change to a directory
+        requiring that its contents have been recursively included. That is,
+        this is not a general purpose tree modification routine, but a helper
+        for commit which is not required to handle situations that do not arise
+        outside of commit.
+
+        :param new_revid: The new revision id for the trees parent.
+        :param delta: An inventory delta (see apply_inventory_delta) describing
+            the changes from the current left most parent revision to new_revid.
+        """
+        # if the tree is updated by a pull to the branch, as happens in
+        # WorkingTree2, when there was no separation between branch and tree,
+        # then just clear merges, efficiency is not a concern for now as this
+        # is legacy environments only, and they are slow regardless.
+        if self.last_revision() == new_revid:
+            self.set_parent_ids([new_revid])
+            return
+        # generic implementation based on Inventory manipulation. See
+        # WorkingTree classes for optimised versions for specific format trees.
+        basis = self.basis_tree()
+        basis.lock_read()
+        inventory = basis.inventory
+        basis.unlock()
+        inventory.apply_delta(delta)
+        rev_tree = RevisionTree(self.branch.repository, inventory, new_revid)
+        self.set_parent_trees([(new_revid, rev_tree)])
+
+
+class MutableTreeHooks(hooks.Hooks):
+    """A dictionary mapping a hook name to a list of callables for mutabletree 
+    hooks.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+
+        """
+        hooks.Hooks.__init__(self)
+        # Invoked before a commit is done in a tree. New in 1.4
+        self['start_commit'] = []
+
+
+# install the default hooks into the MutableTree class.
+MutableTree.hooks = MutableTreeHooks()
+
 
 class _FastPath(object):
     """A path object with fast accessors for things like basename."""
