@@ -17,22 +17,21 @@
 # TODO: For things like --diff-prefix, we want a way to customize the display
 # of the option argument.
 
+import optparse
 import re
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-import optparse
-
 from bzrlib import (
     errors,
-    log,
-    registry,
     revisionspec,
-    symbol_versioning,
     )
 """)
-from bzrlib.trace import warning
 
+from bzrlib import (
+    log,
+    registry,
+    )
 
 def _parse_revision_str(revstr):
     """This handles a revision string -> revno.
@@ -86,16 +85,38 @@ def _parse_revision_str(revstr):
     [<RevisionSpec_branch branch:../../branch2>]
     >>> _parse_revision_str('branch:../../branch2..23')
     [<RevisionSpec_branch branch:../../branch2>, <RevisionSpec_revno 23>]
+    >>> _parse_revision_str('branch:..\\\\branch2')
+    [<RevisionSpec_branch branch:..\\branch2>]
+    >>> _parse_revision_str('branch:..\\\\..\\\\branch2..23')
+    [<RevisionSpec_branch branch:..\\..\\branch2>, <RevisionSpec_revno 23>]
     """
     # TODO: Maybe move this into revisionspec.py
     revs = []
-    # split on the first .. that is not followed by a / ?
-    sep = re.compile("\\.\\.(?!/)")
+    # split on .. that is not followed by a / or \
+    sep = re.compile(r'\.\.(?![\\/])')
     for x in sep.split(revstr):
         revs.append(revisionspec.RevisionSpec.from_string(x or None))
     return revs
 
 
+def _parse_change_str(revstr):
+    """Parse the revision string and return a tuple with left-most
+    parent of the revision.
+
+    >>> _parse_change_str('123')
+    (<RevisionSpec_before before:123>, <RevisionSpec_revno 123>)
+    >>> _parse_change_str('123..124')
+    Traceback (most recent call last):
+      ...
+    RangeInChangeOption: Option --change does not accept revision ranges
+    """
+    revs = _parse_revision_str(revstr)
+    if len(revs) > 1:
+        raise errors.RangeInChangeOption()
+    return (revisionspec.RevisionSpec.from_string('before:' + revstr),
+            revs[0])
+
+
 def _parse_merge_type(typestring):
     return get_merge_type(typestring)
 
@@ -120,34 +141,54 @@ class Option(object):
     Otherwise None.
     """
 
-    # TODO: Some way to show in help a description of the option argument
+    # The dictionary of standard options. These are always legal.
+    STD_OPTIONS = {}
 
+    # The dictionary of commonly used options. these are only legal
+    # if a command explicitly references them by name in the list
+    # of supported options.
     OPTIONS = {}
 
     def __init__(self, name, help='', type=None, argname=None,
-                 short_name=None):
+                 short_name=None, param_name=None, custom_callback=None):
         """Make a new command option.
 
-        name -- regular name of the command, used in the double-dash
+        :param name: regular name of the command, used in the double-dash
             form and also as the parameter to the command's run() 
-            method.
+            method (unless param_name is specified).
 
-        help -- help message displayed in command help
+        :param help: help message displayed in command help
 
-        type -- function called to parse the option argument, or 
+        :param type: function called to parse the option argument, or 
             None (default) if this option doesn't take an argument.
 
-        argname -- name of option argument, if any
+        :param argname: name of option argument, if any
+
+        :param short_name: short option code for use with a single -, e.g.
+            short_name="v" to enable parsing of -v.
+
+        :param param_name: name of the parameter which will be passed to
+            the command's run() method.
+
+        :param custom_callback: a callback routine to be called after normal
+            processing. The signature of the callback routine is
+            (option, name, new_value, parser).
         """
         self.name = name
         self.help = help
         self.type = type
         self._short_name = short_name
         if type is None:
-            assert argname is None
+            if argname:
+                raise ValueError('argname not valid for booleans')
         elif argname is None:
             argname = 'ARG'
         self.argname = argname
+        if param_name is None:
+            self._param_name = self.name
+        else:
+            self._param_name = param_name
+        self.custom_callback = custom_callback
 
     def short_name(self):
         if self._short_name:
@@ -169,12 +210,15 @@ class Option(object):
             option_strings.append('-%s' % short_name)
         optargfn = self.type
         if optargfn is None:
-            parser.add_option(action='store_true', dest=self.name, 
+            parser.add_option(action='callback', 
+                              callback=self._optparse_bool_callback, 
+                              callback_args=(True,),
                               help=self.help,
-                              default=OptionParser.DEFAULT_VALUE,
                               *option_strings)
             negation_strings = ['--%s' % self.get_negation_name()]
-            parser.add_option(action='store_false', dest=self.name, 
+            parser.add_option(action='callback', 
+                              callback=self._optparse_bool_callback, 
+                              callback_args=(False,),
                               help=optparse.SUPPRESS_HELP, *negation_strings)
         else:
             parser.add_option(action='callback', 
@@ -184,8 +228,16 @@ class Option(object):
                               default=OptionParser.DEFAULT_VALUE, 
                               *option_strings)
 
+    def _optparse_bool_callback(self, option, opt_str, value, parser, bool_v):
+        setattr(parser.values, self._param_name, bool_v)
+        if self.custom_callback is not None:
+            self.custom_callback(option, self._param_name, bool_v, parser)
+
     def _optparse_callback(self, option, opt, value, parser):
-        setattr(parser.values, self.name, self.type(value))
+        v = self.type(value)
+        setattr(parser.values, self._param_name, v)
+        if self.custom_callback is not None:
+            self.custom_callback(option, self.name, v, parser)
 
     def iter_switches(self):
         """Iterate through the list of switches provided by the option
@@ -197,6 +249,9 @@ class Option(object):
             argname = argname.upper()
         yield self.name, self.short_name(), argname, self.help
 
+    def is_hidden(self, name):
+        return False
+
 
 class ListOption(Option):
     """Option used to provide a list of values.
@@ -221,11 +276,13 @@ class ListOption(Option):
                           *option_strings)
 
     def _optparse_callback(self, option, opt, value, parser):
-        values = getattr(parser.values, self.name)
+        values = getattr(parser.values, self._param_name)
         if value == '-':
             del values[:]
         else:
             values.append(self.type(value))
+        if self.custom_callback is not None:
+            self.custom_callback(option, self._param_name, values, parser)
 
 
 class RegistryOption(Option):
@@ -300,7 +357,7 @@ class RegistryOption(Option):
         if self.value_switches:
             for key in self.registry.keys():
                 option_strings = ['--%s' % key]
-                if getattr(self.registry.get_info(key), 'hidden', False):
+                if self.is_hidden(key):
                     help = optparse.SUPPRESS_HELP
                 else:
                     help = self.registry.get_help(key)
@@ -311,7 +368,10 @@ class RegistryOption(Option):
 
     def _optparse_value_callback(self, cb_value):
         def cb(option, opt, value, parser):
-            setattr(parser.values, self.name, self.type(cb_value))
+            v = self.type(cb_value)
+            setattr(parser.values, self._param_name, v)
+            if self.custom_callback is not None:
+                self.custom_callback(option, self._param_name, v, parser)
         return cb
 
     def iter_switches(self):
@@ -325,6 +385,11 @@ class RegistryOption(Option):
             for key in sorted(self.registry.keys()):
                 yield key, None, None, self.registry.get_help(key)
 
+    def is_hidden(self, name):
+        if name == self.name:
+            return Option.is_hidden(self, name)
+        return getattr(self.registry.get_info(name), 'hidden', False)
+
 
 class OptionParser(optparse.OptionParser):
     """OptionParser that raises exceptions instead of exiting"""
@@ -345,8 +410,23 @@ def get_optparser(options):
     return parser
 
 
+def custom_help(name, help):
+    """Clone a common option overriding the help."""
+    import copy
+    o = copy.copy(Option.OPTIONS[name])
+    o.help = help
+    return o
+
+
+def _standard_option(name, **kwargs):
+    """Register a standard option."""
+    # All standard options are implicitly 'global' ones
+    Option.STD_OPTIONS[name] = Option(name, **kwargs)
+    Option.OPTIONS[name] = Option.STD_OPTIONS[name]
+
+
 def _global_option(name, **kwargs):
-    """Register o as a global option."""
+    """Register a global option."""
     Option.OPTIONS[name] = Option(name, **kwargs)
 
 
@@ -359,6 +439,34 @@ class MergeTypeRegistry(registry.Registry):
     pass
 
 
+# This is the verbosity level detected during command line parsing.
+# Note that the final value is dependent on the order in which the
+# various flags (verbose, quiet, no-verbose, no-quiet) are given.
+# The final value will be one of the following:
+#
+# * -ve for quiet
+# * 0 for normal
+# * +ve for verbose
+_verbosity_level = 0
+
+
+def _verbosity_level_callback(option, opt_str, value, parser):
+    global _verbosity_level
+    if not value:
+        # Either --no-verbose or --no-quiet was specified
+        _verbosity_level = 0
+    elif opt_str == "verbose":
+        if _verbosity_level > 0:
+            _verbosity_level += 1
+        else:
+            _verbosity_level = 1
+    else:
+        if _verbosity_level < 0:
+            _verbosity_level -= 1
+        else:
+            _verbosity_level = -1
+
+
 _merge_type_registry = MergeTypeRegistry()
 _merge_type_registry.register_lazy('merge3', 'bzrlib.merge', 'Merge3Merger',
                                    "Native diff3-style merge")
@@ -366,7 +474,20 @@ _merge_type_registry.register_lazy('diff3', 'bzrlib.merge', 'Diff3Merger',
                                    "Merge using external diff3")
 _merge_type_registry.register_lazy('weave', 'bzrlib.merge', 'WeaveMerger',
                                    "Weave-based merge")
+_merge_type_registry.register_lazy('lca', 'bzrlib.merge', 'LCAMerger',
+                                   "LCA-newness merge")
 
+# Declare the standard options
+_standard_option('help', short_name='h',
+                 help='Show help message.')
+_standard_option('verbose', short_name='v',
+                 help='Display more information.',
+                 custom_callback=_verbosity_level_callback)
+_standard_option('quiet', short_name='q',
+                 help="Only display errors and warnings.",
+                 custom_callback=_verbosity_level_callback)
+
+# Declare commonly used options
 _global_option('all')
 _global_option('overwrite', help='Ignore differences between branches and '
                'overwrite unconditionally.')
@@ -386,16 +507,18 @@ _global_option('profile',
 _global_option('revision',
                type=_parse_revision_str,
                short_name='r',
-               help='See \'help revisionspec\' for details.')
+               help='See "help revisionspec" for details.')
+_global_option('change',
+               type=_parse_change_str,
+               short_name='c',
+               param_name='revision',
+               help='Select changes introduced by the specified revision. See also "help revisionspec".')
 _global_option('show-ids',
                help='Show internal object ids.')
 _global_option('timezone', 
                type=str,
                help='display timezone as local, original, or utc')
 _global_option('unbound')
-_global_option('verbose',
-               help='Display more information.',
-               short_name='v')
 _global_option('version')
 _global_option('email')
 _global_option('update')
@@ -412,7 +535,6 @@ _global_registry_option('merge-type', 'Select a particular merge algorithm.',
                         _merge_type_registry, value_switches=True,
                         title='Merge algorithm')
 _global_option('pattern', type=str)
-_global_option('quiet', short_name='q')
 _global_option('remember', help='Remember the specified location as a'
                ' default.')
 _global_option('reprocess', help='Reprocess to reduce spurious conflicts.')
@@ -420,7 +542,3 @@ _global_option('kind', type=str)
 _global_option('dry-run',
                help="Show what would be done, but don't actually do anything.")
 _global_option('name-from-revision', help='The path name in the old tree.')
-
-_help_option = Option('help',
-                      help='Show help message.',
-                      short_name='h')
