@@ -17,15 +17,23 @@
 import base64
 import os
 from StringIO import StringIO
+import urlparse
 import xmlrpclib
 
-from bzrlib.tests import TestCase, TestSkipped
+from bzrlib import (
+    config,
+    osutils,
+    tests,
+    ui,
+    )
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
 
 # local import
-from lp_registration import (
+from bzrlib.plugins.launchpad.lp_registration import (
         BaseRequest,
         BranchBugLinkRequest,
         BranchRegistrationRequest,
+        ResolveLaunchpadPathRequest,
         LaunchpadService,
         )
 
@@ -74,19 +82,25 @@ class InstrumentedXMLRPCTransport(xmlrpclib.Transport):
     # Python 2.5's xmlrpclib looks for this.
     _use_datetime = False
 
-    def __init__(self, testcase):
+    def __init__(self, testcase, expect_auth):
         self.testcase = testcase
+        self.expect_auth = expect_auth
 
     def make_connection(self, host):
         host, http_headers, x509 = self.get_host_info(host)
         test = self.testcase
         self.connected_host = host
-        auth_hdrs = [v for k,v in http_headers if k == 'Authorization']
-        assert len(auth_hdrs) == 1
-        authinfo = auth_hdrs[0]
-        expected_auth = 'testuser@launchpad.net:testpassword'
-        test.assertEquals(authinfo,
-                'Basic ' + base64.encodestring(expected_auth).strip())
+        if self.expect_auth:
+            auth_hdrs = [v for k,v in http_headers if k == 'Authorization']
+            if len(auth_hdrs) != 1:
+                raise AssertionError("multiple auth headers: %r"
+                    % (auth_hdrs,))
+            authinfo = auth_hdrs[0]
+            expected_auth = 'testuser@launchpad.net:testpassword'
+            test.assertEquals(authinfo,
+                    'Basic ' + base64.encodestring(expected_auth).strip())
+        elif http_headers:
+            raise AssertionError()
         return InstrumentedXMLRPCConnection(test)
 
     def send_request(self, connection, handler_path, request_body):
@@ -103,20 +117,22 @@ class InstrumentedXMLRPCTransport(xmlrpclib.Transport):
 
     def send_content(self, conn, request_body):
         unpacked, method = xmlrpclib.loads(request_body)
-        assert None not in unpacked, \
-                "xmlrpc result %r shouldn't contain None" % (unpacked,)
+        if None in unpacked:
+            raise AssertionError(
+                "xmlrpc result %r shouldn't contain None" % (unpacked,))
         self.sent_params = unpacked
 
 
 class MockLaunchpadService(LaunchpadService):
 
-    def send_request(self, method_name, method_params):
+    def send_request(self, method_name, method_params, authenticated):
         """Stash away the method details rather than sending them to a real server"""
         self.called_method_name = method_name
         self.called_method_params = method_params
+        self.called_authenticated = authenticated
 
 
-class TestBranchRegistration(TestCase):
+class TestBranchRegistration(TestCaseWithTransport):
     SAMPLE_URL = 'http://bazaar-vcs.org/bzr/bzr.dev/'
     SAMPLE_OWNER = 'jhacker@foo.com'
     SAMPLE_BRANCH_ID = 'bzr.dev'
@@ -124,17 +140,33 @@ class TestBranchRegistration(TestCase):
     def setUp(self):
         super(TestBranchRegistration, self).setUp()
         # make sure we have a reproducible standard environment
-        if 'BZR_LP_XMLRPC_URL' in os.environ:
-            del os.environ['BZR_LP_XMLRPC_URL']
+        self._captureVar('BZR_LP_XMLRPC_URL', None)
 
     def test_register_help(self):
         """register-branch accepts --help"""
         out, err = self.run_bzr(['register-branch', '--help'])
         self.assertContainsRe(out, r'Register a branch')
 
-    def test_register_no_url(self):
+    def test_register_no_url_no_branch(self):
         """register-branch command requires parameters"""
-        self.run_bzr('register-branch', retcode=3)
+        self.make_repository('.')
+        self.run_bzr_error(
+            ['register-branch requires a public branch url - '
+             'see bzr help register-branch'],
+            'register-branch')
+
+    def test_register_no_url_in_published_branch_no_error(self):
+        b = self.make_branch('.')
+        b.set_public_branch('http://test-server.com/bzr/branch')
+        out, err = self.run_bzr(['register-branch', '--dry-run'])
+        self.assertEqual('Branch registered.\n', out)
+        self.assertEqual('', err)
+
+    def test_register_no_url_in_unpublished_branch_errors(self):
+        b = self.make_branch('.')
+        out, err = self.run_bzr_error(['no public branch'],
+            ['register-branch', '--dry-run'])
+        self.assertEqual('', out)
 
     def test_register_dry_run(self):
         out, err = self.run_bzr(['register-branch',
@@ -145,7 +177,7 @@ class TestBranchRegistration(TestCase):
     def test_onto_transport(self):
         """Test how the request is sent by transmitting across a mock Transport"""
         # use a real transport, but intercept at the http/xml layer
-        transport = InstrumentedXMLRPCTransport(self)
+        transport = InstrumentedXMLRPCTransport(self, expect_auth=True)
         service = LaunchpadService(transport)
         service.registrant_email = 'testuser@launchpad.net'
         service.registrant_password = 'testpassword'
@@ -156,7 +188,7 @@ class TestBranchRegistration(TestCase):
                 'author@launchpad.net',
                 'product')
         rego.submit(service)
-        self.assertEquals(transport.connected_host, 'xmlrpc.launchpad.net')
+        self.assertEquals(transport.connected_host, 'xmlrpc.edge.launchpad.net')
         self.assertEquals(len(transport.sent_params), 6)
         self.assertEquals(transport.sent_params,
                 ('http://test-server.com/bzr/branch',  # branch_url
@@ -167,6 +199,17 @@ class TestBranchRegistration(TestCase):
                  'product'))
         self.assertTrue(transport.got_request)
 
+    def test_onto_transport_unauthenticated(self):
+        """Test how an unauthenticated request is transmitted across a mock Transport"""
+        transport = InstrumentedXMLRPCTransport(self, expect_auth=False)
+        service = LaunchpadService(transport)
+        resolve = ResolveLaunchpadPathRequest('bzr')
+        resolve.submit(service)
+        self.assertEquals(transport.connected_host, 'xmlrpc.edge.launchpad.net')
+        self.assertEquals(len(transport.sent_params), 1)
+        self.assertEquals(transport.sent_params, ('bzr', ))
+        self.assertTrue(transport.got_request)
+
     def test_subclass_request(self):
         """Define a new type of xmlrpc request"""
         class DummyRequest(BaseRequest):
@@ -186,10 +229,11 @@ class TestBranchRegistration(TestCase):
         """Send registration to mock server"""
         test_case = self
         class MockRegistrationService(MockLaunchpadService):
-            def send_request(self, method_name, method_params):
+            def send_request(self, method_name, method_params, authenticated):
                 test_case.assertEquals(method_name, "register_branch")
                 test_case.assertEquals(list(method_params),
                         ['url', 'name', 'title', 'description', 'email', 'name'])
+                test_case.assertEquals(authenticated, True)
                 return 'result'
         service = MockRegistrationService()
         rego = BranchRegistrationRequest('url', 'name', 'title',
@@ -201,10 +245,11 @@ class TestBranchRegistration(TestCase):
         """Send registration to mock server"""
         test_case = self
         class MockRegistrationService(MockLaunchpadService):
-            def send_request(self, method_name, method_params):
+            def send_request(self, method_name, method_params, authenticated):
                 test_case.assertEquals(method_name, "register_branch")
                 test_case.assertEquals(list(method_params),
                         ['http://server/branch', 'branch', '', '', '', ''])
+                test_case.assertEquals(authenticated, True)
                 return 'result'
         service = MockRegistrationService()
         rego = BranchRegistrationRequest('http://server/branch')
@@ -215,12 +260,87 @@ class TestBranchRegistration(TestCase):
         """Send bug-branch link to mock server"""
         test_case = self
         class MockService(MockLaunchpadService):
-            def send_request(self, method_name, method_params):
+            def send_request(self, method_name, method_params, authenticated):
                 test_case.assertEquals(method_name, "link_branch_to_bug")
                 test_case.assertEquals(list(method_params),
                         ['http://server/branch', 1234, ''])
+                test_case.assertEquals(authenticated, True)
                 return 'http://launchpad.net/bug/1234'
         service = MockService()
         rego = BranchBugLinkRequest('http://server/branch', 1234)
         result = rego.submit(service)
         self.assertEquals(result, 'http://launchpad.net/bug/1234')
+
+    def test_mock_resolve_lp_url(self):
+        test_case = self
+        class MockService(MockLaunchpadService):
+            def send_request(self, method_name, method_params, authenticated):
+                test_case.assertEquals(method_name, "resolve_lp_path")
+                test_case.assertEquals(list(method_params), ['bzr'])
+                test_case.assertEquals(authenticated, False)
+                return dict(urls=[
+                        'bzr+ssh://bazaar.launchpad.net~bzr/bzr/trunk',
+                        'sftp://bazaar.launchpad.net~bzr/bzr/trunk',
+                        'bzr+http://bazaar.launchpad.net~bzr/bzr/trunk',
+                        'http://bazaar.launchpad.net~bzr/bzr/trunk'])
+        service = MockService()
+        resolve = ResolveLaunchpadPathRequest('bzr')
+        result = resolve.submit(service)
+        self.assertTrue('urls' in result)
+        self.assertEquals(result['urls'], [
+                'bzr+ssh://bazaar.launchpad.net~bzr/bzr/trunk',
+                'sftp://bazaar.launchpad.net~bzr/bzr/trunk',
+                'bzr+http://bazaar.launchpad.net~bzr/bzr/trunk',
+                'http://bazaar.launchpad.net~bzr/bzr/trunk'])
+
+
+class TestGatherUserCredentials(tests.TestCaseInTempDir):
+
+    def setUp(self):
+        super(TestGatherUserCredentials, self).setUp()
+        # make sure we have a reproducible standard environment
+        self._captureVar('BZR_LP_XMLRPC_URL', None)
+
+    def test_gather_user_credentials_has_password(self):
+        service = LaunchpadService()
+        service.registrant_password = 'mypassword'
+        # This should be a basic no-op, since we already have the password
+        service.gather_user_credentials()
+        self.assertEqual('mypassword', service.registrant_password)
+
+    def test_gather_user_credentials_from_auth_conf(self):
+        auth_path = config.authentication_config_filename()
+        service = LaunchpadService()
+        g_conf = config.GlobalConfig()
+        g_conf.set_user_option('email', 'Test User <test@user.com>')
+        f = open(auth_path, 'wb')
+        try:
+            scheme, hostinfo = urlparse.urlsplit(service.service_url)[:2]
+            f.write('[section]\n'
+                    'scheme=%s\n'
+                    'host=%s\n'
+                    'user=test@user.com\n'
+                    'password=testpass\n'
+                    % (scheme, hostinfo))
+        finally:
+            f.close()
+        self.assertIs(None, service.registrant_password)
+        service.gather_user_credentials()
+        self.assertEqual('test@user.com', service.registrant_email)
+        self.assertEqual('testpass', service.registrant_password)
+
+    def test_gather_user_credentials_prompts(self):
+        service = LaunchpadService()
+        self.assertIs(None, service.registrant_password)
+        g_conf = config.GlobalConfig()
+        g_conf.set_user_option('email', 'Test User <test@user.com>')
+        stdout = tests.StringIOWrapper()
+        ui.ui_factory = tests.TestUIFactory(stdin='userpass\n',
+                                            stdout=stdout)
+        self.assertIs(None, service.registrant_password)
+        service.gather_user_credentials()
+        self.assertEqual('test@user.com', service.registrant_email)
+        self.assertEqual('userpass', service.registrant_password)
+        self.assertContainsRe(stdout.getvalue(),
+                             'launchpad.net password for test@user\\.com')
+
