@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,15 +34,17 @@ from bzrlib.branch import BranchReferenceFormat
 from bzrlib.bzrdir import BzrDir, RemoteBzrDirFormat
 from bzrlib.config import BranchConfig, TreeConfig
 from bzrlib.decorators import needs_read_lock, needs_write_lock
-from bzrlib.errors import NoSuchRevision
+from bzrlib.errors import (
+    NoSuchRevision,
+    SmartProtocolError,
+    )
 from bzrlib.lockable_files import LockableFiles
 from bzrlib.pack import ContainerPushParser
 from bzrlib.smart import client, vfs
 from bzrlib.symbol_versioning import (
     deprecated_method,
-    zero_ninetyone,
     )
-from bzrlib.revision import NULL_REVISION
+from bzrlib.revision import ensure_null, NULL_REVISION
 from bzrlib.trace import mutter, note, warning
 
 # Note: RemoteBzrDirFormat is in bzrdir.py
@@ -62,11 +64,10 @@ class RemoteBzrDir(BzrDir):
         self._real_bzrdir = None
 
         if _client is None:
-            self._shared_medium = transport.get_shared_medium()
-            self._client = client._SmartClient(self._shared_medium)
+            medium = transport.get_smart_medium()
+            self._client = client._SmartClient(medium, transport.base)
         else:
             self._client = _client
-            self._shared_medium = None
             return
 
         path = self._path_for_remote_call(self._client)
@@ -137,7 +138,8 @@ class RemoteBzrDir(BzrDir):
         return None, self.open_branch()
 
     def open_branch(self, _unsupported=False):
-        assert _unsupported == False, 'unsupported flag support not implemented yet.'
+        if _unsupported:
+            raise NotImplementedError('unsupported flag support not implemented yet.')
         reference_url = self.get_branch_reference()
         if reference_url is None:
             # branch at this location.
@@ -150,22 +152,22 @@ class RemoteBzrDir(BzrDir):
     def open_repository(self):
         path = self._path_for_remote_call(self._client)
         verb = 'BzrDir.find_repositoryV2'
-        response = self._client.call(verb, path)
-        if (response == ('error', "Generic bzr smart protocol error: "
-                "bad request '%s'" % verb) or
-              response == ('error', "Generic bzr smart protocol error: "
-                "bad request u'%s'" % verb)):
+        try:
+            response = self._client.call(verb, path)
+        except errors.UnknownSmartMethod:
             verb = 'BzrDir.find_repository'
             response = self._client.call(verb, path)
-        assert response[0] in ('ok', 'norepository'), \
-            'unexpected response code %s' % (response,)
         if response[0] == 'norepository':
             raise errors.NoRepositoryPresent(self)
+        elif response[0] != 'ok':
+            raise SmartProtocolError('unexpected response %r' 
+                % response)
         if verb == 'BzrDir.find_repository':
             # servers that don't support the V2 method don't support external
             # references either.
             response = response + ('no', )
-        assert len(response) == 5, 'incorrect response length %s' % (response,)
+        if not (len(response) == 5):
+            raise SmartProtocolError('incorrect response length %s' % (response,))
         if response[1] == '':
             format = RemoteRepositoryFormat()
             format.rich_root_data = (response[2] == 'yes')
@@ -229,12 +231,13 @@ class RemoteRepositoryFormat(repository.RepositoryFormat):
     _matchingbzrdir = RemoteBzrDirFormat
 
     def initialize(self, a_bzrdir, shared=False):
-        assert isinstance(a_bzrdir, RemoteBzrDir), \
-            '%r is not a RemoteBzrDir' % (a_bzrdir,)
+        if not isinstance(a_bzrdir, RemoteBzrDir):
+            raise AssertionError('%r is not a RemoteBzrDir' % (a_bzrdir,))
         return a_bzrdir.create_repository(shared=shared)
     
     def open(self, a_bzrdir):
-        assert isinstance(a_bzrdir, RemoteBzrDir)
+        if not isinstance(a_bzrdir, RemoteBzrDir):
+            raise AssertionError('%r is not a RemoteBzrDir' % (a_bzrdir,))
         return a_bzrdir.open_repository()
 
     def get_format_description(self):
@@ -277,7 +280,7 @@ class RemoteRepository(object):
             self._real_repository = None
         self.bzrdir = remote_bzrdir
         if _client is None:
-            self._client = client._SmartClient(self.bzrdir._shared_medium)
+            self._client = remote_bzrdir._client
         else:
             self._client = _client
         self._format = format
@@ -361,15 +364,19 @@ class RemoteRepository(object):
         self._ensure_real()
         return self._real_repository._generate_text_key_index()
 
+    @symbol_versioning.deprecated_method(symbol_versioning.one_four)
     def get_revision_graph(self, revision_id=None):
         """See Repository.get_revision_graph()."""
+        return self._get_revision_graph(revision_id)
+
+    def _get_revision_graph(self, revision_id):
+        """Private method for using with old (< 1.2) servers to fallback."""
         if revision_id is None:
             revision_id = ''
         elif revision.is_null(revision_id):
             return {}
 
         path = self.bzrdir._path_for_remote_call(self._client)
-        assert type(revision_id) is str
         response = self._client.call_expecting_body(
             'Repository.get_revision_graph', path, revision_id)
         if response[0][0] not in ['ok', 'nosuchrevision']:
@@ -388,7 +395,8 @@ class RemoteRepository(object):
             return revision_graph
         else:
             response_body = response[1].read_body_bytes()
-            assert response_body == ''
+            if response_body:
+                raise SmartProtocolError('unexpected response body')
             raise NoSuchRevision(self, revision_id)
 
     def has_revision(self, revision_id):
@@ -398,7 +406,8 @@ class RemoteRepository(object):
             return True
         path = self.bzrdir._path_for_remote_call(self._client)
         response = self._client.call('Repository.has_revision', path, revision_id)
-        assert response[0] in ('yes', 'no'), 'unexpected response code %s' % (response,)
+        if response[0] not in ('yes', 'no'):
+            raise SmartProtocolError('unexpected response code %s' % (response,))
         return response[0] == 'yes'
 
     def has_revisions(self, revision_ids):
@@ -437,8 +446,9 @@ class RemoteRepository(object):
             fmt_committers = 'yes'
         response = self._client.call_expecting_body(
             'Repository.gather_stats', path, fmt_revid, fmt_committers)
-        assert response[0][0] == 'ok', \
-            'unexpected response code %s' % (response[0],)
+        if response[0][0] != 'ok':
+            raise SmartProtocolError('unexpected response code %s'
+                % (response[0],))
 
         body = response[1].read_body_bytes()
         result = {}
@@ -481,7 +491,8 @@ class RemoteRepository(object):
         """See Repository.is_shared()."""
         path = self.bzrdir._path_for_remote_call(self._client)
         response = self._client.call('Repository.is_shared', path)
-        assert response[0] in ('yes', 'no'), 'unexpected response code %s' % (response,)
+        if response[0] not in ('yes', 'no'):
+            raise SmartProtocolError('unexpected response code %s' % (response,))
         return response[0] == 'yes'
 
     def is_write_locked(self):
@@ -557,7 +568,8 @@ class RemoteRepository(object):
         :param repository: The repository to fallback to for non-hpss
             implemented operations.
         """
-        assert not isinstance(repository, RemoteRepository)
+        if isinstance(repository, RemoteRepository):
+            raise AssertionError()
         self._real_repository = repository
         if self._lock_mode == 'w':
             # if we are already locked, the real repository must be able to
@@ -630,8 +642,12 @@ class RemoteRepository(object):
         """
         import tempfile
         path = self.bzrdir._path_for_remote_call(self._client)
-        response, protocol = self._client.call_expecting_body(
-            'Repository.tarball', path, compression)
+        try:
+            response, protocol = self._client.call_expecting_body(
+                'Repository.tarball', path, compression)
+        except errors.UnknownSmartMethod:
+            protocol.cancel_read_body()
+            return None
         if response[0] == 'ok':
             # Extract the tarball and return it
             t = tempfile.NamedTemporaryFile()
@@ -639,12 +655,6 @@ class RemoteRepository(object):
             t.write(protocol.read_body_bytes())
             t.seek(0)
             return t
-        if (response == ('error', "Generic bzr smart protocol error: "
-                "bad request 'Repository.tarball'") or
-              response == ('error', "Generic bzr smart protocol error: "
-                "bad request u'Repository.tarball'")):
-            protocol.cancel_read_body()
-            return None
         raise errors.UnexpectedSmartServerResponse(response)
 
     def sprout(self, to_bzrdir, revision_id=None):
@@ -714,8 +724,9 @@ class RemoteRepository(object):
         return self._real_repository.clone(a_bzrdir, revision_id=revision_id)
 
     def make_working_trees(self):
-        """RemoteRepositories never create working trees by default."""
-        return False
+        """See Repository.make_working_trees"""
+        self._ensure_real()
+        return self._real_repository.make_working_trees()
 
     def revision_ids_to_search_result(self, result_set):
         """Convert a set of revision ids to a graph SearchResult."""
@@ -811,32 +822,32 @@ class RemoteRepository(object):
                 100.0 * len(self._requested_parents) / len(ancestry))
         return dict((k, ancestry[k]) for k in present_keys)
 
-    def _response_is_unknown_method(self, response, verb):
-        """Return True if response is an unknonwn method response to verb.
-        
-        :param response: The response from a smart client call_expecting_body
-            call.
-        :param verb: The verb used in that call.
-        :return: True if an unknown method was encountered.
-        """
-        # This might live better on
-        # bzrlib.smart.protocol.SmartClientRequestProtocolOne
-        if (response[0] == ('error', "Generic bzr smart protocol error: "
-                "bad request '%s'" % verb) or
-              response[0] == ('error', "Generic bzr smart protocol error: "
-                "bad request u'%s'" % verb)):
-           response[1].cancel_read_body()
-           return True
-        return False
-
     def _get_parent_map(self, keys):
         """Helper for get_parent_map that performs the RPC."""
-        medium = self._client.get_smart_medium()
+        medium = self._client._medium
         if not medium._remote_is_at_least_1_2:
             # We already found out that the server can't understand
             # Repository.get_parent_map requests, so just fetch the whole
             # graph.
-            return self.get_revision_graph()
+            # XXX: Note that this will issue a deprecation warning. This is ok
+            # :- its because we're working with a deprecated server anyway, and
+            # the user will almost certainly have seen a warning about the
+            # server version already.
+            rg = self.get_revision_graph()
+            # There is an api discrepency between get_parent_map and
+            # get_revision_graph. Specifically, a "key:()" pair in
+            # get_revision_graph just means a node has no parents. For
+            # "get_parent_map" it means the node is a ghost. So fix up the
+            # graph to correct this.
+            #   https://bugs.launchpad.net/bzr/+bug/214894
+            # There is one other "bug" which is that ghosts in
+            # get_revision_graph() are not returned at all. But we won't worry
+            # about that for now.
+            for node_id, parent_ids in rg.iteritems():
+                if parent_ids == ():
+                    rg[node_id] = (NULL_REVISION,)
+            rg[NULL_REVISION] = ()
+            return rg
 
         keys = set(keys)
         if NULL_REVISION in keys:
@@ -872,12 +883,15 @@ class RemoteRepository(object):
         body = self._serialise_search_recipe(recipe)
         path = self.bzrdir._path_for_remote_call(self._client)
         for key in keys:
-            assert type(key) is str
+            if type(key) is not str:
+                raise ValueError(
+                    "key %r not a plain string" % (key,))
         verb = 'Repository.get_parent_map'
         args = (path,) + tuple(keys)
-        response = self._client.call_with_body_bytes_expecting_body(
-            verb, args, self._serialise_search_recipe(recipe))
-        if self._response_is_unknown_method(response, verb):
+        try:
+            response = self._client.call_with_body_bytes_expecting_body(
+                verb, args, self._serialise_search_recipe(recipe))
+        except errors.UnknownSmartMethod:
             # Server does not support this method, so get the whole graph.
             # Worse, we have to force a disconnection, because the server now
             # doesn't realise it has a body on the wire to consume, so the
@@ -889,9 +903,9 @@ class RemoteRepository(object):
             # To avoid having to disconnect repeatedly, we keep track of the
             # fact the server doesn't understand remote methods added in 1.2.
             medium._remote_is_at_least_1_2 = False
-            return self.get_revision_graph()
-        elif response[0][0] not in ['ok']:
-            reponse[1].cancel_read_body()
+            return self.get_revision_graph(None)
+        if response[0][0] not in ['ok']:
+            response[1].cancel_read_body()
             raise errors.UnexpectedSmartServerResponse(response[0])
         if response[0][0] == 'ok':
             coded = bz2.decompress(response[1].read_body_bytes())
@@ -1007,7 +1021,8 @@ class RemoteRepository(object):
         return self._real_repository.pack()
 
     def set_make_working_trees(self, new_value):
-        raise NotImplementedError(self.set_make_working_trees)
+        self._ensure_real()
+        self._real_repository.set_make_working_trees(new_value)
 
     @needs_write_lock
     def sign_revision(self, revision_id, gpg_strategy):
@@ -1046,17 +1061,18 @@ class RemoteRepository(object):
         return self._real_repository.has_signature_for_revision_id(revision_id)
 
     def get_data_stream_for_search(self, search):
-        medium = self._client.get_smart_medium()
+        medium = self._client._medium
         if not medium._remote_is_at_least_1_2:
             self._ensure_real()
             return self._real_repository.get_data_stream_for_search(search)
         REQUEST_NAME = 'Repository.stream_revisions_chunked'
         path = self.bzrdir._path_for_remote_call(self._client)
         body = self._serialise_search_recipe(search.get_recipe())
-        response, protocol = self._client.call_with_body_bytes_expecting_body(
-            REQUEST_NAME, (path,), body)
-
-        if self._response_is_unknown_method((response, protocol), REQUEST_NAME):
+        try:
+            result = self._client.call_with_body_bytes_expecting_body(
+                REQUEST_NAME, (path,), body)
+            response, protocol = result
+        except errors.UnknownSmartMethod:
             # Server does not support this method, so fall back to VFS.
             # Worse, we have to force a disconnection, because the server now
             # doesn't realise it has a body on the wire to consume, so the
@@ -1152,24 +1168,6 @@ class RemoteBranchLockableFiles(LockableFiles):
         self._dir_mode = None
         self._file_mode = None
 
-    def get(self, path):
-        """'get' a remote path as per the LockableFiles interface.
-
-        :param path: the file to 'get'. If this is 'branch.conf', we do not
-             just retrieve a file, instead we ask the smart server to generate
-             a configuration for us - which is retrieved as an INI file.
-        """
-        if path == 'branch.conf':
-            path = self.bzrdir._path_for_remote_call(self._client)
-            response = self._client.call_expecting_body(
-                'Branch.get_config_file', path)
-            assert response[0][0] == 'ok', \
-                'unexpected response code %s' % (response[0],)
-            return StringIO(response[1].read_body_bytes())
-        else:
-            # VFS fallback.
-            return LockableFiles.get(self, path)
-
 
 class RemoteBranchFormat(branch.BranchFormat):
 
@@ -1184,11 +1182,9 @@ class RemoteBranchFormat(branch.BranchFormat):
         return 'Remote BZR Branch'
 
     def open(self, a_bzrdir):
-        assert isinstance(a_bzrdir, RemoteBzrDir)
         return a_bzrdir.open_branch()
 
     def initialize(self, a_bzrdir):
-        assert isinstance(a_bzrdir, RemoteBzrDir)
         return a_bzrdir.create_branch()
 
     def supports_tags(self):
@@ -1220,7 +1216,7 @@ class RemoteBranch(branch.Branch):
         if _client is not None:
             self._client = _client
         else:
-            self._client = client._SmartClient(self.bzrdir._shared_medium)
+            self._client = remote_bzrdir._client
         self.repository = remote_repository
         if real_branch is not None:
             self._real_branch = real_branch
@@ -1240,6 +1236,7 @@ class RemoteBranch(branch.Branch):
         self._control_files = None
         self._lock_mode = None
         self._lock_token = None
+        self._repo_lock_token = None
         self._lock_count = 0
         self._leave_lock = False
 
@@ -1254,7 +1251,9 @@ class RemoteBranch(branch.Branch):
         Used before calls to self._real_branch.
         """
         if not self._real_branch:
-            assert vfs.vfs_enabled()
+            if not vfs.vfs_enabled():
+                raise AssertionError('smart server vfs must be enabled '
+                    'to use vfs implementation')
             self.bzrdir._ensure_real()
             self._real_branch = self.bzrdir._real_bzrdir.open_branch()
             # Give the remote repository the matching real repo.
@@ -1327,7 +1326,8 @@ class RemoteBranch(branch.Branch):
         if not self._lock_mode:
             remote_tokens = self._remote_lock_write(token)
             self._lock_token, self._repo_lock_token = remote_tokens
-            assert self._lock_token, 'Remote server did not return a token!'
+            if not self._lock_token:
+                raise SmartProtocolError('Remote server did not return a token!')
             # TODO: We really, really, really don't want to call _ensure_real
             # here, but it's the easiest way to ensure coherency between the
             # state of the RemoteBranch and RemoteRepository objects and the
@@ -1392,7 +1392,8 @@ class RemoteBranch(branch.Branch):
                 # Only write-locked branched need to make a remote method call
                 # to perfom the unlock.
                 return
-            assert self._lock_token, 'Locked, but no token!'
+            if not self._lock_token:
+                raise AssertionError('Locked, but no token!')
             branch_token = self._lock_token
             repo_token = self._repo_lock_token
             self._lock_token = None
@@ -1418,7 +1419,8 @@ class RemoteBranch(branch.Branch):
         """See Branch.last_revision_info()."""
         path = self.bzrdir._path_for_remote_call(self._client)
         response = self._client.call('Branch.last_revision_info', path)
-        assert response[0] == 'ok', 'unexpected response code %s' % (response,)
+        if response[0] != 'ok':
+            raise SmartProtocolError('unexpected response code %s' % (response,))
         revno = int(response[1])
         last_revision = response[2]
         return (revno, last_revision)
@@ -1428,8 +1430,8 @@ class RemoteBranch(branch.Branch):
         path = self.bzrdir._path_for_remote_call(self._client)
         response = self._client.call_expecting_body(
             'Branch.revision_history', path)
-        assert response[0][0] == 'ok', ('unexpected response code %s'
-                                        % (response[0],))
+        if response[0][0] != 'ok':
+            raise SmartProtocolError('unexpected response code %s' % (response,))
         result = response[1].read_body_bytes().split('\x00')
         if result == ['']:
             return []
@@ -1450,9 +1452,8 @@ class RemoteBranch(branch.Branch):
             path, self._lock_token, self._repo_lock_token, rev_id)
         if response[0] == 'NoSuchRevision':
             raise NoSuchRevision(self, rev_id)
-        else:
-            assert response == ('ok',), (
-                'unexpected response code %r' % (response,))
+        elif response[0] != 'ok':
+            raise SmartProtocolError('unexpected response code %s' % (response,))
         self._cache_revision_history(rev_history)
 
     def get_parent(self):
@@ -1463,9 +1464,6 @@ class RemoteBranch(branch.Branch):
         self._ensure_real()
         return self._real_branch.set_parent(url)
         
-    def get_config(self):
-        return RemoteBranchConfig(self)
-
     def sprout(self, to_bzrdir, revision_id=None):
         # Like Branch.sprout, except that it sprouts a branch in the default
         # format, because RemoteBranches can't be created at arbitrary URLs.
@@ -1500,10 +1498,23 @@ class RemoteBranch(branch.Branch):
     def is_locked(self):
         return self._lock_count >= 1
 
+    @needs_write_lock
     def set_last_revision_info(self, revno, revision_id):
-        self._ensure_real()
-        self._clear_cached_state()
-        return self._real_branch.set_last_revision_info(revno, revision_id)
+        revision_id = ensure_null(revision_id)
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response = self._client.call('Branch.set_last_revision_info',
+                path, self._lock_token, self._repo_lock_token, str(revno), revision_id)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            self._clear_cached_state()
+            return self._real_branch.set_last_revision_info(revno, revision_id)
+        if response == ('ok',):
+            self._clear_cached_state()
+        elif response[0] == 'NoSuchRevision':
+            raise NoSuchRevision(self, response[1])
+        else:
+            raise errors.UnexpectedSmartServerResponse(response)
 
     def generate_revision_history(self, revision_id, last_rev=None,
                                   other_branch=None):
@@ -1526,19 +1537,6 @@ class RemoteBranch(branch.Branch):
             other, stop_revision=stop_revision, overwrite=overwrite)
 
 
-class RemoteBranchConfig(BranchConfig):
-
-    def username(self):
-        self.branch._ensure_real()
-        return self.branch._real_branch.get_config().username()
-
-    def _get_branch_data_config(self):
-        self.branch._ensure_real()
-        if self._branch_data_config is None:
-            self._branch_data_config = TreeConfig(self.branch._real_branch)
-        return self._branch_data_config
-
-
 def _extract_tar(tar, to_dir):
     """Extract all the contents of a tarfile object.
 
