@@ -20,7 +20,7 @@
 
 from bzrlib import (
     errors,
-    symbol_versioning
+    symbol_versioning,
     )
 from bzrlib.deprecated_graph import (
     all_descendants,
@@ -31,7 +31,6 @@ from bzrlib.deprecated_graph import (
 from bzrlib.osutils import contains_whitespace
 from bzrlib.progress import DummyProgress
 from bzrlib.symbol_versioning import (deprecated_function,
-        zero_eight,
         )
 
 NULL_REVISION="null:"
@@ -115,22 +114,15 @@ class Revision(object):
     def get_summary(self):
         """Get the first line of the log message for this revision.
         """
-        return self.message.split('\n', 1)[0]
+        return self.message.lstrip().split('\n', 1)[0]
 
+    def get_apparent_author(self):
+        """Return the apparent author of this revision.
 
-def is_ancestor(revision_id, candidate_id, branch):
-    """Return true if candidate_id is an ancestor of revision_id.
-
-    A false negative will be returned if any intermediate descendent of
-    candidate_id is not present in any of the revision_sources.
-    
-    revisions_source is an object supporting a get_revision operation that
-    behaves like Branch's.
-    """
-    if is_null(candidate_id):
-        return True
-    return (candidate_id in branch.repository.get_ancestry(revision_id,
-            topo_sorted=False))
+        If the revision properties contain the author name,
+        return it. Otherwise return the committer name.
+        """
+        return self.properties.get('author', self.committer)
 
 
 def iter_ancestors(revision_id, revision_source, only_present=False):
@@ -180,6 +172,7 @@ def __get_closest(intersection):
     return matches
 
 
+@deprecated_function(symbol_versioning.one_four)
 def revision_graph(revision, revision_source):
     """Produce a graph of the ancestry of the specified revision.
     
@@ -219,12 +212,10 @@ def _revision_graph(revision, revision_source):
         for parent in parents:
             descendants[parent][node] = 1
         ancestors[node] = set(parents)
-
-    assert root not in descendants[root]
-    assert root not in ancestors[root]
     return root, ancestors, descendants
 
 
+@deprecated_function(symbol_versioning.one_three)
 def combined_graph(revision_a, revision_b, revision_source):
     """Produce a combined ancestry graph.
     Return graph root, ancestors map, descendants map, set of common nodes"""
@@ -248,6 +239,7 @@ def combined_graph(revision_a, revision_b, revision_source):
     return root, ancestors, descendants, common
 
 
+@deprecated_function(symbol_versioning.one_three)
 def common_ancestor(revision_a, revision_b, revision_source, 
                     pb=DummyProgress()):
     if None in (revision_a, revision_b):
@@ -312,9 +304,10 @@ def common_ancestor(revision_a, revision_b, revision_source,
 
 class MultipleRevisionSources(object):
     """Proxy that looks in multiple branches for revisions."""
+
+    @symbol_versioning.deprecated_method(symbol_versioning.one_three)
     def __init__(self, *args):
         object.__init__(self)
-        assert len(args) != 0
         self._revision_sources = args
 
     def revision_parents(self, revision_id):
@@ -430,51 +423,6 @@ class MultipleRevisionSources(object):
             source.unlock()
 
 
-@deprecated_function(zero_eight)
-def get_intervening_revisions(ancestor_id, rev_id, rev_source,
-                              revision_history=None):
-    """Find the longest line of descent from maybe_ancestor to revision.
-    Revision history is followed where possible.
-
-    If ancestor_id == rev_id, list will be empty.
-    Otherwise, rev_id will be the last entry.  ancestor_id will never appear.
-    If ancestor_id is not an ancestor, NotAncestor will be thrown
-    """
-    root, ancestors, descendants = revision_graph(rev_id, rev_source)
-    if len(descendants) == 0:
-        raise errors.NoSuchRevision(rev_source, rev_id)
-    if ancestor_id not in descendants:
-        rev_source.get_revision(ancestor_id)
-        raise errors.NotAncestor(rev_id, ancestor_id)
-    root_descendants = all_descendants(descendants, ancestor_id)
-    root_descendants.add(ancestor_id)
-    if rev_id not in root_descendants:
-        raise errors.NotAncestor(rev_id, ancestor_id)
-    distances = node_distances(descendants, ancestors, ancestor_id,
-                               root_descendants=root_descendants)
-
-    def best_ancestor(rev_id):
-        best = None
-        for anc_id in ancestors[rev_id]:
-            try:
-                distance = distances[anc_id]
-            except KeyError:
-                continue
-            if revision_history is not None and anc_id in revision_history:
-                return anc_id
-            elif best is None or distance > best[1]:
-                best = (anc_id, distance)
-        return best[0]
-
-    next = rev_id
-    path = []
-    while next != ancestor_id:
-        path.append(next)
-        next = best_ancestor(next)
-    path.reverse()
-    return path
-
-
 def is_reserved_id(revision_id):
     """Determine whether a revision id is reserved
 
@@ -490,8 +438,11 @@ def check_not_reserved_id(revision_id):
 
 
 def ensure_null(revision_id):
-    """Ensure only NULL_REVISION is used to represent the null revisionn"""
+    """Ensure only NULL_REVISION is used to represent the null revision"""
     if revision_id is None:
+        symbol_versioning.warn('NULL_REVISION should be used for the null'
+            ' revision instead of None, as of bzr 0.91.',
+            DeprecationWarning, stacklevel=2)
         return NULL_REVISION
     else:
         return revision_id
