@@ -41,7 +41,7 @@ def _recv_tuple(from_file):
 
 
 def _decode_tuple(req_line):
-    if req_line == None or req_line == '':
+    if req_line is None or req_line == '':
         return None
     if req_line[-1] != '\n':
         raise errors.SmartProtocolError("request %r not terminated" % req_line)
@@ -73,8 +73,9 @@ class SmartProtocolBase(object):
 class SmartServerRequestProtocolOne(SmartProtocolBase):
     """Server-side encoding and decoding logic for smart version 1."""
     
-    def __init__(self, backing_transport, write_func):
+    def __init__(self, backing_transport, write_func, root_client_path='/'):
         self._backing_transport = backing_transport
+        self._root_client_path = root_client_path
         self.excess_buffer = ''
         self._finished = False
         self.in_buffer = ''
@@ -88,7 +89,8 @@ class SmartServerRequestProtocolOne(SmartProtocolBase):
         
         :param bytes: must be a byte string
         """
-        assert isinstance(bytes, str)
+        if not isinstance(bytes, str):
+            raise ValueError(bytes)
         self.in_buffer += bytes
         if not self.has_dispatched:
             if '\n' not in self.in_buffer:
@@ -100,7 +102,8 @@ class SmartServerRequestProtocolOne(SmartProtocolBase):
                 first_line += '\n'
                 req_args = _decode_tuple(first_line)
                 self.request = request.SmartServerRequestHandler(
-                    self._backing_transport, commands=request.request_handlers)
+                    self._backing_transport, commands=request.request_handlers,
+                    root_client_path=self._root_client_path)
                 self.request.dispatch_command(req_args[0], req_args[1:])
                 if self.request.finished_reading:
                     # trivial request
@@ -131,19 +134,21 @@ class SmartServerRequestProtocolOne(SmartProtocolBase):
             self.request.accept_body(body_data)
             if self._body_decoder.finished_reading:
                 self.request.end_of_body()
-                assert self.request.finished_reading, \
-                    "no more body, request not finished"
+                if not self.request.finished_reading:
+                    raise AssertionError("no more body, request not finished")
             if self.request.response is not None:
                 self._send_response(self.request.response)
                 self.excess_buffer = self.in_buffer
                 self.in_buffer = ''
             else:
-                assert not self.request.finished_reading, \
-                    "no response and we have finished reading."
+                if self.request.finished_reading:
+                    raise AssertionError(
+                        "no response and we have finished reading.")
 
     def _send_response(self, response):
         """Send a smart server response down the output stream."""
-        assert not self._finished, 'response already sent'
+        if self._finished:
+            raise AssertionError('response already sent')
         args = response.args
         body = response.body
         self._finished = True
@@ -151,7 +156,8 @@ class SmartServerRequestProtocolOne(SmartProtocolBase):
         self._write_success_or_failure_prefix(response)
         self._write_func(_encode_tuple(args))
         if body is not None:
-            assert isinstance(body, str), 'body must be a str'
+            if not isinstance(body, str):
+                raise ValueError(body)
             bytes = self._encode_bulk_data(body)
             self._write_func(bytes)
 
@@ -200,15 +206,18 @@ class SmartServerRequestProtocolTwo(SmartServerRequestProtocolOne):
 
     def _send_response(self, response):
         """Send a smart server response down the output stream."""
-        assert not self._finished, 'response already sent'
+        if (self._finished):
+            raise AssertionError('response already sent')
         self._finished = True
         self._write_protocol_version()
         self._write_success_or_failure_prefix(response)
         self._write_func(_encode_tuple(response.args))
         if response.body is not None:
-            assert isinstance(response.body, str), 'body must be a str'
-            assert response.body_stream is None, (
-                'body_stream and body cannot both be set')
+            if not isinstance(response.body, str):
+                raise AssertionError('body must be a str')
+            if not (response.body_stream is None):
+                raise AssertionError(
+                    'body_stream and body cannot both be set')
             bytes = self._encode_bulk_data(response.body)
             self._write_func(bytes)
         elif response.body_stream is not None:
@@ -467,6 +476,7 @@ class SmartClientRequestProtocolOne(SmartProtocolBase):
         self._request = request
         self._body_buffer = None
         self._request_start_time = None
+        self._last_verb = None
 
     def call(self, *args):
         if 'hpss' in debug.debug_flags:
@@ -476,6 +486,7 @@ class SmartClientRequestProtocolOne(SmartProtocolBase):
             self._request_start_time = time.time()
         self._write_args(args)
         self._request.finished_writing()
+        self._last_verb = args[0]
 
     def call_with_body_bytes(self, args, body):
         """Make a remote call of args with body bytes 'body'.
@@ -494,6 +505,7 @@ class SmartClientRequestProtocolOne(SmartProtocolBase):
         bytes = self._encode_bulk_data(body)
         self._request.accept_bytes(bytes)
         self._request.finished_writing()
+        self._last_verb = args[0]
 
     def call_with_body_readv_array(self, args, body):
         """Make a remote call with a readv array.
@@ -513,6 +525,7 @@ class SmartClientRequestProtocolOne(SmartProtocolBase):
         self._request.finished_writing()
         if 'hpss' in debug.debug_flags:
             mutter('              %d bytes in readv request', len(readv_bytes))
+        self._last_verb = args[0]
 
     def cancel_read_body(self):
         """After expecting a body, a response code may indicate one otherwise.
@@ -537,10 +550,28 @@ class SmartClientRequestProtocolOne(SmartProtocolBase):
                 self._request_start_time = None
             else:
                 mutter('   result:   %s', repr(result)[1:-1])
+        self._response_is_unknown_method(result)
         if not expect_body:
             self._request.finished_reading()
         return result
 
+    def _response_is_unknown_method(self, result_tuple):
+        """Raise UnexpectedSmartServerResponse if the response is an 'unknonwn
+        method' response to the request.
+        
+        :param response: The response from a smart client call_expecting_body
+            call.
+        :param verb: The verb used in that call.
+        :raises: UnexpectedSmartServerResponse
+        """
+        if (result_tuple == ('error', "Generic bzr smart protocol error: "
+                "bad request '%s'" % self._last_verb) or
+              result_tuple == ('error', "Generic bzr smart protocol error: "
+                "bad request u'%s'" % self._last_verb)):
+            # The response will have no body, so we've finished reading.
+            self._request.finished_reading()
+            raise errors.UnknownSmartMethod(self._last_verb)
+        
     def read_body_bytes(self, count=-1):
         """Read bytes from the body, decoding into a byte stream.
         
