@@ -24,20 +24,40 @@ from bzrlib import (
     errors,
     registry,
     revision,
+    urlutils,
     )
 from bzrlib.bundle.serializer import write_bundle
+from bzrlib.transport import get_transport
+from bzrlib.transport.chroot import ChrootServer
 
 
 class SmartServerRequest(object):
-    """Base class for request handlers."""
+    """Base class for request handlers.
+    
+    To define a new request, subclass this class and override the `do` method
+    (and if appropriate, `do_body` as well).  Request implementors should take
+    care to call `translate_client_path` and `transport_from_client_path` as
+    appropriate when dealing with paths received from the client.
+    """
 
-    def __init__(self, backing_transport):
+    def __init__(self, backing_transport, root_client_path='/'):
         """Constructor.
 
         :param backing_transport: the base transport to be used when performing
             this request.
+        :param root_client_path: the client path that maps to the root of
+            backing_transport.  This is used to interpret relpaths received
+            from the client.  Clients will not be able to refer to paths above
+            this root.  If root_client_path is None, then no translation will
+            be performed on client paths.  Default is '/'.
         """
         self._backing_transport = backing_transport
+        if root_client_path is not None:
+            if not root_client_path.startswith('/'):
+                root_client_path = '/' + root_client_path
+            if not root_client_path.endswith('/'):
+                root_client_path += '/'
+        self._root_client_path = root_client_path
 
     def _check_enabled(self):
         """Raises DisabledMethod if this method is disabled."""
@@ -66,6 +86,8 @@ class SmartServerRequest(object):
 
     def do_body(self, body_bytes):
         """Called if the client sends a body with the request.
+
+        The do() method is still called, and must have returned None.
         
         Must return a SmartServerResponse.
         """
@@ -74,6 +96,41 @@ class SmartServerRequest(object):
         # this NotImplementedError and translate it into a 'bad request' error
         # to send to the client.
         raise NotImplementedError(self.do_body)
+    
+    def translate_client_path(self, client_path):
+        """Translate a path received from a network client into a local
+        relpath.
+
+        All paths received from the client *must* be translated.
+
+        :param client_path: the path from the client.
+        :returns: a relpath that may be used with self._backing_transport
+            (unlike the untranslated client_path, which must not be used with
+            the backing transport).
+        """
+        if self._root_client_path is None:
+            # no translation necessary!
+            return client_path
+        if not client_path.startswith('/'):
+            client_path = '/' + client_path
+        if client_path.startswith(self._root_client_path):
+            path = client_path[len(self._root_client_path):]
+            relpath = urlutils.joinpath('/', path)
+            if not relpath.startswith('/'):
+                raise ValueError(relpath)
+            return '.' + relpath
+        else:
+            raise errors.PathNotChild(client_path, self._root_client_path)
+
+    def transport_from_client_path(self, client_path):
+        """Get a backing transport corresponding to the location referred to by
+        a network client.
+
+        :seealso: translate_client_path
+        :returns: a transport cloned from self._backing_transport
+        """
+        relpath = self.translate_client_path(client_path)
+        return self._backing_transport.clone(relpath)
 
 
 class SmartServerResponse(object):
@@ -83,17 +140,31 @@ class SmartServerResponse(object):
     SuccessfulSmartServerResponse and FailedSmartServerResponse as appropriate.
     """
 
-    def __init__(self, args, body=None):
+    def __init__(self, args, body=None, body_stream=None):
+        """Constructor.
+
+        :param args: tuple of response arguments.
+        :param body: string of a response body.
+        :param body_stream: iterable of bytestrings to be streamed to the
+            client.
+        """
         self.args = args
+        if body is not None and body_stream is not None:
+            raise errors.BzrError(
+                "'body' and 'body_stream' are mutually exclusive.")
         self.body = body
+        self.body_stream = body_stream
 
     def __eq__(self, other):
         if other is None:
             return False
-        return other.args == self.args and other.body == self.body
+        return (other.args == self.args and
+                other.body == self.body and
+                other.body_stream is self.body_stream)
 
     def __repr__(self):
-        return "<SmartServerResponse args=%r body=%r>" % (self.is_successful(), 
+        status = {True: 'OK', False: 'ERR'}[self.is_successful()]
+        return "<SmartServerResponse status=%s args=%r body=%r>" % (status,
             self.args, self.body)
 
 
@@ -130,7 +201,7 @@ class SmartServerRequestHandler(object):
     # TODO: Better way of representing the body for commands that take it,
     # and allow it to be streamed into the server.
 
-    def __init__(self, backing_transport, commands):
+    def __init__(self, backing_transport, commands, root_client_path):
         """Constructor.
 
         :param backing_transport: a Transport to handle requests for.
@@ -138,6 +209,7 @@ class SmartServerRequestHandler(object):
             subclasses. e.g. bzrlib.transport.smart.vfs.vfs_commands.
         """
         self._backing_transport = backing_transport
+        self._root_client_path = root_client_path
         self._commands = commands
         self._body_bytes = ''
         self.response = None
@@ -167,7 +239,7 @@ class SmartServerRequestHandler(object):
             command = self._commands.get(cmd)
         except LookupError:
             raise errors.SmartProtocolError("bad request %r" % (cmd,))
-        self._command = command(self._backing_transport)
+        self._command = command(self._backing_transport, self._root_client_path)
         self._run_handler_code(self._command.execute, args, {})
 
     def _run_handler_code(self, callable, args, kwargs):
@@ -234,7 +306,7 @@ class GetBundleRequest(SmartServerRequest):
 
     def do(self, path, revision_id):
         # open transport relative to our base
-        t = self._backing_transport.clone(path)
+        t = self.transport_from_client_path(path)
         control, extra_path = bzrdir.BzrDir.open_containing_from_transport(t)
         repo = control.open_repository()
         tmpf = tempfile.TemporaryFile()
@@ -268,10 +340,15 @@ request_handlers.register_lazy(
     'Branch.revision_history', 'bzrlib.smart.branch', 'SmartServerRequestRevisionHistory')
 request_handlers.register_lazy(
     'Branch.set_last_revision', 'bzrlib.smart.branch', 'SmartServerBranchRequestSetLastRevision')
+request_handlers.register_lazy(
+    'Branch.set_last_revision_info', 'bzrlib.smart.branch',
+    'SmartServerBranchRequestSetLastRevisionInfo')
 request_handlers.register_lazy(
     'Branch.unlock', 'bzrlib.smart.branch', 'SmartServerBranchRequestUnlock')
 request_handlers.register_lazy(
-    'BzrDir.find_repository', 'bzrlib.smart.bzrdir', 'SmartServerRequestFindRepository')
+    'BzrDir.find_repository', 'bzrlib.smart.bzrdir', 'SmartServerRequestFindRepositoryV1')
+request_handlers.register_lazy(
+    'BzrDir.find_repositoryV2', 'bzrlib.smart.bzrdir', 'SmartServerRequestFindRepositoryV2')
 request_handlers.register_lazy(
     'BzrDirFormat.initialize', 'bzrlib.smart.bzrdir', 'SmartServerRequestInitializeBzrDir')
 request_handlers.register_lazy(
@@ -305,6 +382,17 @@ request_handlers.register_lazy(
 request_handlers.register_lazy('Repository.gather_stats',
                                'bzrlib.smart.repository',
                                'SmartServerRepositoryGatherStats')
+request_handlers.register_lazy('Repository.get_parent_map',
+                               'bzrlib.smart.repository',
+                               'SmartServerRepositoryGetParentMap')
+request_handlers.register_lazy(
+    'Repository.stream_knit_data_for_revisions',
+    'bzrlib.smart.repository',
+    'SmartServerRepositoryStreamKnitDataForRevisions')
+request_handlers.register_lazy(
+    'Repository.stream_revisions_chunked',
+    'bzrlib.smart.repository',
+    'SmartServerRepositoryStreamRevisionsChunked')
 request_handlers.register_lazy(
     'Repository.get_revision_graph', 'bzrlib.smart.repository', 'SmartServerRepositoryGetRevisionGraph')
 request_handlers.register_lazy(
