@@ -33,7 +33,7 @@ from bzrlib.weave import WeaveFile, Weave
 from bzrlib.store import TransportStore
 from bzrlib.atomicfile import AtomicFile
 from bzrlib.symbol_versioning import (deprecated_method,
-        zero_eight)
+        )
 from bzrlib.trace import mutter
 import bzrlib.ui
 
@@ -54,6 +54,8 @@ class VersionedFileStore(TransportStore):
         self._precious = precious
         self._versionedfile_class = versionedfile_class
         self._versionedfile_kwargs = versionedfile_kwargs
+        # Used for passing get_scope to versioned file constructors;
+        self.get_scope = None
 
     def _clear_cache_id(self, file_id, transaction):
         """WARNING may lead to inconsistent object references for file_id.
@@ -72,7 +74,6 @@ class VersionedFileStore(TransportStore):
 
     def filename(self, file_id):
         """Return the path relative to the transport root."""
-        file_id = osutils.safe_file_id(file_id)
         return self._relpath(file_id)
 
     def __iter__(self):
@@ -90,7 +91,6 @@ class VersionedFileStore(TransportStore):
                     break # only one suffix can match
 
     def has_id(self, file_id):
-        file_id = osutils.safe_file_id(file_id)
         suffixes = self._versionedfile_class.get_suffixes()
         filename = self.filename(file_id)
         for suffix in suffixes:
@@ -100,14 +100,12 @@ class VersionedFileStore(TransportStore):
 
     def get_empty(self, file_id, transaction):
         """Get an empty weave, which implies deleting the existing one first."""
-        file_id = osutils.safe_file_id(file_id)
         if self.has_id(file_id):
             self.delete(file_id, transaction)
         return self.get_weave_or_empty(file_id, transaction)
 
     def delete(self, file_id, transaction):
         """Remove file_id from the store."""
-        file_id = osutils.safe_file_id(file_id)
         suffixes = self._versionedfile_class.get_suffixes()
         filename = self.filename(file_id)
         for suffix in suffixes:
@@ -134,7 +132,6 @@ class VersionedFileStore(TransportStore):
         file_id. This is used to reduce duplicate filename calculations when
         using 'get_weave_or_empty'. FOR INTERNAL USE ONLY.
         """
-        file_id = osutils.safe_file_id(file_id)
         weave = transaction.map.find_weave(file_id)
         if weave is not None:
             #mutter("cache hit in %s for %s", self, file_id)
@@ -143,7 +140,7 @@ class VersionedFileStore(TransportStore):
             _filename = self.filename(file_id)
         if transaction.writeable():
             w = self._versionedfile_class(_filename, self._transport, self._file_mode,
-                                          **self._versionedfile_kwargs)
+                get_scope=self.get_scope, **self._versionedfile_kwargs)
             transaction.map.add_weave(file_id, w)
             transaction.register_dirty(w)
         else:
@@ -152,21 +149,12 @@ class VersionedFileStore(TransportStore):
                                           self._file_mode,
                                           create=False,
                                           access_mode='r',
+                                          get_scope=self.get_scope,
                                           **self._versionedfile_kwargs)
             transaction.map.add_weave(file_id, w)
             transaction.register_clean(w, precious=self._precious)
         return w
 
-    @deprecated_method(zero_eight)
-    def get_lines(self, file_id, rev_id, transaction):
-        """Return text from a particular version of a weave.
-
-        Returned as a list of lines.
-        """
-        file_id = osutils.safe_file_id(file_id)
-        w = self.get_weave(file_id, transaction)
-        return w.get_lines(rev_id)
-    
     def _make_new_versionedfile(self, file_id, transaction,
         known_missing=False, _filename=None):
         """Make a new versioned file.
@@ -183,15 +171,16 @@ class VersionedFileStore(TransportStore):
             # we try without making the directory first because thats optimising
             # for the common case.
             weave = self._versionedfile_class(_filename, self._transport, self._file_mode, create=True,
-                                              **self._versionedfile_kwargs)
+                get_scope=self.get_scope, **self._versionedfile_kwargs)
         except errors.NoSuchFile:
             if not self._prefixed:
                 # unexpected error - NoSuchFile is expected to be raised on a
                 # missing dir only and that only occurs when we are prefixed.
                 raise
             self._transport.mkdir(self.hash_prefix(file_id), mode=self._dir_mode)
-            weave = self._versionedfile_class(_filename, self._transport, 
+            weave = self._versionedfile_class(_filename, self._transport,
                                               self._file_mode, create=True,
+                                              get_scope=self.get_scope,
                                               **self._versionedfile_kwargs)
         return weave
 
@@ -202,7 +191,6 @@ class VersionedFileStore(TransportStore):
         # of calculating the filename before doing a cache lookup is more than
         # compensated for by not calculating the filename when making new
         # versioned files.
-        file_id = osutils.safe_file_id(file_id)
         _filename = self.filename(file_id)
         try:
             return self.get_weave(file_id, transaction, _filename=_filename)
@@ -214,49 +202,11 @@ class VersionedFileStore(TransportStore):
             transaction.register_dirty(weave)
             return weave
 
-    @deprecated_method(zero_eight)
-    def put_weave(self, file_id, weave, transaction):
-        """This is a deprecated API: It writes an entire collection of ids out.
-        
-        This became inappropriate when we made a versioned file api which
-        tracks the state of the collection of versions for a single id.
-        
-        Its maintained for backwards compatability but will only work on
-        weave stores - pre 0.8 repositories.
-        """
-        file_id = osutils.safe_file_id(file_id)
-        self._put_weave(file_id, weave, transaction)
-
     def _put_weave(self, file_id, weave, transaction):
         """Preserved here for upgrades-to-weaves to use."""
         myweave = self._make_new_versionedfile(file_id, transaction)
         myweave.join(weave)
 
-    @deprecated_method(zero_eight)
-    def add_text(self, file_id, rev_id, new_lines, parents, transaction):
-        """This method was a shorthand for 
-
-        vfile = self.get_weave_or_empty(file_id, transaction)
-        vfile.add_lines(rev_id, parents, new_lines)
-        """
-        file_id = osutils.safe_file_id(file_id)
-        vfile = self.get_weave_or_empty(file_id, transaction)
-        vfile.add_lines(rev_id, parents, new_lines)
-        
-    @deprecated_method(zero_eight)
-    def add_identical_text(self, file_id, old_rev_id, new_rev_id, parents,
-                           transaction):
-        """This method was a shorthand for
-
-        vfile = self.get_weave_or_empty(file_id, transaction)
-        vfile.clone_text(new_rev_id, old_rev_id, parents)
-        """
-        file_id = osutils.safe_file_id(file_id)
-        old_rev_id = osutils.safe_revision_id(old_rev_id)
-        new_rev_id = osutils.safe_revision_id(new_rev_id)
-        vfile = self.get_weave_or_empty(file_id, transaction)
-        vfile.clone_text(new_rev_id, old_rev_id, parents)
- 
     def copy(self, source, result_id, transaction):
         """Copy the source versioned file to result_id in this store."""
         self._clear_cache_id(result_id, transaction)
@@ -292,7 +242,6 @@ class VersionedFileStore(TransportStore):
         :param from_transaction: required current transaction in from_store.
         """
         from bzrlib.transactions import PassThroughTransaction
-        assert isinstance(from_store, WeaveStore)
         if from_transaction is None:
             warn("WeaveStore.copy_multi without a from_transaction parameter "
                  "is deprecated. Please provide a from_transaction.",
@@ -310,7 +259,6 @@ class VersionedFileStore(TransportStore):
             to_transaction = PassThroughTransaction()
         pb = bzrlib.ui.ui_factory.nested_progress_bar()
         try:
-            file_ids = [osutils.safe_file_id(f) for f in file_ids]
             for count, f in enumerate(file_ids):
                 mutter("copy weave {%s} into %s", f, self)
                 pb.update('copy', count, len(file_ids))
