@@ -21,11 +21,14 @@ The methods here allow for api symbol versioning.
 """
 
 __all__ = ['deprecated_function',
+           'deprecated_in',
            'deprecated_list',
            'deprecated_method',
            'DEPRECATED_PARAMETER',
            'deprecated_passed',
-           'warn', 'set_warning_method', 'zero_seven',
+           'set_warning_method',
+           'warn',
+           'zero_seven',
            'zero_eight',
            'zero_nine',
            'zero_ten',
@@ -45,10 +48,14 @@ __all__ = ['deprecated_function',
            'one_one',
            'one_two',
            'one_three',
+           'one_four',
+           'one_five',
            ]
 
 from warnings import warn
 
+import bzrlib
+
 
 DEPRECATED_PARAMETER = "A deprecated parameter marker."
 zero_seven = "%s was deprecated in version 0.7."
@@ -71,6 +78,18 @@ zero_ninetythree = one_zero # Maintained for backwards compatibility
 one_one = "%s was deprecated in version 1.1."
 one_two = "%s was deprecated in version 1.2."
 one_three = "%s was deprecated in version 1.3."
+one_four = "%s was deprecated in version 1.4."
+one_five = "%s was deprecated in version 1.5."
+
+
+def deprecated_in(version_tuple):
+    """Generate a message that something was deprecated in a release.
+
+    >>> deprecated_in((1, 4, 0))
+    '%s was deprecated in version 1.4'
+    """
+    return ("%s was deprecated in version "
+            + bzrlib._format_version_tuple(version_tuple))
 
 def set_warning_method(method):
     """Set the warning method to be used by this module.
@@ -169,7 +188,7 @@ def deprecated_passed(parameter_value):
     # we cannot just forward to a new method name.I.e. in the following
     # examples we would want to have callers that pass any value to 'bad' be
     # given a warning - because we have applied:
-    # @deprecated_parameter('bad', zero_seven)
+    # @deprecated_parameter('bad', deprecated_in((1, 5, 0))
     #
     # def __init__(self, bad=None)
     # def __init__(self, bad, other)
@@ -237,7 +256,8 @@ class DeprecatedDict(dict):
         ):
         """Create a dict that warns when read or modified.
 
-        :param deprecation_version: something like zero_nine
+        :param deprecation_version: string for the warning format to raise,
+            typically from deprecated_in()
         :param initial_value: The contents of the dict
         :param variable_name: This allows better warnings to be printed
         :param advice: String of advice on what callers should do instead 
@@ -262,7 +282,8 @@ def deprecated_list(deprecation_version, variable_name,
                     initial_value, extra=None):
     """Create a list that warns when modified
 
-    :param deprecation_version: something like zero_nine
+    :param deprecation_version: string for the warning format to raise,
+        typically from deprecated_in()
     :param initial_value: The contents of the list
     :param variable_name: This allows better warnings to be printed
     :param extra: Extra info to print when printing a warning
@@ -307,3 +328,29 @@ def deprecated_list(deprecation_version, variable_name,
                 return self._warn_deprecated(list.pop)
 
     return _DeprecatedList(initial_value)
+
+
+def suppress_deprecation_warnings():
+    """Call this function to suppress all deprecation warnings.
+
+    When this is a final release version, we don't want to annoy users with
+    lots of deprecation warnings. We only want the deprecation warnings when
+    running a dev or release candidate.
+    """
+    import warnings
+    warnings.filterwarnings('ignore', category=DeprecationWarning)
+
+
+def activate_deprecation_warnings():
+    """Call this function to activate deprecation warnings.
+
+    When running in a 'final' release we suppress deprecation warnings.
+    However, the test suite wants to see them. So when running selftest, we
+    re-enable the deprecation warnings.
+
+    Note: warnings that have already been issued under 'ignore' will not be
+    reported after this point. The 'warnings' module has already marked them as
+    handled, so they don't get issued again.
+    """
+    import warnings
+    warnings.filterwarnings('default', category=DeprecationWarning)
