@@ -62,10 +62,14 @@ class DisabledTags(_Tags):
     lookup_tag = _not_supported
     delete_tag = _not_supported
 
-    def merge_to(self, to_tags):
+    def merge_to(self, to_tags, overwrite=False):
         # we never have anything to copy
         pass
 
+    def get_reverse_tag_dict(self):
+        # There aren't any tags, so the reverse mapping is empty.
+        return {}
+
 
 class BasicTags(_Tags):
     """Tag storage in an unversioned branch control file.
@@ -82,6 +86,9 @@ class BasicTags(_Tags):
         # all done with a write lock held, so this looks atomic
         self.branch.lock_write()
         try:
+            master = self.branch.get_master_branch()
+            if master is not None:
+                master.tags.set_tag(tag_name, tag_target)
             td = self.get_tag_dict()
             td[tag_name] = tag_target
             self._set_tag_dict(td)
@@ -134,6 +141,12 @@ class BasicTags(_Tags):
                 del d[tag_name]
             except KeyError:
                 raise errors.NoSuchTag(tag_name)
+            master = self.branch.get_master_branch()
+            if master is not None:
+                try:
+                    master.tags.delete_tag(tag_name)
+                except errors.NoSuchTag:
+                    pass
             self._set_tag_dict(d)
         finally:
             self.branch.unlock()
@@ -143,7 +156,7 @@ class BasicTags(_Tags):
 
         :param new_dict: Dictionary from tag name to target.
         """
-        self.branch.lock_read()
+        self.branch.lock_write()
         try:
             self.branch._transport.put_bytes('tags',
                 self._serialize_tag_dict(new_dict))
@@ -152,7 +165,7 @@ class BasicTags(_Tags):
 
     def _serialize_tag_dict(self, tag_dict):
         td = dict((k.encode('utf-8'), v)
-                for k,v in tag_dict.items())
+                  for k,v in tag_dict.items())
         return bencode.bencode(td)
 
     def _deserialize_tag_dict(self, tag_content):
@@ -170,7 +183,7 @@ class BasicTags(_Tags):
             raise ValueError("failed to deserialize tag dictionary %r: %s"
                 % (tag_content, e))
 
-    def merge_to(self, to_tags):
+    def merge_to(self, to_tags, overwrite=False):
         """Copy tags between repositories if necessary and possible.
         
         This method has common command-line behaviour about handling 
@@ -180,9 +193,7 @@ class BasicTags(_Tags):
         exist keep their existing definitions.
 
         :param to_tags: Branch to receive these tags
-        :param just_warn: If the destination doesn't support tags and the 
-            source does have tags, just give a warning.  Otherwise, raise
-            TagsNotSupported (default).
+        :param overwrite: Overwrite conflicting tags in the target branch
 
         :returns: A list of tags that conflicted, each of which is 
             (tagname, source_target, dest_target).
@@ -200,20 +211,22 @@ class BasicTags(_Tags):
         to_tags.branch.lock_write()
         try:
             dest_dict = to_tags.get_tag_dict()
-            result, conflicts = self._reconcile_tags(source_dict, dest_dict)
+            result, conflicts = self._reconcile_tags(source_dict, dest_dict,
+                                                     overwrite)
             if result != dest_dict:
                 to_tags._set_tag_dict(result)
         finally:
             to_tags.branch.unlock()
         return conflicts
 
-    def _reconcile_tags(self, source_dict, dest_dict):
+    def _reconcile_tags(self, source_dict, dest_dict, overwrite):
         """Do a two-way merge of two tag dictionaries.
 
         only in source => source value
         only in destination => destination value
         same definitions => that
-        different definitions => keep destination value, give a warning
+        different definitions => if overwrite is False, keep destination
+            value and give a warning, otherwise use the source value
 
         :returns: (result_dict,
             [(conflicting_tag, source_target, dest_target)])
@@ -221,7 +234,7 @@ class BasicTags(_Tags):
         conflicts = []
         result = dict(dest_dict) # copy
         for name, target in source_dict.items():
-            if name not in result:
+            if name not in result or overwrite:
                 result[name] = target
             elif result[name] == target:
                 pass
