@@ -27,19 +27,20 @@ import sys
 from bzrlib.tests import (
                           adapt_modules,
                           TestCaseWithTransport,
-                          TestSuite,
-                          TestLoader,
                           iter_suite_tests,
                           )
 from bzrlib.tests.EncodingAdapter import EncodingTestAdapter
 from bzrlib.symbol_versioning import (
     deprecated_method,
-    zero_eighteen,
     )
 import bzrlib.ui as ui
 
 
-def test_suite():
+def load_tests(basic_tests, module, loader):
+    suite = loader.suiteClass()
+    # add the tests for this module
+    suite.addTests(basic_tests)
+
     testmod_names = [
                      'bzrlib.tests.blackbox.test_add',
                      'bzrlib.tests.blackbox.test_added',
@@ -50,8 +51,10 @@ def test_suite():
                      'bzrlib.tests.blackbox.test_break_lock',
                      'bzrlib.tests.blackbox.test_breakin',
                      'bzrlib.tests.blackbox.test_bound_branches',
+                     'bzrlib.tests.blackbox.test_bundle_info',
                      'bzrlib.tests.blackbox.test_cat',
                      'bzrlib.tests.blackbox.test_cat_revision',
+                     'bzrlib.tests.blackbox.test_check',
                      'bzrlib.tests.blackbox.test_checkout',
                      'bzrlib.tests.blackbox.test_command_encoding',
                      'bzrlib.tests.blackbox.test_commit',
@@ -62,6 +65,7 @@ def test_suite():
                      'bzrlib.tests.blackbox.test_export',
                      'bzrlib.tests.blackbox.test_find_merge_base',
                      'bzrlib.tests.blackbox.test_help',
+                     'bzrlib.tests.blackbox.test_hooks',
                      'bzrlib.tests.blackbox.test_ignore',
                      'bzrlib.tests.blackbox.test_ignored',
                      'bzrlib.tests.blackbox.test_info',
@@ -83,6 +87,7 @@ def test_suite():
                      'bzrlib.tests.blackbox.test_pull',
                      'bzrlib.tests.blackbox.test_push',
                      'bzrlib.tests.blackbox.test_reconcile',
+                     'bzrlib.tests.blackbox.test_reconfigure',
                      'bzrlib.tests.blackbox.test_remerge',
                      'bzrlib.tests.blackbox.test_remove',
                      'bzrlib.tests.blackbox.test_re_sign',
@@ -98,10 +103,12 @@ def test_suite():
                      'bzrlib.tests.blackbox.test_sign_my_commits',
                      'bzrlib.tests.blackbox.test_split',
                      'bzrlib.tests.blackbox.test_status',
+                     'bzrlib.tests.blackbox.test_switch',
                      'bzrlib.tests.blackbox.test_tags',
                      'bzrlib.tests.blackbox.test_testament',
                      'bzrlib.tests.blackbox.test_too_much',
                      'bzrlib.tests.blackbox.test_uncommit',
+                     'bzrlib.tests.blackbox.test_unknowns',
                      'bzrlib.tests.blackbox.test_update',
                      'bzrlib.tests.blackbox.test_upgrade',
                      'bzrlib.tests.blackbox.test_version',
@@ -109,13 +116,13 @@ def test_suite():
                      'bzrlib.tests.blackbox.test_versioning',
                      'bzrlib.tests.blackbox.test_whoami',
                      ]
+    # add the tests for the sub modules
+    suite.addTests(loader.loadTestsFromModuleNames(testmod_names))
+
     test_encodings = [
         'bzrlib.tests.blackbox.test_non_ascii',
     ]
 
-    loader = TestLoader()
-    suite = loader.loadTestsFromModuleNames(testmod_names) 
-
     adapter = EncodingTestAdapter()
     adapt_modules(test_encodings, adapter, loader, suite)
 
@@ -124,12 +131,6 @@ def test_suite():
 
 class ExternalBase(TestCaseWithTransport):
 
-    @deprecated_method(zero_eighteen)
-    def runbzr(self, args, retcode=0):
-        if isinstance(args, basestring):
-            args = args.split()
-        return self.run_bzr(args, retcode=retcode)
-
     def check_output(self, output, *args):
         """Verify that the expected output matches what bzr says.
 
