@@ -36,15 +36,18 @@ class TestLS(TestCaseWithTransport):
                                  ('a', 'hello\n'),
                                  ])
 
-    def ls_equals(self, value, *args):
-        out, err = self.run_bzr('ls', *args)
+    def ls_equals(self, value, args=None):
+        command = 'ls'
+        if args is not None:
+            command += ' ' + args
+        out, err = self.run_bzr(command)
         self.assertEqual('', err)
         self.assertEqualDiff(value, out)
 
     def test_ls_null_verbose(self):
         # Can't supply both
         self.run_bzr_error(['Cannot set both --verbose and --null'],
-                           'ls', '--verbose', '--null')
+                           'ls --verbose --null')
 
     def test_ls_basic(self):
         """Test the abilities of 'bzr ls'"""
@@ -59,8 +62,8 @@ class TestLS(TestCaseWithTransport):
         self.ls_equals('', '--versioned')
         self.ls_equals('.bzrignore\n'
                        'a\n',
-                       '--unknown', '--ignored', '--versioned')
-        self.ls_equals('', '--ignored', '--versioned')
+                       '--unknown --ignored --versioned')
+        self.ls_equals('', '--ignored --versioned')
         self.ls_equals('.bzrignore\0a\0', '--null')
 
     def test_ls_added(self):
@@ -95,10 +98,10 @@ class TestLS(TestCaseWithTransport):
             '?        .bzrignore\n'
             'V        a                                         a-id\n'
             'V        subdir/                                   subdir-id\n', 
-            '--show-ids', '--verbose')
+            '--show-ids --verbose')
         self.ls_equals('.bzrignore\0\0'
                        'a\0a-id\0'
-                       'subdir\0subdir-id\0', '--show-ids', '--null')
+                       'subdir\0subdir-id\0', '--show-ids --null')
 
     def test_ls_recursive(self):
         self.build_tree(['subdir/', 'subdir/b'])
@@ -112,7 +115,7 @@ class TestLS(TestCaseWithTransport):
         self.ls_equals('V        .bzrignore\n'
                        'V        a\n'
                        'V        subdir/\n'
-                       , '--verbose', '--non-recursive')
+                       , '--verbose --non-recursive')
 
         # Check what happens in a sub-directory
         os.chdir('subdir')
@@ -128,11 +131,11 @@ class TestLS(TestCaseWithTransport):
                        'a\0'
                        'subdir\0'
                        'subdir/b\0'
-                       , '--from-root', '--null')
+                       , '--from-root --null')
         self.ls_equals('.bzrignore\n'
                        'a\n'
                        'subdir\n'
-                       , '--from-root', '--non-recursive')
+                       , '--from-root --non-recursive')
 
     def test_ls_path(self):
         """If a path is specified, files are listed with that prefix"""
@@ -150,14 +153,14 @@ class TestLS(TestCaseWithTransport):
                        '../a\0'
                        '../subdir\0'
                        '../subdir/b\0' ,
-                       '..', '--null')
+                       '.. --null')
         self.ls_equals('?        ../.bzrignore\n'
                        '?        ../a\n'
                        'V        ../subdir/\n'
                        'V        ../subdir/b\n' ,
-                       '..', '--verbose')
-        self.run_bzr_error('cannot specify both --from-root and PATH', 'ls',
-                           '--from-root', '..')
+                       '.. --verbose')
+        self.run_bzr_error('cannot specify both --from-root and PATH',
+                           'ls --from-root ..')
 
     def test_ls_revision(self):
         self.wt.add(['a'])
@@ -166,12 +169,12 @@ class TestLS(TestCaseWithTransport):
         self.build_tree(['subdir/'])
 
         # Check what happens when we supply a specific revision
-        self.ls_equals('a\n', '--revision', '1')
+        self.ls_equals('a\n', '--revision 1')
         self.ls_equals('V        a\n'
-                       , '--verbose', '--revision', '1')
+                       , '--verbose --revision 1')
 
         os.chdir('subdir')
-        self.ls_equals('', '--revision', '1')
+        self.ls_equals('', '--revision 1')
 
     def test_ls_branch(self):
         """If a branch is specified, files are listed from it"""
@@ -185,7 +188,7 @@ class TestLS(TestCaseWithTransport):
                        'branchdir')
         self.ls_equals('branchdir/subdir\n'
                        'branchdir/subdir/b\n',
-                       'branchdir', '--revision', '1')
+                       'branchdir --revision 1')
 
     def test_ls_ignored(self):
         # Now try to do ignored files.
@@ -222,4 +225,4 @@ class TestLS(TestCaseWithTransport):
                        '--kind=directory')
         self.ls_equals('',
                        '--kind=symlink')
-        self.run_bzr_error('invalid kind specified', 'ls', '--kind=pile')
+        self.run_bzr_error('invalid kind specified', 'ls --kind=pile')
