@@ -34,9 +34,6 @@ from bzrlib.tests import (
                           TestUIFactory,
                           TestSkipped,
                           )
-from bzrlib.symbol_versioning import (
-    zero_eighteen,
-    )
 from bzrlib.tests.blackbox import ExternalBase
 
 
@@ -111,14 +108,6 @@ class TestRunBzr(ExternalBase):
         self.working_dir = working_dir
         return '', ''
 
-    def test_args(self):
-        """Test that run_bzr passes args correctly to _run_bzr_core"""
-        self.callDeprecated(
-                ['passing varargs to run_bzr was deprecated in version 0.18.'],
-                self.run_bzr,
-                'arg1', 'arg2', 'arg3', retcode=1)
-        self.assertEqual(['arg1', 'arg2', 'arg3'], self.argv)
-
     def test_encoding(self):
         """Test that run_bzr passes encoding to _run_bzr_core"""
         self.run_bzr('foo bar')
@@ -182,7 +171,7 @@ class TestBenchmarkTests(TestCaseWithTransport):
     def test_benchmark_runs_benchmark_tests(self):
         """bzr selftest --benchmark should not run the default test suite."""
         # We test this by passing a regression test name to --benchmark, which
-        # should result in 0 rests run.
+        # should result in 0 tests run.
         old_root = TestCaseWithMemoryTransport.TEST_ROOT
         try:
             TestCaseWithMemoryTransport.TEST_ROOT = None
@@ -270,6 +259,7 @@ class TestRunBzrSubprocess(TestCaseWithTransport):
     def test_run_bzr_subprocess(self):
         """The run_bzr_helper_external comand behaves nicely."""
         result = self.run_bzr_subprocess('--version')
+        result = self.run_bzr_subprocess(['--version'])
         result = self.run_bzr_subprocess('--version', retcode=None)
         self.assertContainsRe(result[0], 'is free software')
         self.assertRaises(AssertionError, self.run_bzr_subprocess, 
@@ -277,8 +267,8 @@ class TestRunBzrSubprocess(TestCaseWithTransport):
         result = self.run_bzr_subprocess('--versionn', retcode=3)
         result = self.run_bzr_subprocess('--versionn', retcode=None)
         self.assertContainsRe(result[1], 'unknown command')
-        err = self.run_bzr_subprocess('merge', '--merge-type', 'magic merge', 
-                                      retcode=3)[1]
+        err = self.run_bzr_subprocess(['merge', '--merge-type',
+                                      'magic merge'], retcode=3)[1]
         self.assertContainsRe(err, 'Bad value "magic merge" for option'
                               ' "merge-type"')
 
@@ -392,7 +382,7 @@ class TestRunBzrSubprocessCommands(TestCaseWithTransport):
         raise _DontSpawnProcess()
 
     def test_run_bzr_subprocess_no_plugins(self):
-        self.assertRaises(_DontSpawnProcess, self.run_bzr_subprocess)
+        self.assertRaises(_DontSpawnProcess, self.run_bzr_subprocess, '')
         command = self._popen_args[0]
         self.assertEqual(sys.executable, command[0])
         self.assertEqual(self.get_bzr_path(), command[1])
@@ -400,7 +390,7 @@ class TestRunBzrSubprocessCommands(TestCaseWithTransport):
 
     def test_allow_plugins(self):
         self.assertRaises(_DontSpawnProcess,
-                          self.run_bzr_subprocess, allow_plugins=True)
+                          self.run_bzr_subprocess, '', allow_plugins=True)
         command = self._popen_args[0]
         self.assertEqual([], command[2:])
 
@@ -481,13 +471,12 @@ class TestSelftestListOnly(TestCase):
     @staticmethod
     def _parse_test_list(lines, newlines_in_header=1):
         "Parse a list of lines into a tuple of 3 lists (header,body,footer)."
-
         in_header = True
         in_footer = False
         header = []
         body = []
         footer = []
-        header_newlines_found = 0 
+        header_newlines_found = 0
         for line in lines:
             if in_header:
                 if line == '':
@@ -505,7 +494,7 @@ class TestSelftestListOnly(TestCase):
                 footer.append(line)
         # If the last body line is blank, drop it off the list
         if len(body) > 0 and body[-1] == '':
-            body.pop()                
+            body.pop()
         return (header,body,footer)
 
     def test_list_only(self):
@@ -545,6 +534,11 @@ class TestSelftestListOnly(TestCase):
                                           'selftest', '--randomize', 'now'])
         (header_rand,tests_rand,dummy) = self._parse_test_list(
             out_rand.splitlines(), 2)
+        # XXX: The following line asserts that the randomized order is not the
+        # same as the default order.  It is just possible that they'll get
+        # randomized into the same order and this will falsely fail, but
+        # that's very unlikely in practice because there are thousands of
+        # tests.
         self.assertNotEqual(tests_all, tests_rand)
         self.assertEqual(sorted(tests_all), sorted(tests_rand))
         # Check that the seed can be reused to get the exact same order
@@ -557,3 +551,20 @@ class TestSelftestListOnly(TestCase):
             out_rand2.splitlines(), 2)
         self.assertEqual(tests_rand, tests_rand2)
 
+
+class TestSelftestWithIdList(TestCaseInTempDir):
+
+    def test_load_list(self):
+        # We don't want to call selftest for the whole suite, so we start with
+        # a reduced list.
+        test_list_fname = 'test.list'
+        fl = open(test_list_fname, 'wt')
+        fl.write('%s\n' % self.id())
+        fl.close()
+        out, err = self.run_bzr(
+            ['selftest', '--load-list', test_list_fname, '--list'])
+        self.assertContainsRe(out, "Listed 1 test in")
+
+    def test_load_unknown(self):
+        out, err = self.run_bzr('selftest --load-list I_do_not_exist ',
+                                retcode=3)
