@@ -17,9 +17,13 @@
 
 
 import os
+import sys
 from StringIO import StringIO
 
-from bzrlib import merge_directive
+from bzrlib import (
+    branch as _mod_branch,
+    merge_directive,
+    )
 from bzrlib.bundle import serializer
 from bzrlib.bzrdir import BzrDir
 from bzrlib import tests
@@ -116,7 +120,7 @@ class TestSend(tests.TestCaseWithTransport):
         # which would break patch-based bundles
         self.make_trees()        
         os.chdir('branch')
-        stdout = self.run_bzr_subprocess('send', '-o-')[0]
+        stdout = self.run_bzr_subprocess('send -o-')[0]
         br = read_bundle(StringIO(stdout))
         self.assertRevisions(br, ['revision3'])
 
@@ -186,9 +190,28 @@ class TestSend(tests.TestCaseWithTransport):
         stdout = self.run_bzr('send -f branch --output -')[0]
         self.assertContainsRe(stdout, 'revision3')
 
-    def test_output_option_required(self):
+    def test_mailto_option(self):
         self.make_trees()
-        self.run_bzr_error(('File must be specified with --output',),
+        branch = _mod_branch.Branch.open('branch')
+        branch.get_config().set_user_option('mail_client', 'editor')
+        self.run_bzr_error(('No mail-to address specified',), 'send -f branch')
+        branch.get_config().set_user_option('mail_client', 'bogus')
+        self.run_bzr('send -f branch -o-')
+        self.run_bzr_error(('Unknown mail client: bogus',),
+                           'send -f branch --mail-to jrandom@example.org')
+        branch.get_config().set_user_option('submit_to', 'jrandom@example.org')
+        self.run_bzr_error(('Unknown mail client: bogus',),
+                           'send -f branch')
+
+    def test_mailto_child_option(self):
+        """Make sure that child_submit_to is used."""
+        self.make_trees()
+        branch = _mod_branch.Branch.open('branch')
+        branch.get_config().set_user_option('mail_client', 'bogus')
+        parent = _mod_branch.Branch.open('parent')
+        parent.get_config().set_user_option('child_submit_to', 
+                           'somebody@example.org')
+        self.run_bzr_error(('Unknown mail client: bogus',),
                            'send -f branch')
 
     def test_format(self):
@@ -207,3 +230,30 @@ class TestSend(tests.TestCaseWithTransport):
         self.assertIs(merge_directive.MergeDirective, md.__class__)
         self.run_bzr_error(['Bad value .* for option .format.'],
                             'send -f branch -o- --format=0.999')[0]
+
+    def test_message_option(self):
+        self.make_trees()
+        self.run_bzr('send', retcode=3)
+        md = self.send_directive(['--from', 'branch'])
+        self.assertIs(None, md.message)
+        md = self.send_directive(['--from', 'branch', '-m', 'my message'])
+        self.assertEqual('my message', md.message)
+
+    def test_omitted_revision(self):
+        self.make_trees()
+        md = self.send_directive(['-r-2..', '--from', 'branch'])
+        self.assertEqual('revision2', md.base_revision_id)
+        self.assertEqual('revision3', md.revision_id)
+        md = self.send_directive(['-r..3', '--from', 'branch',
+                                 'grandparent'])
+        self.assertEqual('revision1', md.base_revision_id)
+        self.assertEqual('revision3', md.revision_id)
+
+    def test_nonexistant_branch(self):
+        if sys.platform == "win32":
+            location = "C:/i/do/not/exist/"
+        else:
+            location = "/i/do/not/exist/"
+        out, err = self.run_bzr(["send", "--from", location], retcode=3)
+        self.assertEqual(out, '')
+        self.assertEqual(err, 'bzr: ERROR: Not a branch: "%s".\n' % location)
