@@ -44,20 +44,22 @@ from bzrlib.workingtree import WorkingTree
 class BranchStatus(TestCaseWithTransport):
     
     def assertStatus(self, expected_lines, working_tree,
-        revision=None, short=False):
+        revision=None, short=False, pending=True):
         """Run status in working_tree and look for output.
         
         :param expected_lines: The lines to look for.
         :param working_tree: The tree to run status in.
         """
-        output_string = self.status_string(working_tree, revision, short)
+        output_string = self.status_string(working_tree, revision, short,
+                pending)
         self.assertEqual(expected_lines, output_string.splitlines(True))
     
-    def status_string(self, wt, revision=None, short=False):
+    def status_string(self, wt, revision=None, short=False, pending=True):
         # use a real file rather than StringIO because it doesn't handle
         # Unicode very well.
         tof = codecs.getwriter('utf-8')(TemporaryFile())
-        show_tree_status(wt, to_file=tof, revision=revision, short=short)
+        show_tree_status(wt, to_file=tof, revision=revision, short=short,
+                show_pending=pending)
         tof.seek(0)
         return tof.read().decode('utf-8')
 
@@ -93,15 +95,26 @@ class BranchStatus(TestCaseWithTransport):
                 '  bye.c\n',
                 '  hello.c\n',
                 'pending merges:\n',
-                '  pending@pending-0-0\n',
+                '  (ghost) pending@pending-0-0\n',
             ],
             wt)
         self.assertStatus([
                 '?   bye.c\n',
                 '?   hello.c\n',
-                'P   pending@pending-0-0\n',
+                'P   (ghost) pending@pending-0-0\n',
             ],
             wt, short=True)
+        self.assertStatus([
+                'unknown:\n',
+                '  bye.c\n',
+                '  hello.c\n',
+            ],
+            wt, pending=False)
+        self.assertStatus([
+                '?   bye.c\n',
+                '?   hello.c\n',
+            ],
+            wt, short=True, pending=False)
 
     def test_branch_status_revisions(self):
         """Tests branch status with revisions"""
@@ -230,6 +243,13 @@ class BranchStatus(TestCaseWithTransport):
         tof.seek(0)
         self.assertEquals(tof.readlines(), ['?   dir2/\n'])
 
+        tof = StringIO()
+        revs = [RevisionSpec.from_string('0'), RevisionSpec.from_string('1')]
+        show_tree_status(wt, specific_files=['test.c'], to_file=tof,
+                         short=True, revision=revs)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(), ['+N  test.c\n'])
+
     def test_specific_files_conflicts(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir2/'])
@@ -296,20 +316,23 @@ class CheckoutStatus(BranchStatus):
 class TestStatus(TestCaseWithTransport):
 
     def test_status_plain(self):
-        self.run_bzr("init")
+        tree = self.make_branch_and_tree('.')
 
         self.build_tree(['hello.txt'])
         result = self.run_bzr("status")[0]
         self.assertContainsRe(result, "unknown:\n  hello.txt\n")
 
-        self.run_bzr("add hello.txt")
+        tree.add("hello.txt")
         result = self.run_bzr("status")[0]
         self.assertContainsRe(result, "added:\n  hello.txt\n")
 
-        self.run_bzr("commit -m added")
+        tree.commit(message="added")
         result = self.run_bzr("status -r 0..1")[0]
         self.assertContainsRe(result, "added:\n  hello.txt\n")
 
+        result = self.run_bzr("status -c 1")[0]
+        self.assertContainsRe(result, "added:\n  hello.txt\n")
+
         self.build_tree(['world.txt'])
         result = self.run_bzr("status -r 0")[0]
         self.assertContainsRe(result, "added:\n  hello.txt\n" \
@@ -318,17 +341,17 @@ class TestStatus(TestCaseWithTransport):
         self.assertEquals(result2, result)
 
     def test_status_short(self):
-        self.run_bzr("init")
+        tree = self.make_branch_and_tree('.')
 
         self.build_tree(['hello.txt'])
         result = self.run_bzr("status --short")[0]
         self.assertContainsRe(result, "[?]   hello.txt\n")
 
-        self.run_bzr("add hello.txt")
+        tree.add("hello.txt")
         result = self.run_bzr("status --short")[0]
         self.assertContainsRe(result, "[+]N  hello.txt\n")
 
-        self.run_bzr("commit -m added")
+        tree.commit(message="added")
         result = self.run_bzr("status --short -r 0..1")[0]
         self.assertContainsRe(result, "[+]N  hello.txt\n")
 
@@ -340,17 +363,17 @@ class TestStatus(TestCaseWithTransport):
         self.assertEquals(result2, result)
 
     def test_status_versioned(self):
-        self.run_bzr("init")
+        tree = self.make_branch_and_tree('.')
 
         self.build_tree(['hello.txt'])
         result = self.run_bzr("status --versioned")[0]
         self.assertNotContainsRe(result, "unknown:\n  hello.txt\n")
 
-        self.run_bzr("add hello.txt")
+        tree.add("hello.txt")
         result = self.run_bzr("status --versioned")[0]
         self.assertContainsRe(result, "added:\n  hello.txt\n")
 
-        self.run_bzr("commit -m added")
+        tree.commit("added")
         result = self.run_bzr("status --versioned -r 0..1")[0]
         self.assertContainsRe(result, "added:\n  hello.txt\n")
 
@@ -361,6 +384,28 @@ class TestStatus(TestCaseWithTransport):
         result2 = self.run_bzr("status --versioned -r 0..")[0]
         self.assertEquals(result2, result)
 
+    def test_status_SV(self):
+        tree = self.make_branch_and_tree('.')
+
+        self.build_tree(['hello.txt'])
+        result = self.run_bzr("status -SV")[0]
+        self.assertNotContainsRe(result, "hello.txt")
+
+        tree.add("hello.txt")
+        result = self.run_bzr("status -SV")[0]
+        self.assertContainsRe(result, "[+]N  hello.txt\n")
+
+        tree.commit(message="added")
+        result = self.run_bzr("status -SV -r 0..1")[0]
+        self.assertContainsRe(result, "[+]N  hello.txt\n")
+
+        self.build_tree(['world.txt'])
+        result = self.run_bzr("status -SV -r 0")[0]
+        self.assertContainsRe(result, "[+]N  hello.txt\n")
+
+        result2 = self.run_bzr("status -SV -r 0..")[0]
+        self.assertEquals(result2, result)
+
     def assertStatusContains(self, pattern):
         """Run status, and assert it contains the given pattern"""
         result = self.run_bzr("status --short")[0]
@@ -379,6 +424,26 @@ class TestStatus(TestCaseWithTransport):
         rmdir('directory')
         self.assertStatusContains('RD  file => directory')
 
+    def test_status_illegal_revision_specifiers(self):
+        out, err = self.run_bzr('status -r 1..23..123', retcode=3)
+        self.assertContainsRe(err, 'one or two revision specifiers')
+
+    def test_status_no_pending(self):
+        a_tree = self.make_branch_and_tree('a')
+        self.build_tree(['a/a'])
+        a_tree.add('a')
+        a_tree.commit('a')
+        b_tree = a_tree.bzrdir.sprout('b').open_workingtree()
+        self.build_tree(['b/b'])
+        b_tree.add('b')
+        b_tree.commit('b')
+
+        chdir('a')
+        self.run_bzr('merge ../b')
+        out, err = self.run_bzr('status --no-pending')
+        self.assertEquals(out, "added:\n  b\n")
+
+
 
 class TestStatusEncodings(TestCaseWithTransport):
     
