@@ -23,7 +23,6 @@ from bzrlib.bzrdir import BzrDir
 from bzrlib import errors
 from bzrlib.memorytree import MemoryTree
 from bzrlib.revision import NULL_REVISION
-from bzrlib.tests import TestSkipped
 from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
 
 
@@ -89,7 +88,14 @@ class TestPull(TestCaseWithBranch):
         tree_a.commit('message 2', rev_id='rev2a')
         tree_b.commit('message 2', rev_id='rev2b')
         self.assertRaises(errors.DivergedBranches, tree_a.pull, tree_b.branch)
-        tree_a.branch.pull(tree_a.branch, overwrite=True,
+        self.assertRaises(errors.DivergedBranches,
+                          tree_a.branch.pull, tree_b.branch,
+                          overwrite=False, stop_revision='rev2b')
+        # It should not have updated the branch tip, but it should have fetched
+        # the revision
+        self.assertEqual('rev2a', tree_a.branch.last_revision())
+        self.assertTrue(tree_a.branch.repository.has_revision('rev2b'))
+        tree_a.branch.pull(tree_b.branch, overwrite=True,
                            stop_revision='rev2b')
         self.assertEqual('rev2b', tree_a.branch.last_revision())
         self.assertEqual(tree_b.branch.revision_history(),
@@ -124,7 +130,8 @@ class TestPullHook(TestCaseWithBranch):
     def test_post_pull_empty_history(self):
         target = self.make_branch('target')
         source = self.make_branch('source')
-        Branch.hooks.install_hook('post_pull', self.capture_post_pull_hook)
+        Branch.hooks.install_named_hook('post_pull',
+            self.capture_post_pull_hook, None)
         target.pull(source)
         # with nothing there we should still get a notification, and
         # have both branches locked at the notification time.
@@ -152,7 +159,8 @@ class TestPullHook(TestCaseWithBranch):
             local = BzrDir.create_branch_convenience('local2')
             local.bind(target)
         source = self.make_branch('source')
-        Branch.hooks.install_hook('post_pull', self.capture_post_pull_hook)
+        Branch.hooks.install_named_hook('post_pull',
+            self.capture_post_pull_hook, None)
         local.pull(source)
         # with nothing there we should still get a notification, and
         # have both branches locked at the notification time.
@@ -171,7 +179,8 @@ class TestPullHook(TestCaseWithBranch):
         sourcedir = target.bzrdir.clone(self.get_url('source'))
         source = MemoryTree.create_on_branch(sourcedir.open_branch())
         rev2 = source.commit('rev 2')
-        Branch.hooks.install_hook('post_pull', self.capture_post_pull_hook)
+        Branch.hooks.install_named_hook('post_pull',
+            self.capture_post_pull_hook, None)
         target.branch.pull(source.branch)
         # with nothing there we should still get a notification, and
         # have both branches locked at the notification time.
