@@ -17,26 +17,32 @@
 
 """Commands behaviour tests for bzr.
 
-This tests the behaviour of the commands.
-The API is tested in the tests/blackbox files.
+Test the internal behaviour of the commands (the blackbox tests are intended to
+test the usage of the commands).
 """
 
-from bzrlib.tests import TestLoader
+# FIXME: If the separation described above from the blackbox tests is not worth
+# it, all the tests defined below should be moved to blackbox instead. 
 
+def load_tests(basic_tests, module, loader):
+    suite = loader.suiteClass()
+    # add the tests for this module
+    suite.addTests(basic_tests)
 
-def test_suite():
     testmod_names = [
         'bzrlib.tests.commands.test_branch',
         'bzrlib.tests.commands.test_cat',
         'bzrlib.tests.commands.test_checkout',
+        'bzrlib.tests.commands.test_commit',
         'bzrlib.tests.commands.test_init',
         'bzrlib.tests.commands.test_init_repository',
         'bzrlib.tests.commands.test_merge',
         'bzrlib.tests.commands.test_missing',
         'bzrlib.tests.commands.test_pull',
         'bzrlib.tests.commands.test_push',
+        'bzrlib.tests.commands.test_update',
         ]
-    loader = TestLoader()
-    suite = loader.loadTestsFromModuleNames(testmod_names)
+    # add the tests for the sub modules
+    suite.addTests(loader.loadTestsFromModuleNames(testmod_names))
 
     return suite
