@@ -24,6 +24,7 @@ from bzrlib import (
     bzrdir,
     errors,
     graph,
+    osutils,
     remote,
     repository,
     )
@@ -36,7 +37,7 @@ from bzrlib.repofmt.weaverepo import (
     )
 from bzrlib.revision import NULL_REVISION, Revision
 from bzrlib.smart import server
-from bzrlib.symbol_versioning import one_two, one_three
+from bzrlib.symbol_versioning import one_two, one_three, one_four
 from bzrlib.tests import (
     KnownFailure,
     TestCaseWithTransport,
@@ -322,7 +323,7 @@ class TestRepository(TestCaseWithRepository):
         repo = tree.branch.repository
         revision_ids = ['a-rev', 'b-rev', 'c-rev']
         revisions = repo.get_revisions(revision_ids)
-        assert len(revisions) == 3, repr(revisions)
+        self.assertEqual(len(revisions), 3)
         zipped = zip(revisions, revision_ids)
         self.assertEqual(len(zipped), 3)
         for revision, revision_id in zipped:
@@ -335,36 +336,6 @@ class TestRepository(TestCaseWithRepository):
         rev_tree = tree.branch.repository.revision_tree(tree.last_revision())
         self.assertEqual('rev_id', rev_tree.inventory.root.revision)
 
-    def DISABLED_DELETE_OR_FIX_BEFORE_MERGE_test_create_basis_inventory(self):
-        # Needs testing here because differences between repo and working tree
-        # basis inventory formats can lead to bugs.
-        t = self.make_branch_and_tree('.')
-        b = t.branch
-        open('a', 'wb').write('a\n')
-        t.add('a')
-        t.commit('a', rev_id='r1')
-
-        t._control_files.get_utf8('basis-inventory-cache')
-
-        basis_inv = t.basis_tree().inventory
-        self.assertEquals('r1', basis_inv.revision_id)
-        
-        store_inv = b.repository.get_inventory('r1')
-        self.assertEquals(store_inv._byid, basis_inv._byid)
-
-        open('b', 'wb').write('b\n')
-        t.add('b')
-        t.commit('b', rev_id='r2')
-
-        t._control_files.get_utf8('basis-inventory-cache')
-
-        basis_inv_txt = t.read_basis_inventory()
-        basis_inv = bzrlib.xml7.serializer_v7.read_inventory_from_string(basis_inv_txt)
-        self.assertEquals('r2', basis_inv.revision_id)
-        store_inv = b.repository.get_inventory('r2')
-
-        self.assertEquals(store_inv._byid, basis_inv._byid)
-
     def test_upgrade_from_format4(self):
         from bzrlib.tests.test_upgrade import _upgrade_dir_template
         if self.repository_format.get_format_description() \
@@ -403,6 +374,8 @@ class TestRepository(TestCaseWithRepository):
         tree.add('foo', 'file1')
         tree.commit('message', rev_id='rev_id')
         repo = tree.branch.repository
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
         try:
             stream = self.applyDeprecated(one_two, repo.get_data_stream,
                 ['rev_id'])
@@ -423,6 +396,8 @@ class TestRepository(TestCaseWithRepository):
         # Get a data stream (a file-like object) for that revision
         search = graph.SearchResult(set(['rev_id']), set([NULL_REVISION]), 1,
             set(['rev_id']))
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
         try:
             stream = repo.get_data_stream_for_search(search)
         except NotImplementedError:
@@ -467,6 +442,8 @@ class TestRepository(TestCaseWithRepository):
         dest_repo = self.make_repository('dest')
         search = dest_repo.search_missing_revision_ids(source_repo,
             revision_id='rev_id')
+        source_repo.lock_read()
+        self.addCleanup(source_repo.unlock)
         try:
             stream = source_repo.get_data_stream_for_search(search)
         except NotImplementedError, e:
@@ -549,6 +526,8 @@ class TestRepository(TestCaseWithRepository):
         tree.add('foo', 'file1')
         tree.commit('message', rev_id='rev_id')
         repo = tree.branch.repository
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
 
         # Item keys will be in this order, for maximum convenience for
         # generating data to insert into knit repository:
@@ -642,6 +621,23 @@ class TestRepository(TestCaseWithRepository):
         self.assertEqual('This might be a signature',
                          repo.get_signature_text('A'))
 
+    def test_add_revision_inventory_sha1(self):
+        repo = self.make_repository('repo')
+        inv = Inventory(revision_id='A')
+        inv.root.revision = 'A'
+        inv.root.file_id = 'fixed-root'
+        repo.lock_write()
+        repo.start_write_group()
+        repo.add_revision('A', Revision('A', committer='B', timestamp=0,
+                          timezone=0, message='C'), inv=inv)
+        repo.commit_write_group()
+        repo.unlock()
+        repo.lock_read()
+        self.assertEquals(osutils.sha_string(
+            repo._serializer.write_inventory_to_string(inv)),
+            repo.get_revision('A').inventory_sha1)
+        repo.unlock()
+
     def test_install_revisions(self):
         wt = self.make_branch_and_tree('source')
         wt.commit('A', allow_pointless=True, rev_id='A')
@@ -779,6 +775,22 @@ class TestRepository(TestCaseWithRepository):
             self.assertEqual([repo.bzrdir.root_transport.base],
                              [b.base for b in repo.find_branches(using=True)])
 
+    def test_set_get_make_working_trees_true(self):
+        repo = self.make_repository('repo')
+        try:
+            repo.set_make_working_trees(True)
+        except errors.RepositoryUpgradeRequired, e:
+            raise TestNotApplicable('Format does not support this flag.')
+        self.assertTrue(repo.make_working_trees())
+
+    def test_set_get_make_working_trees_false(self):
+        repo = self.make_repository('repo')
+        try:
+            repo.set_make_working_trees(False)
+        except errors.RepositoryUpgradeRequired, e:
+            raise TestNotApplicable('Format does not support this flag.')
+        self.assertFalse(repo.make_working_trees())
+
 
 class TestRepositoryLocking(TestCaseWithRepository):
 
@@ -877,9 +889,9 @@ class TestCaseWithComplexRepository(TestCaseWithRepository):
         self.addCleanup(repository.unlock)
         trees1 = list(repository.revision_trees(revision_ids))
         trees2 = [repository.revision_tree(t) for t in revision_ids]
-        assert len(trees1) == len(trees2)
+        self.assertEqual(len(trees1), len(trees2))
         for tree1, tree2 in zip(trees1, trees2):
-            assert not tree2.changes_from(tree1).has_changed()
+            self.assertFalse(tree2.changes_from(tree1).has_changed())
 
     def test_get_deltas_for_revisions(self):
         repository = self.bzrdir.open_repository()
@@ -890,7 +902,7 @@ class TestCaseWithComplexRepository(TestCaseWithRepository):
         deltas1 = list(repository.get_deltas_for_revisions(revisions))
         deltas2 = [repository.get_revision_delta(r.revision_id) for r in
                    revisions]
-        assert deltas1 == deltas2
+        self.assertEqual(deltas1, deltas2)
 
     def test_all_revision_ids(self):
         # all_revision_ids -> all revisions
@@ -915,24 +927,28 @@ class TestCaseWithComplexRepository(TestCaseWithRepository):
                           'rev3':('rev2', ),
                           'rev4':('rev3', ),
                           },
-                         self.bzrdir.open_repository().get_revision_graph(None))
+            self.applyDeprecated(one_four,
+                self.bzrdir.open_repository().get_revision_graph, None))
         self.assertEqual({'rev1':()},
-                         self.bzrdir.open_repository().get_revision_graph('rev1'))
+            self.applyDeprecated(one_four,
+                self.bzrdir.open_repository().get_revision_graph, 'rev1'))
         self.assertEqual({'rev1':(),
                           'rev2':('rev1', )},
-                         self.bzrdir.open_repository().get_revision_graph('rev2'))
-        self.assertRaises(errors.NoSuchRevision,
-                          self.bzrdir.open_repository().get_revision_graph,
-                          'orphan')
+            self.applyDeprecated(one_four,
+                self.bzrdir.open_repository().get_revision_graph, 'rev2'))
+        self.assertRaises(errors.NoSuchRevision, self.applyDeprecated, one_four,
+            self.bzrdir.open_repository().get_revision_graph, 'orphan')
         # and ghosts are not mentioned
         self.assertEqual({'rev1':(),
                           'rev2':('rev1', ),
                           'rev3':('rev2', ),
                           },
-                         self.bzrdir.open_repository().get_revision_graph('rev3'))
+            self.applyDeprecated(one_four,
+                self.bzrdir.open_repository().get_revision_graph, 'rev3'))
         # and we can ask for the NULLREVISION graph
         self.assertEqual({},
-            self.bzrdir.open_repository().get_revision_graph(NULL_REVISION))
+            self.applyDeprecated(one_four,
+                self.bzrdir.open_repository().get_revision_graph, NULL_REVISION))
 
     def test_get_revision_graph_with_ghosts(self):
         # we can get a graph object with roots, ghosts, ancestors and
