@@ -37,6 +37,10 @@ class TestAll(TestCaseWithTransport):
         super(TestAll, self).setUp()
         self.store = self.store_factory.create(self.get_url('.'))
         self.transaction = PassThroughTransaction()
+        self.store.get_scope = self.get_transaction
+
+    def get_transaction(self):
+        return self.transaction
 
     def test_add_has_get(self):
         rev = self.add_sample_rev()
@@ -58,17 +62,24 @@ class TestAll(TestCaseWithTransport):
         # has of a non present id -> False
         self.assertFalse(self.store.has_revision_id('missing', self.transaction))
 
-    def test_has_None(self):
-        # has of None -> True
+    def test_has_null(self):
+        # has of null -> True
         self.assertTrue(self.store.has_revision_id('null:', self.transaction))
 
-    def test_get_revision_none(self):
-        # get_revision(None) -> raises NoSuchRevision
+    def test_get_revision_missing(self):
+        # get_revision('B') -> raises NoSuchRevision
         self.assertRaises(errors.NoSuchRevision,
                           self.store.get_revision,
                           'B',
                           self.transaction)
 
+    def test_get_revision_null(self):
+        # get_revision(null) -> raises ReservedId
+        self.assertRaises(errors.ReservedId,
+                          self.store.get_revision,
+                          'null:',
+                          self.transaction)
+
     def test_add_signature_text_missing(self):
         # add of a text signature for a missing revision must work, to allow
         # revisions to be added after the signature.
