@@ -28,8 +28,10 @@ from bzrlib import (
     treebuilder,
     )
 from bzrlib.bzrdir import BzrDir
+from bzrlib.bundle import read_mergeable_from_url
 from bzrlib.bundle.apply_bundle import install_bundle, merge_bundle
 from bzrlib.bundle.bundle_data import BundleTree
+from bzrlib.directory_service import directories
 from bzrlib.bundle.serializer import write_bundle, read_bundle, v09, v4
 from bzrlib.bundle.serializer.v08 import BundleSerializerV08
 from bzrlib.bundle.serializer.v09 import BundleSerializerV09
@@ -40,11 +42,17 @@ from bzrlib.errors import (BzrError, TestamentMismatch, NotABundle, BadBundle,
                            NoSuchFile,)
 from bzrlib.merge import Merge3Merger
 from bzrlib.repofmt import knitrepo
-from bzrlib.osutils import has_symlinks, sha_file
-from bzrlib.tests import (TestCaseInTempDir, TestCaseWithTransport,
-                          TestCase, TestSkipped, test_commit)
+from bzrlib.osutils import sha_file, sha_string
+from bzrlib.tests import (
+    SymlinkFeature,
+    TestCase,
+    TestCaseInTempDir,
+    TestCaseWithTransport,
+    TestSkipped,
+    test_read_bundle,
+    test_commit,
+    )
 from bzrlib.transform import TreeTransform
-from bzrlib.workingtree import WorkingTree
 
 
 class MockTree(object):
@@ -161,14 +169,14 @@ class BTreeTester(TestCase):
         self.assertEqual(btree.path2id("grandparent/parent"), "b")
         self.assertEqual(btree.path2id("grandparent/parent/file"), "c")
 
-        assert btree.path2id("grandparent2") is None
-        assert btree.path2id("grandparent2/parent") is None
-        assert btree.path2id("grandparent2/parent/file") is None
+        self.assertTrue(btree.path2id("grandparent2") is None)
+        self.assertTrue(btree.path2id("grandparent2/parent") is None)
+        self.assertTrue(btree.path2id("grandparent2/parent/file") is None)
 
         btree.note_rename("grandparent", "grandparent2")
-        assert btree.old_path("grandparent") is None
-        assert btree.old_path("grandparent/parent") is None
-        assert btree.old_path("grandparent/parent/file") is None
+        self.assertTrue(btree.old_path("grandparent") is None)
+        self.assertTrue(btree.old_path("grandparent/parent") is None)
+        self.assertTrue(btree.old_path("grandparent/parent/file") is None)
 
         self.assertEqual(btree.id2path("a"), "grandparent2")
         self.assertEqual(btree.id2path("b"), "grandparent2/parent")
@@ -178,9 +186,9 @@ class BTreeTester(TestCase):
         self.assertEqual(btree.path2id("grandparent2/parent"), "b")
         self.assertEqual(btree.path2id("grandparent2/parent/file"), "c")
 
-        assert btree.path2id("grandparent") is None
-        assert btree.path2id("grandparent/parent") is None
-        assert btree.path2id("grandparent/parent/file") is None
+        self.assertTrue(btree.path2id("grandparent") is None)
+        self.assertTrue(btree.path2id("grandparent/parent") is None)
+        self.assertTrue(btree.path2id("grandparent/parent/file") is None)
 
         btree.note_rename("grandparent/parent", "grandparent2/parent2")
         self.assertEqual(btree.id2path("a"), "grandparent2")
@@ -191,8 +199,8 @@ class BTreeTester(TestCase):
         self.assertEqual(btree.path2id("grandparent2/parent2"), "b")
         self.assertEqual(btree.path2id("grandparent2/parent2/file"), "c")
 
-        assert btree.path2id("grandparent2/parent") is None
-        assert btree.path2id("grandparent2/parent/file") is None
+        self.assertTrue(btree.path2id("grandparent2/parent") is None)
+        self.assertTrue(btree.path2id("grandparent2/parent/file") is None)
 
         btree.note_rename("grandparent/parent/file", 
                           "grandparent2/parent2/file2")
@@ -204,7 +212,7 @@ class BTreeTester(TestCase):
         self.assertEqual(btree.path2id("grandparent2/parent2"), "b")
         self.assertEqual(btree.path2id("grandparent2/parent2/file2"), "c")
 
-        assert btree.path2id("grandparent2/parent2/file") is None
+        self.assertTrue(btree.path2id("grandparent2/parent2/file") is None)
 
     def test_moves(self):
         """Ensure that file moves have the proper effect on children"""
@@ -213,7 +221,7 @@ class BTreeTester(TestCase):
                           "grandparent/alt_parent/file")
         self.assertEqual(btree.id2path("c"), "grandparent/alt_parent/file")
         self.assertEqual(btree.path2id("grandparent/alt_parent/file"), "c")
-        assert btree.path2id("grandparent/parent/file") is None
+        self.assertTrue(btree.path2id("grandparent/parent/file") is None)
 
     def unified_diff(self, old, new):
         out = StringIO()
@@ -225,8 +233,8 @@ class BTreeTester(TestCase):
         btree = self.make_tree_1()[0]
         btree.note_rename("grandparent/parent/file", 
                           "grandparent/alt_parent/file")
-        assert btree.id2path("e") is None
-        assert btree.path2id("grandparent/parent/file") is None
+        self.assertTrue(btree.id2path("e") is None)
+        self.assertTrue(btree.path2id("grandparent/parent/file") is None)
         btree.note_id("e", "grandparent/parent/file")
         return btree
 
@@ -290,8 +298,8 @@ class BTreeTester(TestCase):
         btree = self.make_tree_1()[0]
         self.assertEqual(btree.get_file("c").read(), "Hello\n")
         btree.note_deletion("grandparent/parent/file")
-        assert btree.id2path("c") is None
-        assert btree.path2id("grandparent/parent/file") is None
+        self.assertTrue(btree.id2path("c") is None)
+        self.assertTrue(btree.path2id("grandparent/parent/file") is None)
 
     def sorted_ids(self, tree):
         ids = list(tree)
@@ -456,8 +464,7 @@ class BundleTester(object):
         ancestors = write_bundle(self.b1.repository, rev_id, 'null:', s,
                                  format=self.format)
         s.seek(0)
-        assert isinstance(s.getvalue(), str), (
-            "Bundle isn't a bytestring:\n %s..." % repr(s.getvalue())[:40])
+        self.assertIsInstance(s.getvalue(), str)
         install_bundle(tree.branch.repository, read_bundle(s))
         for ancestor in ancestors:
             old = self.b1.repository.revision_tree(ancestor)
@@ -493,6 +500,13 @@ class BundleTester(object):
         sure everything matches the builtin branch.
         """
         to_tree = self.get_checkout(base_rev_id, checkout_dir=checkout_dir)
+        to_tree.lock_write()
+        try:
+            self._valid_apply_bundle(base_rev_id, info, to_tree)
+        finally:
+            to_tree.unlock()
+
+    def _valid_apply_bundle(self, base_rev_id, info, to_tree):
         original_parents = to_tree.get_parent_ids()
         repository = to_tree.branch.repository
         original_parents = to_tree.get_parent_ids()
@@ -642,8 +656,7 @@ class BundleTester(object):
         bundle = self.get_valid_bundle('a@cset-0-6', 'a@cset-0-7')
 
     def test_symlink_bundle(self):
-        if not has_symlinks():
-            raise TestSkipped("No symlink support")
+        self.requireFeature(SymlinkFeature)
         self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
         tt = TreeTransform(self.tree1)
@@ -795,6 +808,10 @@ class BundleTester(object):
                           rev_id='i18n-1', committer=u'William Dod\xe9')
 
         if sys.platform == 'darwin':
+            from bzrlib.workingtree import WorkingTree3
+            if type(self.tree1) is WorkingTree3:
+                self.knownFailure("Bug #141438: fails for WorkingTree3 on OSX")
+
             # On Mac the '\xe9' gets changed to 'e\u0301'
             self.assertEqual([u'.bzr', u'with Dode\u0301'],
                              sorted(os.listdir(u'b1')))
@@ -997,19 +1014,29 @@ class BundleTester(object):
         self.assertNotContainsRe(inv_text, 'format="5"')
         self.assertContainsRe(inv_text, 'format="7"')
 
-    def test_across_models(self):
+    def make_repo_with_installed_revisions(self):
         tree = self.make_simple_tree('knit')
         tree.commit('hello', rev_id='rev1')
         tree.commit('hello', rev_id='rev2')
         bundle = read_bundle(self.create_bundle_text('null:', 'rev2')[0])
         repo = self.make_repository('repo', format='dirstate-with-subtree')
         bundle.install_revisions(repo)
+        return repo
+
+    def test_across_models(self):
+        repo = self.make_repo_with_installed_revisions()
         inv = repo.get_inventory('rev2')
         self.assertEqual('rev2', inv.root.revision)
         root_vf = repo.weave_store.get_weave(inv.root.file_id,
                                              repo.get_transaction())
         self.assertEqual(root_vf.versions(), ['rev1', 'rev2'])
 
+    def test_inv_hash_across_serializers(self):
+        repo = self.make_repo_with_installed_revisions()
+        recorded_inv_sha1 = repo.get_inventory_sha1('rev2')
+        xml = repo.get_inventory_xml('rev2')
+        self.assertEqual(sha_string(xml), recorded_inv_sha1)
+
     def test_across_models_incompatible(self):
         tree = self.make_simple_tree('dirstate-with-subtree')
         tree.commit('hello', rev_id='rev1')
@@ -1063,6 +1090,33 @@ class BundleTester(object):
                               " slashes")
         bundle = self.get_valid_bundle('null:', 'rev/id')
 
+    def test_skip_file(self):
+        """Make sure we don't accidentally write to the wrong versionedfile"""
+        self.tree1 = self.make_branch_and_tree('tree')
+        self.b1 = self.tree1.branch
+        # rev1 is not present in bundle, done by fetch
+        self.build_tree_contents([('tree/file2', 'contents1')])
+        self.tree1.add('file2', 'file2-id')
+        self.tree1.commit('rev1', rev_id='reva')
+        self.build_tree_contents([('tree/file3', 'contents2')])
+        # rev2 is present in bundle, and done by fetch
+        # having file1 in the bunle causes file1's versionedfile to be opened.
+        self.tree1.add('file3', 'file3-id')
+        self.tree1.commit('rev2')
+        # Updating file2 should not cause an attempt to add to file1's vf
+        target = self.tree1.bzrdir.sprout('target').open_workingtree()
+        self.build_tree_contents([('tree/file2', 'contents3')])
+        self.tree1.commit('rev3', rev_id='rev3')
+        bundle = self.get_valid_bundle('reva', 'rev3')
+        if getattr(bundle, 'get_bundle_reader', None) is None:
+            raise TestSkipped('Bundle format cannot provide reader')
+        # be sure that file1 comes before file2
+        for b, m, k, r, f in bundle.get_bundle_reader().iter_records():
+            if f == 'file3-id':
+                break
+            self.assertNotEqual(f, 'file2-id')
+        bundle.install_revisions(target.branch.repository)
+
 
 class V08BundleTester(BundleTester, TestCaseWithTransport):
 
@@ -1250,7 +1304,7 @@ class V4BundleTester(BundleTester, TestCaseWithTransport):
         new_text = self.get_raw(StringIO(''.join(bundle_txt)))
         new_text = new_text.replace('<file file_id="exe-1"',
                                     '<file executable="y" file_id="exe-1"')
-        new_text = new_text.replace('B372', 'B387')
+        new_text = new_text.replace('B222', 'B237')
         bundle_txt = StringIO()
         bundle_txt.write(serializer._get_bundle_header('4'))
         bundle_txt.write('\n')
@@ -1297,7 +1351,8 @@ class V4BundleTester(BundleTester, TestCaseWithTransport):
         self.assertEqual('contents2\nstatic\n', vf.get_text('rev2'))
         rtree = target_repo.revision_tree('rev2')
         inventory_vf = target_repo.get_inventory_weave()
-        self.assertEqual(['rev1'], inventory_vf.get_parents('rev2'))
+        self.assertEqual({'rev2':('rev1',)},
+            inventory_vf.get_parent_map(['rev2']))
         self.assertEqual('changed file',
                          target_repo.get_revision('rev2').message)
 
@@ -1459,7 +1514,27 @@ class TestBundleWriterReader(TestCase):
             'storage_kind':'fulltext'}, 'file', 'revid', 'fileid')
         writer.end()
         fileobj.seek(0)
-        record_iter = v4.BundleReader(fileobj).iter_records()
+        reader = v4.BundleReader(fileobj, stream_input=True)
+        record_iter = reader.iter_records()
+        record = record_iter.next()
+        self.assertEqual((None, {'foo': 'bar', 'storage_kind': 'header'},
+            'info', None, None), record)
+        record = record_iter.next()
+        self.assertEqual(("Record body", {'storage_kind': 'fulltext',
+                          'parents': ['1', '3']}, 'file', 'revid', 'fileid'),
+                          record)
+
+    def test_roundtrip_record_memory_hungry(self):
+        fileobj = StringIO()
+        writer = v4.BundleWriter(fileobj)
+        writer.begin()
+        writer.add_info_record(foo='bar')
+        writer._add_record("Record body", {'parents': ['1', '3'],
+            'storage_kind':'fulltext'}, 'file', 'revid', 'fileid')
+        writer.end()
+        fileobj.seek(0)
+        reader = v4.BundleReader(fileobj, stream_input=False)
+        record_iter = reader.iter_records()
         record = record_iter.next()
         self.assertEqual((None, {'foo': 'bar', 'storage_kind': 'header'},
             'info', None, None), record)
@@ -1497,3 +1572,21 @@ class TestBundleWriterReader(TestCase):
         self.assertEqual((None, {'foo': 'bar', 'storage_kind': 'header'},
             'info', None, None), record)
         self.assertRaises(BadBundle, record_iter.next)
+
+
+class TestReadMergeableFromUrl(TestCaseWithTransport):
+
+    def test_read_mergeable_skips_local(self):
+        """A local bundle named like the URL should not be read.
+        """
+        out, wt = test_read_bundle.create_bundle_file(self)
+        class FooService(object):
+            """A directory service that always returns source"""
+
+            def look_up(self, name, url):
+                return 'source'
+        directories.register('foo:', FooService, 'Testing directory service')
+        self.addCleanup(lambda: directories.remove('foo:'))
+        self.build_tree_contents([('./foo:bar', out.getvalue())])
+        self.assertRaises(errors.NotABundle, read_mergeable_from_url,
+                          'foo:bar')
