@@ -39,9 +39,6 @@ from bzrlib.errors import (NotBranchError,
                            UnknownFormatError,
                            UnsupportedFormatError,
                            )
-from bzrlib.symbol_versioning import (
-    zero_ninetyone,
-    )
 from bzrlib.tests import (
     TestCase,
     TestCaseWithTransport,
@@ -66,16 +63,13 @@ class TestDefaultFormat(TestCase):
         old_format = bzrdir.BzrDirFormat.get_default_format()
         # default is BzrDirFormat6
         self.failUnless(isinstance(old_format, bzrdir.BzrDirMetaFormat1))
-        self.applyDeprecated(symbol_versioning.zero_fourteen, 
-                             bzrdir.BzrDirFormat.set_default_format, 
-                             SampleBzrDirFormat())
+        bzrdir.BzrDirFormat._set_default_format(SampleBzrDirFormat())
         # creating a bzr dir should now create an instrumented dir.
         try:
             result = bzrdir.BzrDir.create('memory:///')
             self.failUnless(isinstance(result, SampleBzrDir))
         finally:
-            self.applyDeprecated(symbol_versioning.zero_fourteen,
-                bzrdir.BzrDirFormat.set_default_format, old_format)
+            bzrdir.BzrDirFormat._set_default_format(old_format)
         self.assertEqual(old_format, bzrdir.BzrDirFormat.get_default_format())
 
 
@@ -281,43 +275,6 @@ class TestBzrDirFormat(TestCaseWithTransport):
         # now open_downlevel should fail too.
         self.assertRaises(UnknownFormatError, bzrdir.BzrDir.open_unsupported, url)
 
-    def test_create_repository_deprecated(self):
-        # new interface is to make the bzrdir, then a repository within that.
-        format = SampleBzrDirFormat()
-        repo = self.applyDeprecated(zero_ninetyone,
-                bzrdir.BzrDir.create_repository,
-                self.get_url(), format=format)
-        self.assertEqual('A repository', repo)
-
-    def test_create_repository_shared(self):
-        # new interface is to make the bzrdir, then a repository within that.
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        repo = self.applyDeprecated(zero_ninetyone,
-                bzrdir.BzrDir.create_repository,
-                '.', shared=True)
-        self.assertTrue(repo.is_shared())
-
-    def test_create_repository_nonshared(self):
-        # new interface is to make the bzrdir, then a repository within that.
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        repo = self.applyDeprecated(zero_ninetyone,
-                bzrdir.BzrDir.create_repository,
-                '.')
-        self.assertFalse(repo.is_shared())
-
-    def test_create_repository_under_shared(self):
-        # an explicit create_repository always does so.
-        # we trust the format is right from the 'create_repository test'
-        # new interface is to make the bzrdir, then a repository within that.
-        format = bzrdir.format_registry.make_bzrdir('knit')
-        self.make_repository('.', shared=True, format=format)
-        repo = self.applyDeprecated(zero_ninetyone,
-                bzrdir.BzrDir.create_repository,
-                self.get_url('child'),
-                format=format)
-        self.assertTrue(isinstance(repo, repository.Repository))
-        self.assertTrue(repo.bzrdir.root_transport.base.endswith('child/'))
-
     def test_create_branch_and_repo_uses_default(self):
         format = SampleBzrDirFormat()
         branch = bzrdir.BzrDir.create_branch_and_repo(self.get_url(),
@@ -454,6 +411,18 @@ class TestBzrDirFormat(TestCaseWithTransport):
         branch.bzrdir.open_workingtree()
 
 
+class TestRepositoryAcquisitionPolicy(TestCaseWithTransport):
+
+    def test_acquire_repository_standalone(self):
+        """The default acquisition policy should create a standalone branch."""
+        my_bzrdir = self.make_bzrdir('.')
+        repo_policy = my_bzrdir.determine_repository_policy()
+        repo = repo_policy.acquire_repository()
+        self.assertEqual(repo.bzrdir.root_transport.base,
+                         my_bzrdir.root_transport.base)
+        self.assertFalse(repo.is_shared())
+
+
 class ChrootedTests(TestCaseWithTransport):
     """A support class that provides readonly urls outside the local namespace.
 
