@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@ import os
 import re
 import sys
 
+import bzrlib
 from bzrlib import (
     bzrdir,
     errors,
@@ -26,15 +27,18 @@ from bzrlib import (
     )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
-import bzrlib.errors
 from bzrlib.repofmt import knitrepo
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
+from bzrlib.tests.http_utils import TestCaseWithWebserver
 from bzrlib.tests.test_revision import make_branches
 from bzrlib.trace import mutter
 from bzrlib.upgrade import Convert
 from bzrlib.workingtree import WorkingTree
 
+# These tests are a bit old; please instead add new tests into
+# interrepository_implementations/ so they'll run on all relevant
+# combinations.
+
 
 def has_revision(branch, revision_id):
     return branch.repository.has_revision(revision_id)
@@ -94,18 +98,13 @@ def fetch_steps(self, br_a, br_b, writable_a):
     self.assertEquals(fetched, 3, "fetched %d instead of 3" % fetched)
     # InstallFailed should be raised if the branch is missing the revision
     # that was requested.
-    self.assertRaises(bzrlib.errors.InstallFailed, br_a3.fetch, br_a2, 'pizza')
-    # InstallFailed should be raised if the branch is missing a revision
-    # from its own revision history
-    br_a2.append_revision('a-b-c')
-    self.assertRaises(bzrlib.errors.InstallFailed, br_a3.fetch, br_a2)
+    self.assertRaises(errors.InstallFailed, br_a3.fetch, br_a2, 'pizza')
 
-    # TODO: ADHB 20070116 Perhaps set_last_revision shouldn't accept
-    #       revisions which are not present?  In that case, this test
-    #       must be rewritten.
-    #
-    #       RBC 20060403 the way to do this is to uncommit the revision from
-    #       the repository after the commit
+    # TODO: Test trying to fetch from a branch that points to a revision not
+    # actually present in its repository.  Not every branch format allows you
+    # to directly point to such revisions, so it's a bit complicated to
+    # construct.  One way would be to uncommit and gc the revision, but not
+    # every branch supports that.  -- mbp 20070814
 
     #TODO: test that fetch correctly does reweaving when needed. RBC 20051008
     # Note that this means - updating the weave when ghosts are filled in to 
@@ -116,7 +115,7 @@ class TestFetch(TestCaseWithTransport):
 
     def test_fetch(self):
         #highest indices a: 5, b: 7
-        br_a, br_b = make_branches(self)
+        br_a, br_b = make_branches(self, format='dirstate-tags')
         fetch_steps(self, br_a, br_b, br_a)
 
     def test_fetch_self(self):
@@ -229,6 +228,8 @@ class TestMergeFileHistory(TestCaseWithTransport):
         br2 = Branch.open('br2')
         br1 = Branch.open('br1')
         wt2 = WorkingTree.open('br2').merge_from_branch(br1)
+        br2.lock_read()
+        self.addCleanup(br2.unlock)
         for rev_id, text in [('1-2', 'original from 1\n'),
                              ('1-3', 'agreement\n'),
                              ('2-1', 'contents in 2\n'),
@@ -262,11 +263,12 @@ class TestHttpFetch(TestCaseWithWebserver):
 
     def test_weaves_are_retrieved_once(self):
         self.build_tree(("source/", "source/file", "target/"))
-        wt = self.make_branch_and_tree('source')
+        # This test depends on knit dasta storage.
+        wt = self.make_branch_and_tree('source', format='dirstate-tags')
         branch = wt.branch
         wt.add(["file"], ["id"])
         wt.commit("added file")
-        print >>open("source/file", 'w'), "blah"
+        open("source/file", 'w').write("blah\n")
         wt.commit("changed file")
         target = BzrDir.create_branch_and_repo("target/")
         source = Branch.open(self.get_readonly_url("source/"))
@@ -291,8 +293,14 @@ class TestHttpFetch(TestCaseWithWebserver):
                                                      http_logs))
         # FIXME naughty poking in there.
         self.get_readonly_server().logs = []
-        # check there is nothing more to fetch
-        source = Branch.open(self.get_readonly_url("source/"))
+        # check there is nothing more to fetch.  We take care to re-use the
+        # existing transport so that the request logs we're about to examine
+        # aren't cluttered with redundant probes for a smart server.
+        # XXX: Perhaps this further parameterisation: test http with smart
+        # server, and test http without smart server?
+        source = Branch.open(
+            self.get_readonly_url("source/"),
+            possible_transports=[source.bzrdir.root_transport])
         self.assertEqual(target.fetch(source), (0, []))
         # should make just two requests
         http_logs = self.get_readonly_server().logs
@@ -306,3 +314,75 @@ class TestHttpFetch(TestCaseWithWebserver):
         self.assertTrue(1 >= self._count_log_matches('last-revision',
                                                      http_logs))
         self.assertEqual(4, len(http_logs))
+
+
+class Test1To2Fetch(TestCaseWithTransport):
+    """Tests for Model1To2 failure modes"""
+
+    def make_tree_and_repo(self):
+        self.tree = self.make_branch_and_tree('tree', format='pack-0.92')
+        self.repo = self.make_repository('rich-repo', format='rich-root-pack')
+        self.repo.lock_write()
+        self.addCleanup(self.repo.unlock)
+
+    def do_fetch_order_test(self, first, second):
+        """Test that fetch works no matter what the set order of revision is.
+
+        This test depends on the order of items in a set, which is
+        implementation-dependant, so we test A, B and then B, A.
+        """
+        self.make_tree_and_repo()
+        self.tree.commit('Commit 1', rev_id=first)
+        self.tree.commit('Commit 2', rev_id=second)
+        self.repo.fetch(self.tree.branch.repository, second)
+
+    def test_fetch_order_AB(self):
+        """See do_fetch_order_test"""
+        self.do_fetch_order_test('A', 'B')
+
+    def test_fetch_order_BA(self):
+        """See do_fetch_order_test"""
+        self.do_fetch_order_test('B', 'A')
+
+    def get_parents(self, file_id, revision_id):
+        transaction = self.repo.get_transaction()
+        vf = self.repo.weave_store.get_weave(file_id, transaction)
+        return vf.get_parents_with_ghosts(revision_id)
+
+    def test_fetch_ghosts(self):
+        self.make_tree_and_repo()
+        self.tree.commit('first commit', rev_id='left-parent')
+        self.tree.add_parent_tree_id('ghost-parent')
+        fork = self.tree.bzrdir.sprout('fork', 'null:').open_workingtree()
+        fork.commit('not a ghost', rev_id='not-ghost-parent')
+        self.tree.branch.repository.fetch(fork.branch.repository,
+                                     'not-ghost-parent')
+        self.tree.add_parent_tree_id('not-ghost-parent')
+        self.tree.commit('second commit', rev_id='second-id')
+        self.repo.fetch(self.tree.branch.repository, 'second-id')
+        root_id = self.tree.get_root_id()
+        self.assertEqual(['left-parent', 'ghost-parent', 'not-ghost-parent'],
+                         self.get_parents(root_id, 'second-id'))
+
+    def make_two_commits(self, change_root, fetch_twice):
+        self.make_tree_and_repo()
+        self.tree.commit('first commit', rev_id='first-id')
+        if change_root:
+            self.tree.set_root_id('unique-id')
+        self.tree.commit('second commit', rev_id='second-id')
+        if fetch_twice:
+            self.repo.fetch(self.tree.branch.repository, 'first-id')
+        self.repo.fetch(self.tree.branch.repository, 'second-id')
+
+    def test_fetch_changed_root(self):
+        self.make_two_commits(change_root=True, fetch_twice=False)
+        self.assertEqual([], self.get_parents('unique-id', 'second-id'))
+
+    def test_two_fetch_changed_root(self):
+        self.make_two_commits(change_root=True, fetch_twice=True)
+        self.assertEqual([], self.get_parents('unique-id', 'second-id'))
+
+    def test_two_fetches(self):
+        self.make_two_commits(change_root=False, fetch_twice=True)
+        self.assertEqual(['first-id'],
+                         self.get_parents('TREE_ROOT', 'second-id'))
