@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,11 +17,16 @@
 from StringIO import StringIO
 
 import bzrlib
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    lockdir,
+    osutils,
+    )
 from bzrlib.errors import BzrBadParameterNotString, NoSuchFile, ReadOnlyError
 from bzrlib.lockable_files import LockableFiles, TransportLock
-from bzrlib import lockdir
-from bzrlib.lockdir import LockDir
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    )
 from bzrlib.tests import TestCaseInTempDir
 from bzrlib.tests.test_smart import TestCaseWithSmartMedium
 from bzrlib.tests.test_transactions import DummyWeave
@@ -33,11 +38,17 @@ from bzrlib.transport import get_transport
 
 
 # these tests are applied in each parameterized suite for LockableFiles
+#
+# they use an old style of parameterization, but we want to remove this class
+# so won't modernize them now. - mbp 20080430
 class _TestLockableFiles_mixin(object):
 
     def test_read_write(self):
         self.assertRaises(NoSuchFile, self.lockable.get, 'foo')
-        self.assertRaises(NoSuchFile, self.lockable.get_utf8, 'foo')
+        self.assertRaises(NoSuchFile,
+            self.applyDeprecated,
+            deprecated_in((1, 5, 0)),
+            self.lockable.get_utf8, 'foo')
         self.lockable.lock_write()
         try:
             unicode_string = u'bar\u1234'
@@ -49,16 +60,24 @@ class _TestLockableFiles_mixin(object):
             self.lockable.put('foo', StringIO(byte_string))
             self.assertEqual(byte_string,
                              self.lockable.get('foo').read())
+            unicode_stream = self.applyDeprecated(
+                deprecated_in((1, 5, 0)),
+                self.lockable.get_utf8,
+                'foo')
             self.assertEqual(unicode_string,
-                             self.lockable.get_utf8('foo').read())
+                unicode_stream.read())
             self.assertRaises(BzrBadParameterNotString,
                               self.lockable.put_utf8,
                               'bar',
                               StringIO(unicode_string)
                               )
             self.lockable.put_utf8('bar', unicode_string)
+            unicode_stream = self.applyDeprecated(
+                deprecated_in((1, 5, 0)),
+                self.lockable.get_utf8,
+                'bar')
             self.assertEqual(unicode_string,
-                             self.lockable.get_utf8('bar').read())
+                unicode_stream.read())
             self.assertEqual(byte_string,
                              self.lockable.get('bar').read())
             self.lockable.put_bytes('raw', 'raw\xffbytes')
@@ -347,7 +366,7 @@ class TestLockableFiles_LockDir(TestCaseInTempDir,
         self.lockable.create_lock()
 
     def get_lockable(self):
-        return LockableFiles(self.transport, 'my-lock', LockDir)
+        return LockableFiles(self.transport, 'my-lock', lockdir.LockDir)
 
     def test_lock_created(self):
         self.assertTrue(self.transport.has('my-lock'))
@@ -357,10 +376,16 @@ class TestLockableFiles_LockDir(TestCaseInTempDir,
         self.assertFalse(self.transport.has('my-lock/held/info'))
         self.assertTrue(self.transport.has('my-lock'))
 
+    def test__file_modes(self):
+        self.transport.mkdir('readonly')
+        osutils.make_readonly('readonly')
+        lockable = LockableFiles(self.transport.clone('readonly'), 'test-lock',
+                                 lockdir.LockDir)
+        # The directory mode should be read-write-execute for the current user
+        self.assertEqual(00700, lockable._dir_mode & 00700)
+        # Files should be read-write for the current user
+        self.assertEqual(00600, lockable._file_mode & 00700)
 
-    # TODO: Test the lockdir inherits the right file and directory permissions
-    # from the LockableFiles.
-        
 
 class TestLockableFiles_RemoteLockDir(TestCaseWithSmartMedium,
                               _TestLockableFiles_mixin):
