@@ -620,7 +620,7 @@ class TestMergeDirective2Branch(tests.TestCaseWithTransport,
         public_branch=None, message=None, base_revision_id=None):
         include_patch = (patch_type in ('bundle', 'diff'))
         include_bundle = (patch_type == 'bundle')
-        assert patch_type in ('bundle', 'diff', None)
+        self.assertTrue(patch_type in ('bundle', 'diff', None))
         return merge_directive.MergeDirective2.from_objects(
             repository, revision_id, time, timezone, target_branch,
             include_patch, include_bundle, local_target_branch, public_branch,
