@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -47,7 +47,6 @@ from bzrlib.workingtree import WorkingTree
 
 def chmod_r(base, file_mode, dir_mode):
     """Recursively chmod from a base directory"""
-    assert os.path.isdir(base)
     os.chmod(base, dir_mode)
     for root, dirs, files in os.walk(base):
         for d in dirs:
@@ -67,7 +66,6 @@ def check_mode_r(test, base, file_mode, dir_mode, include_base=True):
     :param dir_mode: The mode for all directories
     :param include_base: If false, only check the subdirectories
     """
-    assert os.path.isdir(base)
     t = get_transport(".")
     if include_base:
         test.assertTransportMode(t, base, dir_mode)
@@ -282,7 +280,7 @@ class TestSftpPermissions(TestCaseWithSFTPServer):
             t.put_bytes('b', 'txt', mode=0666)
             self.assertTransportMode(t, 'b', 0666)
 
-            t._sftp.mkdir('c', mode=0777)
+            t._get_sftp().mkdir('c', mode=0777)
             self.assertTransportMode(t, 'c', 0777 &~umask)
 
             t.mkdir('d', mode=0777)
