@@ -24,29 +24,19 @@ from bzrlib.progress import (
         TTYProgressBar,
         DotsProgressBar,
         ProgressBarStack,
+        InstrumentedProgress,
         )
 from bzrlib.tests import TestCase
 
 
 class FakeStack:
+
     def __init__(self, top):
         self.__top = top
 
     def top(self):
         return self.__top
 
-class InstrumentedProgress(TTYProgressBar):
-    """TTYProgress variant that tracks outcomes"""
-
-    def __init__(self, *args, **kwargs):
-        self.always_throttled = True
-        TTYProgressBar.__init__(self, *args, **kwargs)
-
-    def throttle(self, old_message):
-        result = TTYProgressBar.throttle(self, old_message)
-        if result is False:
-            self.always_throttled = False
-        
 
 class _TTYStringIO(StringIO):
     """A helper class which makes a StringIO look like a terminal"""
@@ -63,6 +53,7 @@ class _NonTTYStringIO(StringIO):
 
 
 class TestProgress(TestCase):
+
     def setUp(self):
         q = DummyProgress()
         self.top = ChildProgress(_stack=FakeStack(q))
