@@ -1,4 +1,4 @@
-# Copyright (C) 2006 by Canonical Ltd
+# Copyright (C) 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,8 +17,7 @@
 """Tests for reconiliation behaviour that is repository independent."""
 
 
-import bzrlib
-import bzrlib.errors as errors
+from bzrlib import bzrdir, errors, tests
 from bzrlib.reconcile import reconcile, Reconciler
 from bzrlib.revision import Revision
 from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
@@ -30,10 +29,10 @@ class TestWorksWithSharedRepositories(TestCaseWithRepository):
 
     def test_reweave_empty(self):
         # we want a repo capable format
-        parent = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('.')
+        parent = bzrdir.BzrDirMetaFormat1().initialize('.')
         parent.create_repository(shared=True)
         parent.root_transport.mkdir('child')
-        child = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('child')
+        child = bzrdir.BzrDirMetaFormat1().initialize('child')
         self.assertRaises(errors.NoRepositoryPresent, child.open_repository)
         reconciler = Reconciler(child)
         reconciler.reconcile()
@@ -44,3 +43,27 @@ class TestWorksWithSharedRepositories(TestCaseWithRepository):
         self.assertEqual(0, reconciler.inconsistent_parents)
         # and no garbage inventories
         self.assertEqual(0, reconciler.garbage_inventories)
+
+
+class TestReconciler(tests.TestCaseWithTransport):
+    
+    def test_reconciler_with_no_branch(self):
+        repo = self.make_repository('repo')
+        reconciler = Reconciler(repo.bzrdir)
+        reconciler.reconcile()
+        # no inconsistent parents should have been found
+        # but the values should have been set.
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and no garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
+        self.assertIs(None, reconciler.fixed_branch_history)
+
+    def test_reconciler_finds_branch(self):
+        a_branch = self.make_branch('a_branch')
+        reconciler = Reconciler(a_branch.bzrdir)
+        reconciler.reconcile()
+
+        # It should have checked the repository, and the branch
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        self.assertEqual(0, reconciler.garbage_inventories)
+        self.assertIs(False, reconciler.fixed_branch_history)
