@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,6 +45,8 @@ from bzrlib.remote import (
 from bzrlib.revision import NULL_REVISION
 from bzrlib.smart import server, medium
 from bzrlib.smart.client import _SmartClient
+from bzrlib.symbol_versioning import one_four
+from bzrlib.transport import get_transport
 from bzrlib.transport.memory import MemoryTransport
 from bzrlib.transport.remote import RemoteTransport
 
@@ -55,7 +57,6 @@ class BasicRemoteObjectTests(tests.TestCaseWithTransport):
         self.transport_server = server.SmartTCPServer_for_testing
         super(BasicRemoteObjectTests, self).setUp()
         self.transport = self.get_transport()
-        self.client = self.transport.get_smart_client()
         # make a branch that can be opened over the smart transport
         self.local_wt = BzrDir.create_standalone_workingtree('.')
 
@@ -135,41 +136,42 @@ class FakeClient(_SmartClient):
         """Create a FakeClient.
 
         :param responses: A list of response-tuple, body-data pairs to be sent
-            back to callers.
+            back to callers.  A special case is if the response-tuple is
+            'unknown verb', then a UnknownSmartMethod will be raised for that
+            call, using the second element of the tuple as the verb in the
+            exception.
         """
         self.responses = responses
         self._calls = []
         self.expecting_body = False
-        _SmartClient.__init__(self, FakeMedium(fake_medium_base, self._calls))
+        _SmartClient.__init__(self, FakeMedium(self._calls), fake_medium_base)
+
+    def _get_next_response(self):
+        response_tuple = self.responses.pop(0)
+        if response_tuple[0][0] == 'unknown verb':
+            raise errors.UnknownSmartMethod(response_tuple[0][1])
+        return response_tuple
 
     def call(self, method, *args):
         self._calls.append(('call', method, args))
-        return self.responses.pop(0)[0]
+        return self._get_next_response()[0]
 
     def call_expecting_body(self, method, *args):
         self._calls.append(('call_expecting_body', method, args))
-        result = self.responses.pop(0)
+        result = self._get_next_response()
         self.expecting_body = True
         return result[0], FakeProtocol(result[1], self)
 
     def call_with_body_bytes_expecting_body(self, method, args, body):
         self._calls.append(('call_with_body_bytes_expecting_body', method,
             args, body))
-        result = self.responses.pop(0)
+        result = self._get_next_response()
         self.expecting_body = True
         return result[0], FakeProtocol(result[1], self)
 
 
 class FakeMedium(object):
 
-    def __init__(self, base, client_calls):
-        self.base = base
-        self.connection = FakeConnection(client_calls)
-        self._client_calls = client_calls
-
-
-class FakeConnection(object):
-
     def __init__(self, client_calls):
         self._remote_is_at_least_1_2 = True
         self._client_calls = client_calls
@@ -191,6 +193,39 @@ class TestVfsHas(tests.TestCase):
         self.assertTrue(result)
 
 
+class Test_SmartClient_remote_path_from_transport(tests.TestCase):
+    """Tests for the behaviour of _SmartClient.remote_path_from_transport."""
+
+    def assertRemotePath(self, expected, client_base, transport_base):
+        """Assert that the result of _SmartClient.remote_path_from_transport
+        is the expected value for a given client_base and transport_base.
+        """
+        dummy_medium = 'dummy medium'
+        client = _SmartClient(dummy_medium, client_base)
+        transport = get_transport(transport_base)
+        result = client.remote_path_from_transport(transport)
+        self.assertEqual(expected, result)
+        
+    def test_remote_path_from_transport(self):
+        """_SmartClient.remote_path_from_transport calculates a URL for the
+        given transport relative to the root of the client base URL.
+        """
+        self.assertRemotePath('xyz/', 'bzr://host/path', 'bzr://host/xyz')
+        self.assertRemotePath(
+            'path/xyz/', 'bzr://host/path', 'bzr://host/path/xyz')
+
+    def test_remote_path_from_transport_http(self):
+        """Remote paths for HTTP transports are calculated differently to other
+        transports.  They are just relative to the client base, not the root
+        directory of the host.
+        """
+        for scheme in ['http:', 'https:', 'bzr+http:', 'bzr+https:']:
+            self.assertRemotePath(
+                '../xyz/', scheme + '//host/path', scheme + '//host/xyz')
+            self.assertRemotePath(
+                'xyz/', scheme + '//host/path', scheme + '//host/path/xyz')
+
+
 class TestBzrDirOpenBranch(tests.TestCase):
 
     def test_branch_present(self):
@@ -289,6 +324,24 @@ class TestBzrDirOpenBranch(tests.TestCase):
             RemoteBzrDirFormat.probe_transport, OldServerTransport())
 
 
+class TestBzrDirOpenRepository(tests.TestCase):
+
+    def test_backwards_compat_1_2(self):
+        transport = MemoryTransport()
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        client = FakeClient([
+            (('unknown verb', 'RemoteRepository.find_repositoryV2'), ''),
+            (('ok', '', 'no', 'no'), ''),],
+            transport.base)
+        bzrdir = RemoteBzrDir(transport, _client=client)
+        repo = bzrdir.open_repository()
+        self.assertEqual(
+            [('call', 'BzrDir.find_repositoryV2', ('quack/',)),
+             ('call', 'BzrDir.find_repository', ('quack/',))],
+            client._calls)
+
+
 class OldSmartClient(object):
     """A fake smart client for test_old_version that just returns a version one
     response to the 'hello' (query version) command.
@@ -301,6 +354,9 @@ class OldSmartClient(object):
             input_file, output_file)
         return medium.SmartClientStreamMediumRequest(client_medium)
 
+    def protocol_version(self):
+        return 1
+
 
 class OldServerTransport(object):
     """A fake transport for test_old_server that reports it's smart server
@@ -438,18 +494,127 @@ class TestBranchSetLastRevision(tests.TestCase):
         branch.unlock()
 
 
-class TestBranchControlGetBranchConf(tests.TestCaseWithMemoryTransport):
-    """Test branch.control_files api munging...
+class TestBranchSetLastRevisionInfo(tests.TestCase):
 
-    We special case RemoteBranch.control_files.get('branch.conf') to
-    call a specific API so that RemoteBranch's can intercept configuration
-    file reading, allowing them to signal to the client about things like
-    'email is configured for commits'.
+    def test_set_last_revision_info(self):
+        # set_last_revision_info(num, 'rev-id') is translated to calling
+        # Branch.set_last_revision_info(num, 'rev-id') on the wire.
+        transport = MemoryTransport()
+        transport.mkdir('branch')
+        transport = transport.clone('branch')
+        client = FakeClient([
+            # lock_write
+            (('ok', 'branch token', 'repo token'), ),
+            # set_last_revision_info
+            (('ok',), ),
+            # unlock
+            (('ok',), )], transport.base)
+
+        bzrdir = RemoteBzrDir(transport, _client=False)
+        branch = RemoteBranch(bzrdir, None, _client=client)
+        # This is a hack to work around the problem that RemoteBranch currently
+        # unnecessarily invokes _ensure_real upon a call to lock_write.
+        branch._ensure_real = lambda: None
+        # Lock the branch, reset the record of remote calls.
+        branch.lock_write()
+        client._calls = []
+        result = branch.set_last_revision_info(1234, 'a-revision-id')
+        self.assertEqual(
+            [('call', 'Branch.set_last_revision_info',
+                ('branch/', 'branch token', 'repo token',
+                 '1234', 'a-revision-id'))],
+            client._calls)
+        self.assertEqual(None, result)
+
+    def test_no_such_revision(self):
+        # A response of 'NoSuchRevision' is translated into an exception.
+        client = FakeClient([
+            # lock_write
+            (('ok', 'branch token', 'repo token'), ),
+            # set_last_revision_info
+            (('NoSuchRevision', 'revid'), ),
+            # unlock
+            (('ok',), ),
+            ])
+        transport = MemoryTransport()
+        transport.mkdir('branch')
+        transport = transport.clone('branch')
+
+        bzrdir = RemoteBzrDir(transport, _client=False)
+        branch = RemoteBranch(bzrdir, None, _client=client)
+        # This is a hack to work around the problem that RemoteBranch currently
+        # unnecessarily invokes _ensure_real upon a call to lock_write.
+        branch._ensure_real = lambda: None
+        # Lock the branch, reset the record of remote calls.
+        branch.lock_write()
+        client._calls = []
+
+        self.assertRaises(
+            errors.NoSuchRevision, branch.set_last_revision_info, 123, 'revid')
+        branch.unlock()
+
+    def lock_remote_branch(self, branch):
+        """Trick a RemoteBranch into thinking it is locked."""
+        branch._lock_mode = 'w'
+        branch._lock_count = 2
+        branch._lock_token = 'branch token'
+        branch._repo_lock_token = 'repo token'
+
+    def test_backwards_compatibility(self):
+        """If the server does not support the Branch.set_last_revision_info
+        verb (which is new in 1.4), then the client falls back to VFS methods.
+        """
+        # This test is a little messy.  Unlike most tests in this file, it
+        # doesn't purely test what a Remote* object sends over the wire, and
+        # how it reacts to responses from the wire.  It instead relies partly
+        # on asserting that the RemoteBranch will call
+        # self._real_branch.set_last_revision_info(...).
+
+        # First, set up our RemoteBranch with a FakeClient that raises
+        # UnknownSmartMethod, and a StubRealBranch that logs how it is called.
+        transport = MemoryTransport()
+        transport.mkdir('branch')
+        transport = transport.clone('branch')
+        client = FakeClient(
+            [(('unknown verb', 'Branch.set_last_revision_info',), ),],
+            transport.base)
+        bzrdir = RemoteBzrDir(transport, _client=False)
+        branch = RemoteBranch(bzrdir, None, _client=client)
+        class StubRealBranch(object):
+            def __init__(self):
+                self.calls = []
+            def set_last_revision_info(self, revno, revision_id):
+                self.calls.append(
+                    ('set_last_revision_info', revno, revision_id))
+        real_branch = StubRealBranch()
+        branch._real_branch = real_branch
+        self.lock_remote_branch(branch)
+
+        # Call set_last_revision_info, and verify it behaved as expected.
+        result = branch.set_last_revision_info(1234, 'a-revision-id')
+        self.assertEqual(
+            [('call', 'Branch.set_last_revision_info',
+                ('branch/', 'branch token', 'repo token',
+                 '1234', 'a-revision-id')),],
+            client._calls)
+        self.assertEqual(
+            [('set_last_revision_info', 1234, 'a-revision-id')],
+            real_branch.calls)
+
+
+class TestBranchControlGetBranchConf(tests.TestCaseWithMemoryTransport):
+    """Getting the branch configuration should use an abstract method not vfs.
     """
 
     def test_get_branch_conf(self):
+        raise tests.KnownFailure('branch.conf is not retrieved by get_config_file')
+        # We should see that branch.get_config() does a single rpc to get the
+        # remote configuration file, abstracting away where that is stored on
+        # the server.  However at the moment it always falls back to using the
+        # vfs, and this would need some changes in config.py.
+
         # in an empty branch we decode the response properly
-        client = FakeClient([(('ok', ), 'config file body')], self.get_url())
+        client = FakeClient([(('ok', ), '# config file body')], self.get_url())
         # we need to make a real branch because the remote_branch.control_files
         # will trigger _ensure_real.
         branch = self.make_branch('quack')
@@ -457,11 +622,10 @@ class TestBranchControlGetBranchConf(tests.TestCaseWithMemoryTransport):
         # we do not want bzrdir to make any remote calls
         bzrdir = RemoteBzrDir(transport, _client=False)
         branch = RemoteBranch(bzrdir, None, _client=client)
-        result = branch.control_files.get('branch.conf')
+        config = branch.get_config()
         self.assertEqual(
             [('call_expecting_body', 'Branch.get_config_file', ('quack/',))],
             client._calls)
-        self.assertEqual('config file body', result.read())
 
 
 class TestBranchLockWrite(tests.TestCase):
@@ -507,23 +671,7 @@ class TestTransportIsReadonly(tests.TestCase):
         advisory anyway (a transport could be read-write, but then the
         underlying filesystem could be readonly anyway).
         """
-        client = FakeClient([(
-            ('error', "Generic bzr smart protocol error: "
-                      "bad request 'Transport.is_readonly'"), '')])
-        transport = RemoteTransport('bzr://example.com/', medium=False,
-                                    _client=client)
-        self.assertEqual(False, transport.is_readonly())
-        self.assertEqual(
-            [('call', 'Transport.is_readonly', ())],
-            client._calls)
-
-    def test_error_from_old_0_11_server(self):
-        """Same as test_error_from_old_server, but with the slightly different
-        error message from bzr 0.11 servers.
-        """
-        client = FakeClient([(
-            ('error', "Generic bzr smart protocol error: "
-                      "bad request u'Transport.is_readonly'"), '')])
+        client = FakeClient([(('unknown verb', 'Transport.is_readonly'), '')])
         transport = RemoteTransport('bzr://example.com/', medium=False,
                                     _client=client)
         self.assertEqual(False, transport.is_readonly())
@@ -675,17 +823,19 @@ class TestRepositoryGetParentMap(TestRemoteRepository):
         repo.unlock()
 
     def test_get_parent_map_reconnects_if_unknown_method(self):
-        error_msg = (
-            "Generic bzr smart protocol error: "
-            "bad request 'Repository.get_parent_map'")
         responses = [
-            (('error', error_msg), ''),
+            (('unknown verb', 'Repository.get_parent_map'), ''),
             (('ok',), '')]
         transport_path = 'quack'
         repo, client = self.setup_fake_client_and_repository(
             responses, transport_path)
+        self.assertTrue(client._medium._remote_is_at_least_1_2)
         rev_id = 'revision-id'
-        parents = repo.get_parent_map([rev_id])
+        expected_deprecations = [
+            'bzrlib.remote.RemoteRepository.get_revision_graph was deprecated '
+            'in version 1.4.']
+        parents = self.callDeprecated(
+            expected_deprecations, repo.get_parent_map, [rev_id])
         self.assertEqual(
             [('call_with_body_bytes_expecting_body',
               'Repository.get_parent_map', ('quack/', rev_id), '\n\n0'),
@@ -693,7 +843,44 @@ class TestRepositoryGetParentMap(TestRemoteRepository):
              ('call_expecting_body', 'Repository.get_revision_graph',
               ('quack/', ''))],
             client._calls)
+        # The medium is now marked as being connected to an older server
+        self.assertFalse(client._medium._remote_is_at_least_1_2)
 
+    def test_get_parent_map_fallback_parentless_node(self):
+        """get_parent_map falls back to get_revision_graph on old servers.  The
+        results from get_revision_graph are tweaked to match the get_parent_map
+        API.
+
+        Specifically, a {key: ()} result from get_revision_graph means "no
+        parents" for that key, which in get_parent_map results should be
+        represented as {key: ('null:',)}.
+
+        This is the test for https://bugs.launchpad.net/bzr/+bug/214894
+        """
+        rev_id = 'revision-id'
+        responses = [(('ok',), rev_id)]
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(
+            responses, transport_path)
+        client._medium._remote_is_at_least_1_2 = False
+        expected_deprecations = [
+            'bzrlib.remote.RemoteRepository.get_revision_graph was deprecated '
+            'in version 1.4.']
+        parents = self.callDeprecated(
+            expected_deprecations, repo.get_parent_map, [rev_id])
+        self.assertEqual(
+            [('call_expecting_body', 'Repository.get_revision_graph',
+             ('quack/', ''))],
+            client._calls)
+        self.assertEqual({rev_id: ('null:',)}, parents)
+
+    def test_get_parent_map_unexpected_response(self):
+        responses = [
+            (('something unexpected!',), '')]
+        repo, client = self.setup_fake_client_and_repository(responses, 'path')
+        self.assertRaises(
+            errors.UnexpectedSmartServerResponse,
+            repo.get_parent_map, ['a-revision-id'])
 
 
 class TestRepositoryGetRevisionGraph(TestRemoteRepository):
@@ -705,7 +892,8 @@ class TestRepositoryGetRevisionGraph(TestRemoteRepository):
         transport_path = 'empty'
         repo, client = self.setup_fake_client_and_repository(
             responses, transport_path)
-        result = repo.get_revision_graph(NULL_REVISION)
+        result = self.applyDeprecated(one_four, repo.get_revision_graph,
+            NULL_REVISION)
         self.assertEqual([], client._calls)
         self.assertEqual({}, result)
 
@@ -720,7 +908,7 @@ class TestRepositoryGetRevisionGraph(TestRemoteRepository):
         transport_path = 'sinhala'
         repo, client = self.setup_fake_client_and_repository(
             responses, transport_path)
-        result = repo.get_revision_graph()
+        result = self.applyDeprecated(one_four, repo.get_revision_graph)
         self.assertEqual(
             [('call_expecting_body', 'Repository.get_revision_graph',
              ('sinhala/', ''))],
@@ -740,7 +928,7 @@ class TestRepositoryGetRevisionGraph(TestRemoteRepository):
         transport_path = 'sinhala'
         repo, client = self.setup_fake_client_and_repository(
             responses, transport_path)
-        result = repo.get_revision_graph(r2)
+        result = self.applyDeprecated(one_four, repo.get_revision_graph, r2)
         self.assertEqual(
             [('call_expecting_body', 'Repository.get_revision_graph',
              ('sinhala/', r2))],
@@ -755,7 +943,7 @@ class TestRepositoryGetRevisionGraph(TestRemoteRepository):
             responses, transport_path)
         # also check that the right revision is reported in the error
         self.assertRaises(errors.NoSuchRevision,
-            repo.get_revision_graph, revid)
+            self.applyDeprecated, one_four, repo.get_revision_graph, revid)
         self.assertEqual(
             [('call_expecting_body', 'Repository.get_revision_graph',
              ('sinhala/', revid))],
@@ -959,11 +1147,8 @@ class TestRepositoryStreamKnitData(TestRemoteRepository):
     
     def test_backwards_compatibility(self):
         """If the server doesn't recognise this request, fallback to VFS."""
-        error_msg = (
-            "Generic bzr smart protocol error: "
-            "bad request 'Repository.stream_revisions_chunked'")
         responses = [
-            (('error', error_msg), '')]
+            (('unknown verb', 'Repository.stream_revisions_chunked'), '')]
         repo, client = self.setup_fake_client_and_repository(
             responses, 'path')
         self.mock_called = False
