@@ -25,24 +25,33 @@ Tests for low-level protocol encoding are found in test_smart_transport.
 """
 
 import bz2
-from StringIO import StringIO
-import tempfile
+from cStringIO import StringIO
 import tarfile
 
-from bzrlib import bzrdir, errors, pack, smart, tests
+from bzrlib import (
+    bzrdir,
+    errors,
+    pack,
+    smart,
+    tests,
+    urlutils,
+    )
+from bzrlib.branch import BranchReferenceFormat
+import bzrlib.smart.branch
+import bzrlib.smart.bzrdir
+import bzrlib.smart.repository
 from bzrlib.smart.request import (
     FailedSmartServerResponse,
+    SmartServerRequest,
     SmartServerResponse,
     SuccessfulSmartServerResponse,
     )
-import bzrlib.smart.bzrdir
-import bzrlib.smart.branch
-import bzrlib.smart.repository
 from bzrlib.tests import (
     iter_suite_tests,
     split_suite_by_re,
     TestScenarioApplier,
     )
+from bzrlib.transport import chroot, get_transport
 from bzrlib.util import bencode
 
 
@@ -69,6 +78,24 @@ def load_tests(standard_tests, module, loader):
     return result
 
 
+class TestCaseWithChrootedTransport(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        tests.TestCaseWithTransport.setUp(self)
+        self._chroot_server = None
+
+    def get_transport(self, relpath=None):
+        if self._chroot_server is None:
+            backing_transport = tests.TestCaseWithTransport.get_transport(self)
+            self._chroot_server = chroot.ChrootServer(backing_transport)
+            self._chroot_server.setUp()
+            self.addCleanup(self._chroot_server.tearDown)
+        t = get_transport(self._chroot_server.get_url())
+        if relpath is not None:
+            t = t.clone(relpath)
+        return t
+
+
 class TestCaseWithSmartMedium(tests.TestCaseWithTransport):
 
     def setUp(self):
@@ -77,7 +104,10 @@ class TestCaseWithSmartMedium(tests.TestCaseWithTransport):
         # the default or a parameterized class, but rather use the
         # TestCaseWithTransport infrastructure to set up a smart server and
         # transport.
-        self.transport_server = smart.server.SmartTCPServer_for_testing
+        self.transport_server = self.make_transport_server
+
+    def make_transport_server(self):
+        return smart.server.SmartTCPServer_for_testing('-' + self.id())
 
     def get_smart_medium(self):
         """Get a smart medium to use in tests."""
@@ -98,8 +128,39 @@ class TestSmartServerResponse(tests.TestCase):
         self.assertNotEqual(None,
             SmartServerResponse(('ok', )))
 
+    def test__str__(self):
+        """SmartServerResponses can be stringified."""
+        self.assertEqual(
+            "<SmartServerResponse status=OK args=('args',) body='body'>",
+            str(SuccessfulSmartServerResponse(('args',), 'body')))
+        self.assertEqual(
+            "<SmartServerResponse status=ERR args=('args',) body='body'>",
+            str(FailedSmartServerResponse(('args',), 'body')))
 
-class TestSmartServerRequestFindRepository(tests.TestCaseWithTransport):
+
+class TestSmartServerRequest(tests.TestCaseWithMemoryTransport):
+
+    def test_translate_client_path(self):
+        transport = self.get_transport()
+        request = SmartServerRequest(transport, 'foo/')
+        self.assertEqual('./', request.translate_client_path('foo/'))
+        self.assertRaises(
+            errors.InvalidURLJoin, request.translate_client_path, 'foo/..')
+        self.assertRaises(
+            errors.PathNotChild, request.translate_client_path, '/')
+        self.assertRaises(
+            errors.PathNotChild, request.translate_client_path, 'bar/')
+        self.assertEqual('./baz', request.translate_client_path('foo/baz'))
+
+    def test_transport_from_client_path(self):
+        transport = self.get_transport()
+        request = SmartServerRequest(transport, 'foo/')
+        self.assertEqual(
+            transport.base,
+            request.transport_from_client_path('foo/').base)
+
+
+class TestSmartServerRequestFindRepository(tests.TestCaseWithMemoryTransport):
     """Tests for BzrDir.find_repository."""
 
     def test_no_repository(self):
@@ -108,7 +169,7 @@ class TestSmartServerRequestFindRepository(tests.TestCaseWithTransport):
         request = self._request_class(backing)
         self.make_bzrdir('.')
         self.assertEqual(SmartServerResponse(('norepository', )),
-            request.execute(backing.local_abspath('')))
+            request.execute(''))
 
     def test_nonshared_repository(self):
         # nonshared repositorys only allow 'find' to return a handle when the 
@@ -117,10 +178,10 @@ class TestSmartServerRequestFindRepository(tests.TestCaseWithTransport):
         backing = self.get_transport()
         request = self._request_class(backing)
         result = self._make_repository_and_result()
-        self.assertEqual(result, request.execute(backing.local_abspath('')))
+        self.assertEqual(result, request.execute(''))
         self.make_bzrdir('subdir')
         self.assertEqual(SmartServerResponse(('norepository', )),
-            request.execute(backing.local_abspath('subdir')))
+            request.execute('subdir'))
 
     def _make_repository_and_result(self, shared=False, format=None):
         """Convenience function to setup a repository.
@@ -150,15 +211,15 @@ class TestSmartServerRequestFindRepository(tests.TestCaseWithTransport):
         backing = self.get_transport()
         request = self._request_class(backing)
         result = self._make_repository_and_result(shared=True)
-        self.assertEqual(result, request.execute(backing.local_abspath('')))
+        self.assertEqual(result, request.execute(''))
         self.make_bzrdir('subdir')
         result2 = SmartServerResponse(result.args[0:1] + ('..', ) + result.args[2:])
         self.assertEqual(result2,
-            request.execute(backing.local_abspath('subdir')))
+            request.execute('subdir'))
         self.make_bzrdir('subdir/deeper')
         result3 = SmartServerResponse(result.args[0:1] + ('../..', ) + result.args[2:])
         self.assertEqual(result3,
-            request.execute(backing.local_abspath('subdir/deeper')))
+            request.execute('subdir/deeper'))
 
     def test_rich_root_and_subtree_encoding(self):
         """Test for the format attributes for rich root and subtree support."""
@@ -168,7 +229,7 @@ class TestSmartServerRequestFindRepository(tests.TestCaseWithTransport):
         # check the test will be valid
         self.assertEqual('yes', result.args[2])
         self.assertEqual('yes', result.args[3])
-        self.assertEqual(result, request.execute(backing.local_abspath('')))
+        self.assertEqual(result, request.execute(''))
 
     def test_supports_external_lookups_no_v2(self):
         """Test for the supports_external_lookups attribute."""
@@ -177,17 +238,17 @@ class TestSmartServerRequestFindRepository(tests.TestCaseWithTransport):
         result = self._make_repository_and_result(format='dirstate-with-subtree')
         # check the test will be valid
         self.assertEqual('no', result.args[4])
-        self.assertEqual(result, request.execute(backing.local_abspath('')))
+        self.assertEqual(result, request.execute(''))
 
 
-class TestSmartServerRequestInitializeBzrDir(tests.TestCaseWithTransport):
+class TestSmartServerRequestInitializeBzrDir(tests.TestCaseWithMemoryTransport):
 
     def test_empty_dir(self):
         """Initializing an empty dir should succeed and do it."""
         backing = self.get_transport()
         request = smart.bzrdir.SmartServerRequestInitializeBzrDir(backing)
         self.assertEqual(SmartServerResponse(('ok', )),
-            request.execute(backing.local_abspath('.')))
+            request.execute(''))
         made_dir = bzrdir.BzrDir.open_from_transport(backing)
         # no branch, tree or repository is expected with the current 
         # default formart.
@@ -200,7 +261,7 @@ class TestSmartServerRequestInitializeBzrDir(tests.TestCaseWithTransport):
         backing = self.get_transport()
         request = smart.bzrdir.SmartServerRequestInitializeBzrDir(backing)
         self.assertRaises(errors.NoSuchFile,
-            request.execute, backing.local_abspath('subdir'))
+            request.execute, 'subdir')
 
     def test_initialized_dir(self):
         """Initializing an extant bzrdir should fail like the bzrdir api."""
@@ -208,10 +269,10 @@ class TestSmartServerRequestInitializeBzrDir(tests.TestCaseWithTransport):
         request = smart.bzrdir.SmartServerRequestInitializeBzrDir(backing)
         self.make_bzrdir('subdir')
         self.assertRaises(errors.FileExists,
-            request.execute, backing.local_abspath('subdir'))
+            request.execute, 'subdir')
 
 
-class TestSmartServerRequestOpenBranch(tests.TestCaseWithTransport):
+class TestSmartServerRequestOpenBranch(TestCaseWithChrootedTransport):
 
     def test_no_branch(self):
         """When there is no branch, ('nobranch', ) is returned."""
@@ -219,7 +280,7 @@ class TestSmartServerRequestOpenBranch(tests.TestCaseWithTransport):
         request = smart.bzrdir.SmartServerRequestOpenBranch(backing)
         self.make_bzrdir('.')
         self.assertEqual(SmartServerResponse(('nobranch', )),
-            request.execute(backing.local_abspath('')))
+            request.execute(''))
 
     def test_branch(self):
         """When there is a branch, 'ok' is returned."""
@@ -227,7 +288,7 @@ class TestSmartServerRequestOpenBranch(tests.TestCaseWithTransport):
         request = smart.bzrdir.SmartServerRequestOpenBranch(backing)
         self.make_branch('.')
         self.assertEqual(SmartServerResponse(('ok', '')),
-            request.execute(backing.local_abspath('')))
+            request.execute(''))
 
     def test_branch_reference(self):
         """When there is a branch reference, the reference URL is returned."""
@@ -235,15 +296,13 @@ class TestSmartServerRequestOpenBranch(tests.TestCaseWithTransport):
         request = smart.bzrdir.SmartServerRequestOpenBranch(backing)
         branch = self.make_branch('branch')
         checkout = branch.create_checkout('reference',lightweight=True)
-        # TODO: once we have an API to probe for references of any sort, we
-        # can use it here.
-        reference_url = backing.abspath('branch') + '/'
+        reference_url = BranchReferenceFormat().get_reference(checkout.bzrdir)
         self.assertFileEqual(reference_url, 'reference/.bzr/branch/location')
         self.assertEqual(SmartServerResponse(('ok', reference_url)),
-            request.execute(backing.local_abspath('reference')))
+            request.execute('reference'))
 
 
-class TestSmartServerRequestRevisionHistory(tests.TestCaseWithTransport):
+class TestSmartServerRequestRevisionHistory(tests.TestCaseWithMemoryTransport):
 
     def test_empty(self):
         """For an empty branch, the body is empty."""
@@ -251,7 +310,7 @@ class TestSmartServerRequestRevisionHistory(tests.TestCaseWithTransport):
         request = smart.branch.SmartServerRequestRevisionHistory(backing)
         self.make_branch('.')
         self.assertEqual(SmartServerResponse(('ok', ), ''),
-            request.execute(backing.local_abspath('')))
+            request.execute(''))
 
     def test_not_empty(self):
         """For a non-empty branch, the body is empty."""
@@ -265,10 +324,10 @@ class TestSmartServerRequestRevisionHistory(tests.TestCaseWithTransport):
         tree.unlock()
         self.assertEqual(
             SmartServerResponse(('ok', ), ('\x00'.join([r1, r2]))),
-            request.execute(backing.local_abspath('')))
+            request.execute(''))
 
 
-class TestSmartServerBranchRequest(tests.TestCaseWithTransport):
+class TestSmartServerBranchRequest(tests.TestCaseWithMemoryTransport):
 
     def test_no_branch(self):
         """When there is a bzrdir and no branch, NotBranchError is raised."""
@@ -276,7 +335,7 @@ class TestSmartServerBranchRequest(tests.TestCaseWithTransport):
         request = smart.branch.SmartServerBranchRequest(backing)
         self.make_bzrdir('.')
         self.assertRaises(errors.NotBranchError,
-            request.execute, backing.local_abspath(''))
+            request.execute, '')
 
     def test_branch_reference(self):
         """When there is a branch reference, NotBranchError is raised."""
@@ -285,10 +344,10 @@ class TestSmartServerBranchRequest(tests.TestCaseWithTransport):
         branch = self.make_branch('branch')
         checkout = branch.create_checkout('reference',lightweight=True)
         self.assertRaises(errors.NotBranchError,
-            request.execute, backing.local_abspath('checkout'))
+            request.execute, 'checkout')
 
 
-class TestSmartServerBranchRequestLastRevisionInfo(tests.TestCaseWithTransport):
+class TestSmartServerBranchRequestLastRevisionInfo(tests.TestCaseWithMemoryTransport):
 
     def test_empty(self):
         """For an empty branch, the result is ('ok', '0', 'null:')."""
@@ -296,7 +355,7 @@ class TestSmartServerBranchRequestLastRevisionInfo(tests.TestCaseWithTransport):
         request = smart.branch.SmartServerBranchRequestLastRevisionInfo(backing)
         self.make_branch('.')
         self.assertEqual(SmartServerResponse(('ok', '0', 'null:')),
-            request.execute(backing.local_abspath('')))
+            request.execute(''))
 
     def test_not_empty(self):
         """For a non-empty branch, the result is ('ok', 'revno', 'revid')."""
@@ -311,10 +370,10 @@ class TestSmartServerBranchRequestLastRevisionInfo(tests.TestCaseWithTransport):
         tree.unlock()
         self.assertEqual(
             SmartServerResponse(('ok', '2', rev_id_utf8)),
-            request.execute(backing.local_abspath('')))
+            request.execute(''))
 
 
-class TestSmartServerBranchRequestGetConfigFile(tests.TestCaseWithTransport):
+class TestSmartServerBranchRequestGetConfigFile(tests.TestCaseWithMemoryTransport):
 
     def test_default(self):
         """With no file, we get empty content."""
@@ -324,7 +383,7 @@ class TestSmartServerBranchRequestGetConfigFile(tests.TestCaseWithTransport):
         # there should be no file by default
         content = ''
         self.assertEqual(SmartServerResponse(('ok', ), content),
-            request.execute(backing.local_abspath('')))
+            request.execute(''))
 
     def test_with_content(self):
         # SmartServerBranchGetConfigFile should return the content from
@@ -335,10 +394,10 @@ class TestSmartServerBranchRequestGetConfigFile(tests.TestCaseWithTransport):
         branch = self.make_branch('.')
         branch.control_files.put_utf8('branch.conf', 'foo bar baz')
         self.assertEqual(SmartServerResponse(('ok', ), 'foo bar baz'),
-            request.execute(backing.local_abspath('')))
+            request.execute(''))
 
 
-class TestSmartServerBranchRequestSetLastRevision(tests.TestCaseWithTransport):
+class TestSmartServerBranchRequestSetLastRevision(tests.TestCaseWithMemoryTransport):
 
     def test_empty(self):
         backing = self.get_transport()
@@ -350,7 +409,7 @@ class TestSmartServerBranchRequestSetLastRevision(tests.TestCaseWithTransport):
         try:
             self.assertEqual(SmartServerResponse(('ok',)),
                 request.execute(
-                    backing.local_abspath(''), branch_token, repo_token,
+                    '', branch_token, repo_token,
                     'null:'))
         finally:
             b.unlock()
@@ -367,7 +426,7 @@ class TestSmartServerBranchRequestSetLastRevision(tests.TestCaseWithTransport):
             self.assertEqual(
                 SmartServerResponse(('NoSuchRevision', revision_id)),
                 request.execute(
-                    backing.local_abspath(''), branch_token, repo_token,
+                    '', branch_token, repo_token,
                     revision_id))
         finally:
             b.unlock()
@@ -389,7 +448,7 @@ class TestSmartServerBranchRequestSetLastRevision(tests.TestCaseWithTransport):
             self.assertEqual(
                 SmartServerResponse(('ok',)),
                 request.execute(
-                    backing.local_abspath(''), branch_token, repo_token,
+                    '', branch_token, repo_token,
                     rev_id_utf8))
             self.assertEqual([rev_id_utf8], tree.branch.revision_history())
         finally:
@@ -413,25 +472,88 @@ class TestSmartServerBranchRequestSetLastRevision(tests.TestCaseWithTransport):
             self.assertEqual(
                 SmartServerResponse(('ok',)),
                 request.execute(
-                    backing.local_abspath(''), branch_token, repo_token,
+                    '', branch_token, repo_token,
                     rev_id_utf8))
             self.assertEqual([rev_id_utf8], tree.branch.revision_history())
         finally:
             tree.branch.unlock()
 
 
-class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithTransport):
+class TestSmartServerBranchRequestSetLastRevisionInfo(tests.TestCaseWithTransport):
+
+    def lock_branch(self, branch):
+        branch_token = branch.lock_write()
+        repo_token = branch.repository.lock_write()
+        branch.repository.unlock()
+        self.addCleanup(branch.unlock)
+        return branch_token, repo_token
+
+    def make_locked_branch(self, format=None):
+        branch = self.make_branch('.', format=format)
+        branch_token, repo_token = self.lock_branch(branch)
+        return branch, branch_token, repo_token
+
+    def test_empty(self):
+        """An empty branch can have its last revision set to 'null:'."""
+        b, branch_token, repo_token = self.make_locked_branch()
+        backing = self.get_transport()
+        request = smart.branch.SmartServerBranchRequestSetLastRevisionInfo(
+            backing)
+        response = request.execute('', branch_token, repo_token, '0', 'null:')
+        self.assertEqual(SmartServerResponse(('ok',)), response)
+
+    def assertBranchLastRevisionInfo(self, expected_info, branch_relpath):
+        branch = bzrdir.BzrDir.open(branch_relpath).open_branch()
+        self.assertEqual(expected_info, branch.last_revision_info())
+
+    def test_branch_revision_info_is_updated(self):
+        """This method really does update the branch last revision info."""
+        tree = self.make_branch_and_memory_tree('.')
+        tree.lock_write()
+        tree.add('')
+        tree.commit('First commit', rev_id='revision-1')
+        tree.commit('Second commit', rev_id='revision-2')
+        tree.unlock()
+        branch = tree.branch
+
+        branch_token, repo_token = self.lock_branch(branch)
+        backing = self.get_transport()
+        request = smart.branch.SmartServerBranchRequestSetLastRevisionInfo(
+            backing)
+        self.assertBranchLastRevisionInfo((2, 'revision-2'), '.')
+        response = request.execute(
+            '', branch_token, repo_token, '1', 'revision-1')
+        self.assertEqual(SmartServerResponse(('ok',)), response)
+        self.assertBranchLastRevisionInfo((1, 'revision-1'), '.')
+
+    def test_not_present_revid(self):
+        """Some branch formats will check that the revision is present in the
+        repository.  When that check fails, a NoSuchRevision error is returned
+        to the client.
+        """
+        # Make a knit format branch, because that format checks the values
+        # given to set_last_revision_info.
+        b, branch_token, repo_token = self.make_locked_branch(format='knit')
+        backing = self.get_transport()
+        request = smart.branch.SmartServerBranchRequestSetLastRevisionInfo(
+            backing)
+        response = request.execute(
+            '', branch_token, repo_token, '1', 'not-present')
+        self.assertEqual(
+            SmartServerResponse(('NoSuchRevision', 'not-present')), response)
+
+
+class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithMemoryTransport):
 
     def setUp(self):
-        tests.TestCaseWithTransport.setUp(self)
-        self.reduceLockdirTimeout()
+        tests.TestCaseWithMemoryTransport.setUp(self)
 
     def test_lock_write_on_unlocked_branch(self):
         backing = self.get_transport()
         request = smart.branch.SmartServerBranchRequestLockWrite(backing)
         branch = self.make_branch('.', format='knit')
         repository = branch.repository
-        response = request.execute(backing.local_abspath(''))
+        response = request.execute('')
         branch_nonce = branch.control_files._lock.peek().get('nonce')
         repository_nonce = repository.control_files._lock.peek().get('nonce')
         self.assertEqual(
@@ -449,7 +571,7 @@ class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithTransport):
         branch.lock_write()
         branch.leave_lock_in_place()
         branch.unlock()
-        response = request.execute(backing.local_abspath(''))
+        response = request.execute('')
         self.assertEqual(
             SmartServerResponse(('LockContention',)), response)
 
@@ -463,7 +585,7 @@ class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithTransport):
         branch.leave_lock_in_place()
         branch.repository.leave_lock_in_place()
         branch.unlock()
-        response = request.execute(backing.local_abspath(''),
+        response = request.execute('',
                                    branch_token, repo_token)
         self.assertEqual(
             SmartServerResponse(('ok', branch_token, repo_token)), response)
@@ -478,7 +600,7 @@ class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithTransport):
         branch.leave_lock_in_place()
         branch.repository.leave_lock_in_place()
         branch.unlock()
-        response = request.execute(backing.local_abspath(''),
+        response = request.execute('',
                                    branch_token+'xxx', repo_token)
         self.assertEqual(
             SmartServerResponse(('TokenMismatch',)), response)
@@ -490,7 +612,7 @@ class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithTransport):
         branch.repository.lock_write()
         branch.repository.leave_lock_in_place()
         branch.repository.unlock()
-        response = request.execute(backing.local_abspath(''))
+        response = request.execute('')
         self.assertEqual(
             SmartServerResponse(('LockContention',)), response)
 
@@ -498,17 +620,18 @@ class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithTransport):
         backing = self.get_readonly_transport()
         request = smart.branch.SmartServerBranchRequestLockWrite(backing)
         branch = self.make_branch('.')
-        response = request.execute('')
+        root = self.get_transport().clone('/')
+        path = urlutils.relative_url(root.base, self.get_transport().base)
+        response = request.execute(path)
         error_name, lock_str, why_str = response.args
         self.assertFalse(response.is_successful())
         self.assertEqual('LockFailed', error_name)
 
 
-class TestSmartServerBranchRequestUnlock(tests.TestCaseWithTransport):
+class TestSmartServerBranchRequestUnlock(tests.TestCaseWithMemoryTransport):
 
     def setUp(self):
-        tests.TestCaseWithTransport.setUp(self)
-        self.reduceLockdirTimeout()
+        tests.TestCaseWithMemoryTransport.setUp(self)
 
     def test_unlock_on_locked_branch_and_repo(self):
         backing = self.get_transport()
@@ -523,7 +646,7 @@ class TestSmartServerBranchRequestUnlock(tests.TestCaseWithTransport):
         branch.leave_lock_in_place()
         branch.repository.leave_lock_in_place()
         branch.unlock()
-        response = request.execute(backing.local_abspath(''),
+        response = request.execute('',
                                    branch_token, repo_token)
         self.assertEqual(
             SmartServerResponse(('ok',)), response)
@@ -538,7 +661,7 @@ class TestSmartServerBranchRequestUnlock(tests.TestCaseWithTransport):
         request = smart.branch.SmartServerBranchRequestUnlock(backing)
         branch = self.make_branch('.', format='knit')
         response = request.execute(
-            backing.local_abspath(''), 'branch token', 'repo token')
+            '', 'branch token', 'repo token')
         self.assertEqual(
             SmartServerResponse(('TokenMismatch',)), response)
 
@@ -553,12 +676,12 @@ class TestSmartServerBranchRequestUnlock(tests.TestCaseWithTransport):
         # Issue branch lock_write request on the unlocked branch (with locked
         # repo).
         response = request.execute(
-            backing.local_abspath(''), 'branch token', repo_token)
+            '', 'branch token', repo_token)
         self.assertEqual(
             SmartServerResponse(('TokenMismatch',)), response)
 
 
-class TestSmartServerRepositoryRequest(tests.TestCaseWithTransport):
+class TestSmartServerRepositoryRequest(tests.TestCaseWithMemoryTransport):
 
     def test_no_repository(self):
         """Raise NoRepositoryPresent when there is a bzrdir and no repo."""
@@ -571,7 +694,7 @@ class TestSmartServerRepositoryRequest(tests.TestCaseWithTransport):
         self.make_repository('.', shared=True)
         self.make_bzrdir('subdir')
         self.assertRaises(errors.NoRepositoryPresent,
-            request.execute, backing.local_abspath('subdir'))
+            request.execute, 'subdir')
 
 
 class TestSmartServerRepositoryGetParentMap(tests.TestCaseWithTransport):
@@ -583,14 +706,14 @@ class TestSmartServerRepositoryGetParentMap(tests.TestCaseWithTransport):
         tree = self.make_branch_and_memory_tree('.')
 
         self.assertEqual(None,
-            request.execute(backing.local_abspath(''), 'missing-id'))
+            request.execute('', 'missing-id'))
         # Note that it returns a body (of '' bzipped).
         self.assertEqual(
             SuccessfulSmartServerResponse(('ok', ), bz2.compress('')),
             request.do_body('\n\n0\n'))
 
 
-class TestSmartServerRepositoryGetRevisionGraph(tests.TestCaseWithTransport):
+class TestSmartServerRepositoryGetRevisionGraph(tests.TestCaseWithMemoryTransport):
 
     def test_none_argument(self):
         backing = self.get_transport()
@@ -605,7 +728,7 @@ class TestSmartServerRepositoryGetRevisionGraph(tests.TestCaseWithTransport):
         # the lines of revision_id->revision_parent_list has no guaranteed
         # order coming out of a dict, so sort both our test and response
         lines = sorted([' '.join([r2, r1]), r1])
-        response = request.execute(backing.local_abspath(''), '')
+        response = request.execute('', '')
         response.body = '\n'.join(sorted(response.body.split('\n')))
 
         self.assertEqual(
@@ -623,7 +746,7 @@ class TestSmartServerRepositoryGetRevisionGraph(tests.TestCaseWithTransport):
         tree.unlock()
 
         self.assertEqual(SmartServerResponse(('ok', ), rev_id_utf8),
-            request.execute(backing.local_abspath(''), rev_id_utf8))
+            request.execute('', rev_id_utf8))
     
     def test_no_such_revision(self):
         backing = self.get_transport()
@@ -637,10 +760,10 @@ class TestSmartServerRepositoryGetRevisionGraph(tests.TestCaseWithTransport):
         # Note that it still returns body (of zero bytes).
         self.assertEqual(
             SmartServerResponse(('nosuchrevision', 'missingrevision', ), ''),
-            request.execute(backing.local_abspath(''), 'missingrevision'))
+            request.execute('', 'missingrevision'))
 
 
-class TestSmartServerRequestHasRevision(tests.TestCaseWithTransport):
+class TestSmartServerRequestHasRevision(tests.TestCaseWithMemoryTransport):
 
     def test_missing_revision(self):
         """For a missing revision, ('no', ) is returned."""
@@ -648,7 +771,7 @@ class TestSmartServerRequestHasRevision(tests.TestCaseWithTransport):
         request = smart.repository.SmartServerRequestHasRevision(backing)
         self.make_repository('.')
         self.assertEqual(SmartServerResponse(('no', )),
-            request.execute(backing.local_abspath(''), 'revid'))
+            request.execute('', 'revid'))
 
     def test_present_revision(self):
         """For a present revision, ('yes', ) is returned."""
@@ -662,10 +785,10 @@ class TestSmartServerRequestHasRevision(tests.TestCaseWithTransport):
         tree.unlock()
         self.assertTrue(tree.branch.repository.has_revision(rev_id_utf8))
         self.assertEqual(SmartServerResponse(('yes', )),
-            request.execute(backing.local_abspath(''), rev_id_utf8))
+            request.execute('', rev_id_utf8))
 
 
-class TestSmartServerRepositoryGatherStats(tests.TestCaseWithTransport):
+class TestSmartServerRepositoryGatherStats(tests.TestCaseWithMemoryTransport):
 
     def test_empty_revid(self):
         """With an empty revid, we get only size an number and revisions"""
@@ -676,7 +799,7 @@ class TestSmartServerRepositoryGatherStats(tests.TestCaseWithTransport):
         size = stats['size']
         expected_body = 'revisions: 0\nsize: %d\n' % size
         self.assertEqual(SmartServerResponse(('ok', ), expected_body),
-                         request.execute(backing.local_abspath(''), '', 'no'))
+                         request.execute('', '', 'no'))
 
     def test_revid_with_committers(self):
         """For a revid we get more infos."""
@@ -699,7 +822,7 @@ class TestSmartServerRepositoryGatherStats(tests.TestCaseWithTransport):
                          'revisions: 2\n'
                          'size: %d\n' % size)
         self.assertEqual(SmartServerResponse(('ok', ), expected_body),
-                         request.execute(backing.local_abspath(''),
+                         request.execute('',
                                          rev_id_utf8, 'no'))
 
     def test_not_empty_repository_with_committers(self):
@@ -725,11 +848,11 @@ class TestSmartServerRepositoryGatherStats(tests.TestCaseWithTransport):
                          'revisions: 2\n'
                          'size: %d\n' % size)
         self.assertEqual(SmartServerResponse(('ok', ), expected_body),
-                         request.execute(backing.local_abspath(''),
+                         request.execute('',
                                          rev_id_utf8, 'yes'))
 
 
-class TestSmartServerRepositoryIsShared(tests.TestCaseWithTransport):
+class TestSmartServerRepositoryIsShared(tests.TestCaseWithMemoryTransport):
 
     def test_is_shared(self):
         """For a shared repository, ('yes', ) is returned."""
@@ -737,7 +860,7 @@ class TestSmartServerRepositoryIsShared(tests.TestCaseWithTransport):
         request = smart.repository.SmartServerRepositoryIsShared(backing)
         self.make_repository('.', shared=True)
         self.assertEqual(SmartServerResponse(('yes', )),
-            request.execute(backing.local_abspath(''), ))
+            request.execute('', ))
 
     def test_is_not_shared(self):
         """For a shared repository, ('no', ) is returned."""
@@ -745,20 +868,19 @@ class TestSmartServerRepositoryIsShared(tests.TestCaseWithTransport):
         request = smart.repository.SmartServerRepositoryIsShared(backing)
         self.make_repository('.', shared=False)
         self.assertEqual(SmartServerResponse(('no', )),
-            request.execute(backing.local_abspath(''), ))
+            request.execute('', ))
 
 
-class TestSmartServerRepositoryLockWrite(tests.TestCaseWithTransport):
+class TestSmartServerRepositoryLockWrite(tests.TestCaseWithMemoryTransport):
 
     def setUp(self):
-        tests.TestCaseWithTransport.setUp(self)
-        self.reduceLockdirTimeout()
+        tests.TestCaseWithMemoryTransport.setUp(self)
 
     def test_lock_write_on_unlocked_repo(self):
         backing = self.get_transport()
         request = smart.repository.SmartServerRepositoryLockWrite(backing)
         repository = self.make_repository('.', format='knit')
-        response = request.execute(backing.local_abspath(''))
+        response = request.execute('')
         nonce = repository.control_files._lock.peek().get('nonce')
         self.assertEqual(SmartServerResponse(('ok', nonce)), response)
         # The repository is now locked.  Verify that with a new repository
@@ -773,7 +895,7 @@ class TestSmartServerRepositoryLockWrite(tests.TestCaseWithTransport):
         repository.lock_write()
         repository.leave_lock_in_place()
         repository.unlock()
-        response = request.execute(backing.local_abspath(''))
+        response = request.execute('')
         self.assertEqual(
             SmartServerResponse(('LockContention',)), response)
 
@@ -786,11 +908,10 @@ class TestSmartServerRepositoryLockWrite(tests.TestCaseWithTransport):
         self.assertEqual('LockFailed', response.args[0])
 
 
-class TestSmartServerRepositoryUnlock(tests.TestCaseWithTransport):
+class TestSmartServerRepositoryUnlock(tests.TestCaseWithMemoryTransport):
 
     def setUp(self):
-        tests.TestCaseWithTransport.setUp(self)
-        self.reduceLockdirTimeout()
+        tests.TestCaseWithMemoryTransport.setUp(self)
 
     def test_unlock_on_locked_repo(self):
         backing = self.get_transport()
@@ -799,7 +920,7 @@ class TestSmartServerRepositoryUnlock(tests.TestCaseWithTransport):
         token = repository.lock_write()
         repository.leave_lock_in_place()
         repository.unlock()
-        response = request.execute(backing.local_abspath(''), token)
+        response = request.execute('', token)
         self.assertEqual(
             SmartServerResponse(('ok',)), response)
         # The repository is now unlocked.  Verify that with a new repository
@@ -812,7 +933,7 @@ class TestSmartServerRepositoryUnlock(tests.TestCaseWithTransport):
         backing = self.get_transport()
         request = smart.repository.SmartServerRepositoryUnlock(backing)
         repository = self.make_repository('.', format='knit')
-        response = request.execute(backing.local_abspath(''), 'some token')
+        response = request.execute('', 'some token')
         self.assertEqual(
             SmartServerResponse(('TokenMismatch',)), response)
 
@@ -826,7 +947,7 @@ class TestSmartServerRepositoryTarball(tests.TestCaseWithTransport):
         # make some extraneous junk in the repository directory which should
         # not be copied
         self.build_tree(['.bzr/repository/extra-junk'])
-        response = request.execute(backing.local_abspath(''), 'bz2')
+        response = request.execute('', 'bz2')
         self.assertEqual(('ok',), response.args)
         # body should be a tbz2
         body_file = StringIO(response.body)
@@ -841,7 +962,7 @@ class TestSmartServerRepositoryTarball(tests.TestCaseWithTransport):
             "extraneous file present in tar file")
 
 
-class TestSmartServerRepositoryStreamKnitData(tests.TestCaseWithTransport):
+class TestSmartServerRepositoryStreamKnitData(tests.TestCaseWithMemoryTransport):
 
     def test_fetch_revisions(self):
         backing = self.get_transport()
@@ -855,9 +976,8 @@ class TestSmartServerRepositoryStreamKnitData(tests.TestCaseWithTransport):
         r1 = tree.commit('2nd commit', rev_id=rev_id2_utf8)
         tree.unlock()
 
-        response = request.execute(backing.local_abspath(''), rev_id2_utf8)
+        response = request.execute('', rev_id2_utf8)
         self.assertEqual(('ok',), response.args)
-        from cStringIO import StringIO
         unpacker = pack.ContainerReader(StringIO(response.body))
         names = []
         for [name], read_bytes in unpacker.iter_records():
@@ -873,13 +993,13 @@ class TestSmartServerRepositoryStreamKnitData(tests.TestCaseWithTransport):
         request = smart.repository.SmartServerRepositoryStreamKnitDataForRevisions(backing)
         repo = self.make_repository('.')
         rev_id1_utf8 = u'\xc8'.encode('utf-8')
-        response = request.execute(backing.local_abspath(''), rev_id1_utf8)
+        response = request.execute('', rev_id1_utf8)
         self.assertEqual(
             SmartServerResponse(('NoSuchRevision', rev_id1_utf8)),
             response)
 
 
-class TestSmartServerRepositoryStreamRevisionsChunked(tests.TestCaseWithTransport):
+class TestSmartServerRepositoryStreamRevisionsChunked(tests.TestCaseWithMemoryTransport):
 
     def test_fetch_revisions(self):
         backing = self.get_transport()
@@ -894,11 +1014,10 @@ class TestSmartServerRepositoryStreamRevisionsChunked(tests.TestCaseWithTranspor
         tree.commit('2nd commit', rev_id=rev_id2_utf8)
         tree.unlock()
 
-        response = request.execute(backing.local_abspath(''))
+        response = request.execute('')
         self.assertEqual(None, response)
         response = request.do_body("%s\n%s\n1" % (rev_id2_utf8, rev_id1_utf8))
         self.assertEqual(('ok',), response.args)
-        from cStringIO import StringIO
         parser = pack.ContainerPushParser()
         names = []
         for stream_bytes in response.body_stream:
@@ -916,7 +1035,7 @@ class TestSmartServerRepositoryStreamRevisionsChunked(tests.TestCaseWithTranspor
             backing)
         repo = self.make_repository('.')
         rev_id1_utf8 = u'\xc8'.encode('utf-8')
-        response = request.execute(backing.local_abspath(''))
+        response = request.execute('')
         self.assertEqual(None, response)
         response = request.do_body("%s\n\n1" % (rev_id1_utf8,))
         self.assertEqual(
@@ -924,7 +1043,7 @@ class TestSmartServerRepositoryStreamRevisionsChunked(tests.TestCaseWithTranspor
             response)
 
 
-class TestSmartServerIsReadonly(tests.TestCaseWithTransport):
+class TestSmartServerIsReadonly(tests.TestCaseWithMemoryTransport):
 
     def test_is_readonly_no(self):
         backing = self.get_transport()
@@ -961,6 +1080,9 @@ class TestHandlers(tests.TestCase):
         self.assertEqual(
             smart.request.request_handlers.get('Branch.set_last_revision'),
             smart.branch.SmartServerBranchRequestSetLastRevision)
+        self.assertEqual(
+            smart.request.request_handlers.get('Branch.set_last_revision_info'),
+            smart.branch.SmartServerBranchRequestSetLastRevisionInfo)
         self.assertEqual(
             smart.request.request_handlers.get('Branch.unlock'),
             smart.branch.SmartServerBranchRequestUnlock)
