@@ -184,7 +184,7 @@ class SubprocessVendorsTests(TestCase):
         self.assertEqual(
             vendor._get_vendor_specific_argv(
                 "user", "host", 100, command=["bzr"]),
-            ["plink", "-x", "-a", "-ssh", "-2",
+            ["plink", "-x", "-a", "-ssh", "-2", "-batch",
                 "-P", "100",
                 "-l", "user",
                 "host", "bzr"]
@@ -195,7 +195,7 @@ class SubprocessVendorsTests(TestCase):
         self.assertEqual(
             vendor._get_vendor_specific_argv(
                 "user", "host", 100, subsystem="sftp"),
-            ["plink", "-x", "-a", "-ssh", "-2",
+            ["plink", "-x", "-a", "-ssh", "-2", "-batch",
                 "-P", "100",
                 "-l", "user",
                 "-s", "host", "sftp"]
