@@ -21,9 +21,16 @@ import imp
 import os
 import sys
 
+from bzrlib import (
+    errors,
+    symbol_versioning,
+    tests,
+    version_info_formats,
+    )
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.rio import read_stanzas
 
+from bzrlib.version_info_formats.format_custom import CustomVersionInfoBuilder
 from bzrlib.version_info_formats.format_rio import RioVersionInfoBuilder
 from bzrlib.version_info_formats.format_python import PythonVersionInfoBuilder
 
@@ -111,9 +118,12 @@ class TestVersionInfo(TestCaseWithTransport):
         stanza = regen(check_for_clean=True, include_file_revisions=True)
         self.assertEqual(['False'], stanza.get_all('clean'))
 
+        # XXX: This assumes it's being run against a repository that updates
+        # the root revision on every commit.  Newer ones that use
+        # RootCommitBuilder won't update it on each commit.
         file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
-        self.assertEqual(['a', 'b', 'c'], file_rev_stanza.get_all('path'))
-        self.assertEqual(['r3', 'r2', 'unversioned'],
+        self.assertEqual(['', 'a', 'b', 'c'], file_rev_stanza.get_all('path'))
+        self.assertEqual(['r3', 'r3', 'r2', 'unversioned'],
             file_rev_stanza.get_all('revision'))
         os.remove('branch/c')
 
@@ -129,8 +139,10 @@ class TestVersionInfo(TestCaseWithTransport):
         wt.rename_one('b', 'd')
         stanza = regen(check_for_clean=True, include_file_revisions=True)
         file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
-        self.assertEqual(['a', 'b', 'c', 'd'], file_rev_stanza.get_all('path'))
-        self.assertEqual(['modified', 'renamed to d', 'new', 'renamed from b'],
+        self.assertEqual(['', 'a', 'b', 'c', 'd'], 
+                          file_rev_stanza.get_all('path'))
+        self.assertEqual(['r3', 'modified', 'renamed to d', 'new', 
+                          'renamed from b'],
                          file_rev_stanza.get_all('revision'))
 
         wt.commit('modified', rev_id='r4')
@@ -138,8 +150,8 @@ class TestVersionInfo(TestCaseWithTransport):
         os.remove('branch/d')
         stanza = regen(check_for_clean=True, include_file_revisions=True)
         file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
-        self.assertEqual(['a', 'c', 'd'], file_rev_stanza.get_all('path'))
-        self.assertEqual(['r4', 'unversioned', 'removed'],
+        self.assertEqual(['', 'a', 'c', 'd'], file_rev_stanza.get_all('path'))
+        self.assertEqual(['r4', 'r4', 'unversioned', 'removed'],
                          file_rev_stanza.get_all('revision'))
 
     def test_python_version(self):
@@ -182,7 +194,8 @@ class TestVersionInfo(TestCaseWithTransport):
         self.build_tree(['branch/c'])
         tvi = regen(check_for_clean=True, include_file_revisions=True)
         self.assertEqual(False, tvi.version_info['clean'])
-        self.assertEqual(['a', 'b', 'c'], sorted(tvi.file_revisions.keys()))
+        self.assertEqual(['', 'a', 'b', 'c'], 
+                         sorted(tvi.file_revisions.keys()))
         self.assertEqual('r3', tvi.file_revisions['a'])
         self.assertEqual('r2', tvi.file_revisions['b'])
         self.assertEqual('unversioned', tvi.file_revisions['c'])
@@ -199,7 +212,8 @@ class TestVersionInfo(TestCaseWithTransport):
         wt.add('c')
         wt.rename_one('b', 'd')
         tvi = regen(check_for_clean=True, include_file_revisions=True)
-        self.assertEqual(['a', 'b', 'c', 'd'], sorted(tvi.file_revisions.keys()))
+        self.assertEqual(['', 'a', 'b', 'c', 'd'], 
+                          sorted(tvi.file_revisions.keys()))
         self.assertEqual('modified', tvi.file_revisions['a'])
         self.assertEqual('renamed to d', tvi.file_revisions['b'])
         self.assertEqual('new', tvi.file_revisions['c'])
@@ -209,9 +223,76 @@ class TestVersionInfo(TestCaseWithTransport):
         wt.remove(['c', 'd'])
         os.remove('branch/d')
         tvi = regen(check_for_clean=True, include_file_revisions=True)
-        self.assertEqual(['a', 'c', 'd'], sorted(tvi.file_revisions.keys()))
+        self.assertEqual(['', 'a', 'c', 'd'], 
+                          sorted(tvi.file_revisions.keys()))
         self.assertEqual('r4', tvi.file_revisions['a'])
         self.assertEqual('unversioned', tvi.file_revisions['c'])
         self.assertEqual('removed', tvi.file_revisions['d'])
 
+    def test_custom_version_text(self):
+        wt = self.create_branch()
 
+        def regen(tpl, **kwargs):
+            sio = StringIO()
+            builder = CustomVersionInfoBuilder(wt.branch, working_tree=wt,
+                                               template=tpl, **kwargs)
+            builder.generate(sio)
+            val = sio.getvalue()
+            return val
+
+        val = regen('build-date: "{build_date}"\ndate: "{date}"')
+        self.assertContainsRe(val, 'build-date: "[0-9-+: ]+"')
+        self.assertContainsRe(val, 'date: "[0-9-+: ]+"')
+
+        val = regen('revno: {revno}')
+        self.assertEqual(val, 'revno: 3')
+
+        val = regen('revision-id: {revision_id}')
+        self.assertEqual(val, 'revision-id: r3')
+
+        val = regen('clean: {clean}', check_for_clean=True)
+        self.assertEqual(val, 'clean: 1')
+
+        self.build_tree(['branch/c'])
+        val = regen('clean: {clean}', check_for_clean=True)
+        self.assertEqual(val, 'clean: 0')
+        os.remove('branch/c')
+
+    def test_custom_without_template(self):
+        builder = CustomVersionInfoBuilder(None)
+        sio = StringIO()
+        self.assertRaises(errors.NoTemplate, builder.generate, sio)
+
+
+class TestBuilder(version_info_formats.VersionInfoBuilder):
+    pass
+
+
+class TestVersionInfoFormatRegistry(tests.TestCase):
+
+    def setUp(self):
+        super(TestVersionInfoFormatRegistry, self).setUp()
+        registry = version_info_formats.format_registry
+        self._default_key = registry._default_key
+        self._dict = registry._dict.copy()
+        self._help_dict = registry._help_dict.copy()
+        self._info_dict = registry._info_dict.copy()
+        self.addCleanup(self._cleanup)
+
+    def _cleanup(self):
+        # Restore the registry to pristine state after the test runs
+        registry = version_info_formats.format_registry
+        registry._default_key = self._default_key
+        registry._dict = self._dict
+        registry._help_dict = self._help_dict
+        registry._info_dict = self._info_dict
+
+    def test_register_remove(self):
+        registry = version_info_formats.format_registry
+        registry.register('testbuilder',
+            TestBuilder, 'a simple test builder')
+        self.assertIs(TestBuilder, registry.get('testbuilder'))
+        self.assertEqual('a simple test builder',
+                         registry.get_help('testbuilder'))
+        registry.remove('testbuilder')
+        self.assertRaises(KeyError, registry.get, 'testbuilder')
