@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,15 +36,14 @@ from bzrlib.tests import (
                           adapt_modules,
                           default_transport,
                           TestCaseWithTransport,
-                          TestLoader,
                           TestSkipped,
-                          TestSuite,
                           )
 from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
 from bzrlib.tests.workingtree_implementations import (
     WorkingTreeTestProviderAdapter,
     )
 from bzrlib.revisiontree import RevisionTree
+from bzrlib.transform import TransformPreview
 from bzrlib.workingtree import (
     WorkingTreeFormat,
     WorkingTreeFormat3,
@@ -56,27 +55,33 @@ from bzrlib.workingtree_4 import (
     )
 
 
-def return_parameter(something):
+def return_parameter(testcase, something):
     """A trivial thunk to return its input."""
     return something
 
 
-def revision_tree_from_workingtree(tree):
+def revision_tree_from_workingtree(testcase, tree):
     """Create a revision tree from a working tree."""
     revid = tree.commit('save tree', allow_pointless=True, recursive=None)
     return tree.branch.repository.revision_tree(revid)
 
 
-def _dirstate_tree_from_workingtree(tree):
+def _dirstate_tree_from_workingtree(testcase, tree):
     revid = tree.commit('save tree', allow_pointless=True)
     return tree.basis_tree()
 
 
+def preview_tree_pre(testcase, tree):
+    tt = TransformPreview(tree)
+    testcase.addCleanup(tt.finalize)
+    return tt.get_preview_tree()
+
+
 class TestTreeImplementationSupport(TestCaseWithTransport):
 
     def test_revision_tree_from_workingtree(self):
         tree = self.make_branch_and_tree('.')
-        tree = revision_tree_from_workingtree(tree)
+        tree = revision_tree_from_workingtree(self, tree)
         self.assertIsInstance(tree, RevisionTree)
 
 
@@ -89,6 +94,9 @@ class TestCaseWithTree(TestCaseWithBzrDir):
         made_control.create_branch()
         return self.workingtree_format.initialize(made_control)
 
+    def workingtree_to_test_tree(self, tree):
+        return self._workingtree_to_test_tree(self, tree)
+
     def _convert_tree(self, tree, converter=None):
         """helper to convert using the converter or a supplied one."""
         # convert that to the final shape
@@ -262,7 +270,7 @@ class TestCaseWithTree(TestCaseWithBzrDir):
             self.build_tree(paths[1:])
         except UnicodeError:
             raise tests.TestSkipped('filesystem does not support unicode.')
-        if tree.path2id('') is None:
+        if tree.get_root_id() is None:
             # Some trees do not have a root yet.
             tree.add(paths, file_ids)
         else:
@@ -279,8 +287,7 @@ class TestCaseWithTree(TestCaseWithBzrDir):
         self._create_tree_with_utf8(tree)
         tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
         self.build_tree([u'tree2/b\xe5r/z\xf7z'])
-        self.callDeprecated([osutils._file_id_warning],
-                            tree2.add, [u'b\xe5r/z\xf7z'], [u'z\xf7z-id'])
+        tree2.add([u'b\xe5r/z\xf7z'], [u'z\xf7z-id'.encode('utf-8')])
         tree2.commit(u'to m\xe9rge', rev_id=u'r\xe9v-2'.encode('utf8'))
 
         tree.merge_from_branch(tree2.branch)
@@ -302,44 +309,65 @@ class TreeTestProviderAdapter(WorkingTreeTestProviderAdapter):
         # now adjust the scenarios and add the non-working-tree tree scenarios.
         for scenario in self.scenarios:
             # for working tree adapted tests, preserve the tree
-            scenario[1]["workingtree_to_test_tree"] = return_parameter
+            scenario[1]["_workingtree_to_test_tree"] = return_parameter
         # add RevisionTree scenario
-        # this is the 'default format' in that it's used to test the generic InterTree
-        # code.
-        default_format = WorkingTreeFormat3()
-        self.scenarios.append(self.formats_to_scenarios([
-            (default_format, default_format._matchingbzrdir)])[0])
-        self.scenarios[-1] = (RevisionTree.__name__, self.scenarios[-1][1])
-        self.scenarios[-1][1]["workingtree_to_test_tree"] = revision_tree_from_workingtree
+        self.scenarios.append(self.create_tree_scenario(RevisionTree.__name__,
+                              revision_tree_from_workingtree,))
 
-        # also test WorkingTree4's RevisionTree implementation which is specialised.
-        dirstate_format = WorkingTreeFormat4()
-        self.scenarios.append(self.formats_to_scenarios([
-            (dirstate_format, dirstate_format._matchingbzrdir)])[0])
-        self.scenarios[-1] = (DirStateRevisionTree.__name__, self.scenarios[-1][1])
-        self.scenarios[-1][1]["workingtree_to_test_tree"] = _dirstate_tree_from_workingtree
+        # also test WorkingTree4's RevisionTree implementation which is
+        # specialised.
+        self.scenarios.append(self.create_tree_scenario(
+            DirStateRevisionTree.__name__, _dirstate_tree_from_workingtree,
+            WorkingTreeFormat4()))
+        self.scenarios.append(self.create_tree_scenario('PreviewTree',
+            preview_tree_pre))
+
+    def create_tree_scenario(self, name, converter, workingtree_format=None):
+        """Create a scenario for the specified converter
+
+        :param name: The name to append to tests using this converter
+        :param converter: A function that converts a workingtree into the
+            desired format.
+        :param workingtree_format: The particular workingtree format to
+            convert from.
+        :return: a (name, options) tuple, where options is a dict of values
+            to be used as members of the TestCase.
+        """
+        if workingtree_format is None:
+            workingtree_format = WorkingTreeFormat3()
+        scenario_options = WorkingTreeTestProviderAdapter.create_scenario(self,
+            workingtree_format, workingtree_format._matchingbzrdir)[1]
+        scenario_options["_workingtree_to_test_tree"] = converter
+        return name, scenario_options
 
 
-def test_suite():
-    result = TestSuite()
+def load_tests(basic_tests, module, loader):
+    result = loader.suiteClass()
+    # add the tests for this module
+    result.addTests(basic_tests)
+
     test_tree_implementations = [
+        'bzrlib.tests.tree_implementations.test_annotate_iter',
         'bzrlib.tests.tree_implementations.test_get_file_mtime',
+        'bzrlib.tests.tree_implementations.test_get_root_id',
         'bzrlib.tests.tree_implementations.test_get_symlink_target',
         'bzrlib.tests.tree_implementations.test_inv',
         'bzrlib.tests.tree_implementations.test_list_files',
+        'bzrlib.tests.tree_implementations.test_path_content_summary',
         'bzrlib.tests.tree_implementations.test_revision_tree',
         'bzrlib.tests.tree_implementations.test_test_trees',
         'bzrlib.tests.tree_implementations.test_tree',
         'bzrlib.tests.tree_implementations.test_walkdirs',
         ]
+
     adapter = TreeTestProviderAdapter(
         default_transport,
         # None here will cause a readonly decorator to be created
         # by the TestCaseWithTransport.get_readonly_transport method.
         None,
-        [(format, format._matchingbzrdir) for format in 
+        [(format, format._matchingbzrdir) for format in
          WorkingTreeFormat._formats.values() + _legacy_formats])
-    loader = TestLoader()
+
+    # add the tests for the sub modules
     adapt_modules(test_tree_implementations, adapter, loader, result)
-    result.addTests(loader.loadTestsFromModuleNames(['bzrlib.tests.tree_implementations']))
     return result
