@@ -17,6 +17,7 @@
 from bzrlib import (
     errors,
     tests,
+    conflicts,
     )
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.tree_implementations import TestCaseWithTree
@@ -119,3 +120,72 @@ class TestFileIds(TestCaseWithTree):
             self.assertRaises(errors.NoSuchId, tree.id2path, 'a')
         finally:
             tree.unlock()
+
+    def test_all_file_ids(self):
+        work_tree = self.make_branch_and_tree('wt')
+        tree = self.get_tree_no_parents_abc_content(work_tree)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertEqual(tree.all_file_ids(),
+                         set(['b-id', 'root-id', 'c-id', 'a-id']))
+
+
+class TestStoredKind(TestCaseWithTree):
+
+    def test_stored_kind(self):
+        tree = self.make_branch_and_tree('tree')
+        work_tree = self.make_branch_and_tree('wt')
+        tree = self.get_tree_no_parents_abc_content(work_tree)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertEqual('file', tree.stored_kind('a-id'))
+        self.assertEqual('directory', tree.stored_kind('b-id'))
+
+
+class TestFileContent(TestCaseWithTree):
+
+    def test_get_file(self):
+        work_tree = self.make_branch_and_tree('wt')
+        tree = self.get_tree_no_parents_abc_content_2(work_tree)
+        tree.lock_read()
+        try:
+            # Test lookup without path works
+            lines = tree.get_file('a-id').readlines()
+            self.assertEqual(['foobar\n'], lines)
+            # Test lookup with path works
+            lines = tree.get_file('a-id', path='a').readlines()
+            self.assertEqual(['foobar\n'], lines)
+        finally:
+            tree.unlock()
+
+
+class TestExtractFilesBytes(TestCaseWithTree):
+
+    def test_iter_files_bytes(self):
+        work_tree = self.make_branch_and_tree('wt')
+        self.build_tree_contents([('wt/foo', 'foo'),
+                                  ('wt/bar', 'bar'),
+                                  ('wt/baz', 'baz')])
+        work_tree.add(['foo', 'bar', 'baz'], ['foo-id', 'bar-id', 'baz-id'])
+        tree = self._convert_tree(work_tree)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        extracted = dict((i, ''.join(b)) for i, b in
+                         tree.iter_files_bytes([('foo-id', 'id1'),
+                                                ('bar-id', 'id2'),
+                                                ('baz-id', 'id3')]))
+        self.assertEqual('foo', extracted['id1'])
+        self.assertEqual('bar', extracted['id2'])
+        self.assertEqual('baz', extracted['id3'])
+        self.assertRaises(errors.NoSuchId, lambda: list(
+                          tree.iter_files_bytes(
+                          [('qux-id', 'file1-notpresent')])))
+
+
+class TestConflicts(TestCaseWithTree):
+
+    def test_conflicts(self):
+        """Tree.conflicts() should return a ConflictList instance."""
+        work_tree = self.make_branch_and_tree('wt')
+        tree = self._convert_tree(work_tree)
+        self.assertIsInstance(tree.conflicts(), conflicts.ConflictList)
