@@ -65,7 +65,8 @@ def capture_tree_contents(top):
         filenames.sort()
         for fn in filenames:
             fullpath = pathjoin(dirpath, fn)
-            assert not (fullpath[-1] in '@/')
+            if (fullpath[-1] in '@/'):
+                raise AssertionError(fullpath)
             info = os.lstat(fullpath)
             if stat.S_ISLNK(info.st_mode):
                 yield (fullpath + '@', os.readlink(fullpath))
