@@ -308,3 +308,10 @@ class TestRenameOne(TestCaseWithWorkingTree):
                                ('c', 'c-id')], tree)
         self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
                                ('a/b', 'b-id')], tree.basis_tree())
+
+    def test_rename_to_denormalised_fails(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        tree.add(['a'])
+        self.assertRaises((errors.InvalidNormalization, UnicodeEncodeError),
+            tree.rename_one, 'a', u'ba\u030arry')
