@@ -27,6 +27,7 @@ import urllib
 import urlparse
 
 from bzrlib import (
+    config,
     debug,
     errors,
     trace,
@@ -34,16 +35,7 @@ from bzrlib import (
     urlutils,
     )
 from bzrlib.smart import client, medium, protocol
-
-# must do this otherwise urllib can't parse the urls properly :(
-for scheme in ['ssh', 'bzr', 'bzr+loopback', 'bzr+ssh', 'bzr+http']:
-    transport.register_urlparse_netloc_protocol(scheme)
-del scheme
-
-
-# Port 4155 is the default port for bzr://, registered with IANA.
-BZR_DEFAULT_INTERFACE = '0.0.0.0'
-BZR_DEFAULT_PORT = 4155
+from bzrlib.symbol_versioning import (deprecated_method, one_four)
 
 
 class _SmartStat(object):
@@ -111,10 +103,16 @@ class RemoteTransport(transport.ConnectedTransport):
                     trace.mutter('hpss: Built a new medium: %s',
                                  medium.__class__.__name__)
             self._shared_connection = transport._SharedConnection(medium,
-                                                                  credentials)
+                                                                  credentials,
+                                                                  self.base)
+        else:
+            if medium is None:
+                # No medium was specified, so share the medium from the
+                # _from_transport.
+                medium = self._shared_connection.connection
 
         if _client is None:
-            self._client = client._SmartClient(self.get_shared_medium())
+            self._client = client._SmartClient(medium, self.base)
         else:
             self._client = _client
 
@@ -129,19 +127,17 @@ class RemoteTransport(transport.ConnectedTransport):
 
     def is_readonly(self):
         """Smart server transport can do read/write file operations."""
-        resp = self._call2('Transport.is_readonly')
-        if resp == ('yes', ):
-            return True
-        elif resp == ('no', ):
-            return False
-        elif (resp == ('error', "Generic bzr smart protocol error: "
-                                "bad request 'Transport.is_readonly'") or
-              resp == ('error', "Generic bzr smart protocol error: "
-                                "bad request u'Transport.is_readonly'")):
+        try:
+            resp = self._call2('Transport.is_readonly')
+        except errors.UnknownSmartMethod:
             # XXX: nasty hack: servers before 0.16 don't have a
             # 'Transport.is_readonly' verb, so we do what clients before 0.16
             # did: assume False.
             return False
+        if resp == ('yes', ):
+            return True
+        elif resp == ('no', ):
+            return False
         else:
             self._translate_error(resp)
         raise errors.UnexpectedSmartServerResponse(resp)
@@ -152,6 +148,7 @@ class RemoteTransport(transport.ConnectedTransport):
     def get_smart_medium(self):
         return self._get_connection()
 
+    @deprecated_method(one_four)
     def get_shared_medium(self):
         return self._get_shared_connection()
 
@@ -213,6 +210,13 @@ class RemoteTransport(transport.ConnectedTransport):
             self._serialise_optional_mode(mode))
         self._translate_error(resp)
 
+    def open_write_stream(self, relpath, mode=None):
+        """See Transport.open_write_stream."""
+        self.put_bytes(relpath, "", mode)
+        result = transport.AppendBasedFileStream(self, relpath)
+        transport._file_streams[self.abspath(relpath)] = result
+        return result
+
     def put_bytes(self, relpath, upload_contents, mode=None):
         # FIXME: upload_file is probably not safe for non-ascii characters -
         # should probably just pass all parameters as length-delimited
@@ -227,6 +231,7 @@ class RemoteTransport(transport.ConnectedTransport):
             (self._remote_path(relpath), self._serialise_optional_mode(mode)),
             upload_contents)
         self._translate_error(resp)
+        return len(upload_contents)
 
     def put_bytes_non_atomic(self, relpath, bytes, mode=None,
                              create_parent_dir=False,
@@ -283,7 +288,11 @@ class RemoteTransport(transport.ConnectedTransport):
         # the external path for RemoteTransports is the base
         return self.base
 
-    def readv(self, relpath, offsets):
+    def recommended_page_size(self):
+        """Return the recommended page size for this transport."""
+        return 64 * 1024
+        
+    def _readv(self, relpath, offsets):
         if not offsets:
             return
 
@@ -387,6 +396,12 @@ class RemoteTransport(transport.ConnectedTransport):
             else:
                 error_path = resp[1]
             raise errors.ReadError(error_path)
+        elif what == "PermissionDenied":
+            if orig_path is not None:
+                error_path = orig_path
+            else:
+                error_path = resp[1]
+            raise errors.PermissionDenied(error_path)
         else:
             raise errors.SmartProtocolError('unexpected smart server error: %r' % (resp,))
 
@@ -442,11 +457,7 @@ class RemoteTCPTransport(RemoteTransport):
     """
 
     def _build_medium(self):
-        assert self.base.startswith('bzr://')
-        port = self._port
-        if port is None:
-            port = BZR_DEFAULT_PORT
-        return medium.SmartTCPClientMedium(self._host, port), None
+        return medium.SmartTCPClientMedium(self._host, self._port), None
 
 
 class RemoteSSHTransport(RemoteTransport):
@@ -457,12 +468,13 @@ class RemoteSSHTransport(RemoteTransport):
     """
 
     def _build_medium(self):
-        assert self.base.startswith('bzr+ssh://')
         # ssh will prompt the user for a password if needed and if none is
         # provided but it will not give it back, so no credentials can be
         # stored.
+        location_config = config.LocationConfig(self.base)
+        bzr_remote_path = location_config.get_bzr_remote_path()
         return medium.SmartSSHClientMedium(self._host, self._port,
-                                           self._user, self._password), None
+            self._user, self._password, bzr_remote_path=bzr_remote_path), None
 
 
 class RemoteHTTPTransport(RemoteTransport):
@@ -477,8 +489,6 @@ class RemoteHTTPTransport(RemoteTransport):
     """
 
     def __init__(self, base, _from_transport=None, http_transport=None):
-        assert base.startswith('bzr+http://')
-
         if http_transport is None:
             # FIXME: the password may be lost here because it appears in the
             # url only for an intial construction (when the url came from the
@@ -498,7 +508,7 @@ class RemoteHTTPTransport(RemoteTransport):
         """After connecting, HTTP Transport only deals in relative URLs."""
         # Adjust the relpath based on which URL this smart transport is
         # connected to.
-        http_base = urlutils.normalize_url(self._http_transport.base)
+        http_base = urlutils.normalize_url(self.get_smart_medium().base)
         url = urlutils.join(self.base[len('bzr+'):], relpath)
         url = urlutils.normalize_url(url)
         return urlutils.relative_url(http_base, url)
@@ -515,26 +525,14 @@ class RemoteHTTPTransport(RemoteTransport):
         smart requests may be different).  This is so that the server doesn't
         have to handle .bzr/smart requests at arbitrary places inside .bzr
         directories, just at the initial URL the user uses.
-
-        The exception is parent paths (i.e. relative_url of "..").
         """
         if relative_url:
             abs_url = self.abspath(relative_url)
         else:
             abs_url = self.base
-        # We either use the exact same http_transport (for child locations), or
-        # a clone of the underlying http_transport (for parent locations).  This
-        # means we share the connection.
-        norm_base = urlutils.normalize_url(self.base)
-        norm_abs_url = urlutils.normalize_url(abs_url)
-        normalized_rel_url = urlutils.relative_url(norm_base, norm_abs_url)
-        if normalized_rel_url == ".." or normalized_rel_url.startswith("../"):
-            http_transport = self._http_transport.clone(normalized_rel_url)
-        else:
-            http_transport = self._http_transport
         return RemoteHTTPTransport(abs_url,
                                    _from_transport=self,
-                                   http_transport=http_transport)
+                                   http_transport=self._http_transport)
 
 
 def get_test_permutations():
