@@ -254,6 +254,15 @@ class Tree(object):
         """
         raise NotImplementedError(self.get_file_mtime)
 
+    def get_file_size(self, file_id):
+        """Return the size of a file in bytes.
+
+        This applies only to regular files.  If invoked on directories or
+        symlinks, it will return None.
+        :param file_id: The file-id of the file
+        """
+        raise NotImplementedError(self.get_file_size)
+
     def get_file_by_path(self, path):
         return self.get_file(self._inventory.path2id(path), path)
 
@@ -519,7 +528,6 @@ class EmptyTree(Tree):
         return False
 
     def kind(self, file_id):
-        assert self._inventory[file_id].kind == "directory"
         return "directory"
 
     def list_files(self, include_root=False):
@@ -568,7 +576,6 @@ def file_status(filename, old_tree, new_tree):
         # what happened to the file that used to have
         # this name.  There are two possibilities: either it was
         # deleted entirely, or renamed.
-        assert old_id
         if new_inv.has_id(old_id):
             return 'X', old_inv.id2path(old_id), new_inv.id2path(old_id)
         else:
@@ -895,17 +902,3 @@ class InterTree(InterObject):
             # the parent's path is necessarily known at this point.
             yield(file_id, (path, to_path), changed_content, versioned, parent,
                   name, kind, executable)
-
-
-# This was deprecated before 0.12, but did not have an official warning
-@symbol_versioning.deprecated_function(symbol_versioning.zero_twelve)
-def RevisionTree(*args, **kwargs):
-    """RevisionTree has moved to bzrlib.revisiontree.RevisionTree()
-
-    Accessing it as bzrlib.tree.RevisionTree has been deprecated as of
-    bzr 0.12.
-    """
-    from bzrlib.revisiontree import RevisionTree as _RevisionTree
-    return _RevisionTree(*args, **kwargs)
- 
-
