@@ -20,11 +20,16 @@
 
 import os
 
+from bzrlib import (
+    errors,
+    revision as _mod_revision,
+    )
 from bzrlib.branch import Branch
 from bzrlib.errors import BoundBranchOutOfDate
 
 
-def uncommit(branch, dry_run=False, verbose=False, revno=None, tree=None):
+def uncommit(branch, dry_run=False, verbose=False, revno=None, tree=None,
+             local=False):
     """Remove the last revision from the supplied branch.
 
     :param dry_run: Don't actually change anything
@@ -44,10 +49,15 @@ def uncommit(branch, dry_run=False, verbose=False, revno=None, tree=None):
         if tree is not None:
             pending_merges = tree.get_parent_ids()[1:]
 
-        master = branch.get_master_branch()
-        if master is not None:
-            master.lock_write()
-            unlockable.append(master)
+        if local:
+            master = None
+            if branch.get_bound_location() is None:
+                raise errors.LocalRequiresBoundBranch()
+        else:
+            master = branch.get_master_branch()
+            if master is not None:
+                master.lock_write()
+                unlockable.append(master)
         rh = branch.revision_history()
         if master is not None and rh[-1] != master.last_revision():
             raise BoundBranchOutOfDate(branch, master)
@@ -76,7 +86,7 @@ def uncommit(branch, dry_run=False, verbose=False, revno=None, tree=None):
             if master is not None:
                 master.set_revision_history(rh)
             branch.set_revision_history(rh)
-            new_tip = branch.last_revision()
+            new_tip = _mod_revision.ensure_null(branch.last_revision())
             if master is None:
                 hook_local = None
                 hook_master = branch
@@ -84,10 +94,13 @@ def uncommit(branch, dry_run=False, verbose=False, revno=None, tree=None):
                 hook_local = branch
                 hook_master = master
             for hook in Branch.hooks['post_uncommit']:
+                hook_new_tip = new_tip
+                if hook_new_tip == _mod_revision.NULL_REVISION:
+                    hook_new_tip = None
                 hook(hook_local, hook_master, old_revno, old_tip, new_revno,
-                    new_tip)
+                     hook_new_tip)
             if tree is not None:
-                if new_tip is not None:
+                if not _mod_revision.is_null(new_tip):
                     parents = [new_tip]
                 else:
                     parents = []
