@@ -30,17 +30,15 @@ from bzrlib import (
     revision,
     ui,
     )
+from bzrlib.graph import Graph
 from bzrlib.transport.memory import MemoryTransport
 """)
 
 from cStringIO import StringIO
 
 from bzrlib.inter import InterObject
+from bzrlib.symbol_versioning import *
 from bzrlib.textmerge import TextMerge
-from bzrlib.symbol_versioning import (deprecated_function,
-        deprecated_method,
-        zero_eight,
-        )
 
 
 class VersionedFile(object):
@@ -57,10 +55,6 @@ class VersionedFile(object):
     Texts are identified by a version-id string.
     """
 
-    def __init__(self, access_mode):
-        self.finished = False
-        self._access_mode = access_mode
-
     @staticmethod
     def check_not_reserved_id(version_id):
         revision.check_not_reserved_id(version_id)
@@ -69,18 +63,11 @@ class VersionedFile(object):
         """Copy this versioned file to name on transport."""
         raise NotImplementedError(self.copy_to)
 
-    @deprecated_method(zero_eight)
-    def names(self):
-        """Return a list of all the versions in this versioned file.
-
-        Please use versionedfile.versions() now.
-        """
-        return self.versions()
-
     def versions(self):
         """Return a unsorted list of versions."""
         raise NotImplementedError(self.versions)
 
+    @deprecated_method(one_four)
     def has_ghost(self, version_id):
         """Returns whether version is present as a ghost."""
         raise NotImplementedError(self.has_ghost)
@@ -89,43 +76,9 @@ class VersionedFile(object):
         """Returns whether version is present."""
         raise NotImplementedError(self.has_version)
 
-    def add_delta(self, version_id, parents, delta_parent, sha1, noeol, delta):
-        """Add a text to the versioned file via a pregenerated delta.
-
-        :param version_id: The version id being added.
-        :param parents: The parents of the version_id.
-        :param delta_parent: The parent this delta was created against.
-        :param sha1: The sha1 of the full text.
-        :param delta: The delta instructions. See get_delta for details.
-        """
-        version_id = osutils.safe_revision_id(version_id)
-        parents = [osutils.safe_revision_id(v) for v in parents]
-        self._check_write_ok()
-        if self.has_version(version_id):
-            raise errors.RevisionAlreadyPresent(version_id, self)
-        return self._add_delta(version_id, parents, delta_parent, sha1, noeol, delta)
-
-    def _add_delta(self, version_id, parents, delta_parent, sha1, noeol, delta):
-        """Class specific routine to add a delta.
-
-        This generic version simply applies the delta to the delta_parent and
-        then inserts it.
-        """
-        # strip annotation from delta
-        new_delta = []
-        for start, stop, delta_len, delta_lines in delta:
-            new_delta.append((start, stop, delta_len, [text for origin, text in delta_lines]))
-        if delta_parent is not None:
-            parent_full = self.get_lines(delta_parent)
-        else:
-            parent_full = []
-        new_full = self._apply_delta(parent_full, new_delta)
-        # its impossible to have noeol on an empty file
-        if noeol and new_full[-1][-1] == '\n':
-            new_full[-1] = new_full[-1][:-1]
-        self.add_lines(version_id, parents, new_full)
-
-    def add_lines(self, version_id, parents, lines, parent_texts=None):
+    def add_lines(self, version_id, parents, lines, parent_texts=None,
+        left_matching_blocks=None, nostore_sha=None, random_id=False,
+        check_content=True):
         """Add a single text on top of the versioned file.
 
         Must raise RevisionAlreadyPresent if the new version is
@@ -133,37 +86,58 @@ class VersionedFile(object):
 
         Must raise RevisionNotPresent if any of the given parents are
         not present in file history.
-        :param parent_texts: An optional dictionary containing the opaque 
-             representations of some or all of the parents of 
-             version_id to allow delta optimisations. 
-             VERY IMPORTANT: the texts must be those returned
-             by add_lines or data corruption can be caused.
-        :return: An opaque representation of the inserted version which can be
-                 provided back to future add_lines calls in the parent_texts
-                 dictionary.
-        """
-        version_id = osutils.safe_revision_id(version_id)
-        parents = [osutils.safe_revision_id(v) for v in parents]
-        self._check_write_ok()
-        return self._add_lines(version_id, parents, lines, parent_texts)
 
-    def _add_lines(self, version_id, parents, lines, parent_texts):
+        :param lines: A list of lines. Each line must be a bytestring. And all
+            of them except the last must be terminated with \n and contain no
+            other \n's. The last line may either contain no \n's or a single
+            terminated \n. If the lines list does meet this constraint the add
+            routine may error or may succeed - but you will be unable to read
+            the data back accurately. (Checking the lines have been split
+            correctly is expensive and extremely unlikely to catch bugs so it
+            is not done at runtime unless check_content is True.)
+        :param parent_texts: An optional dictionary containing the opaque 
+            representations of some or all of the parents of version_id to
+            allow delta optimisations.  VERY IMPORTANT: the texts must be those
+            returned by add_lines or data corruption can be caused.
+        :param left_matching_blocks: a hint about which areas are common
+            between the text and its left-hand-parent.  The format is
+            the SequenceMatcher.get_matching_blocks format.
+        :param nostore_sha: Raise ExistingContent and do not add the lines to
+            the versioned file if the digest of the lines matches this.
+        :param random_id: If True a random id has been selected rather than
+            an id determined by some deterministic process such as a converter
+            from a foreign VCS. When True the backend may choose not to check
+            for uniqueness of the resulting key within the versioned file, so
+            this should only be done when the result is expected to be unique
+            anyway.
+        :param check_content: If True, the lines supplied are verified to be
+            bytestrings that are correctly formed lines.
+        :return: The text sha1, the number of bytes in the text, and an opaque
+                 representation of the inserted version which can be provided
+                 back to future add_lines calls in the parent_texts dictionary.
+        """
+        self._check_write_ok()
+        return self._add_lines(version_id, parents, lines, parent_texts,
+            left_matching_blocks, nostore_sha, random_id, check_content)
+
+    def _add_lines(self, version_id, parents, lines, parent_texts,
+        left_matching_blocks, nostore_sha, random_id, check_content):
         """Helper to do the class specific add_lines."""
         raise NotImplementedError(self.add_lines)
 
     def add_lines_with_ghosts(self, version_id, parents, lines,
-                              parent_texts=None):
+        parent_texts=None, nostore_sha=None, random_id=False,
+        check_content=True, left_matching_blocks=None):
         """Add lines to the versioned file, allowing ghosts to be present.
         
-        This takes the same parameters as add_lines.
+        This takes the same parameters as add_lines and returns the same.
         """
-        version_id = osutils.safe_revision_id(version_id)
-        parents = [osutils.safe_revision_id(v) for v in parents]
         self._check_write_ok()
         return self._add_lines_with_ghosts(version_id, parents, lines,
-                                           parent_texts)
+            parent_texts, nostore_sha, random_id, check_content, left_matching_blocks)
 
-    def _add_lines_with_ghosts(self, version_id, parents, lines, parent_texts):
+    def _add_lines_with_ghosts(self, version_id, parents, lines, parent_texts,
+        nostore_sha, random_id, check_content, left_matching_blocks):
         """Helper to do class specific add_lines_with_ghosts."""
         raise NotImplementedError(self.add_lines_with_ghosts)
 
@@ -183,13 +157,7 @@ class VersionedFile(object):
             if '\n' in line[:-1]:
                 raise errors.BzrBadParameterContainsNewline("lines")
 
-    def _check_write_ok(self):
-        """Is the versioned file marked as 'finished' ? Raise if it is."""
-        if self.finished:
-            raise errors.OutSideTransaction()
-        if self._access_mode != 'w':
-            raise errors.ReadOnlyObjectDirtiedError(self)
-
+    @deprecated_method(one_four)
     def enable_cache(self):
         """Tell this versioned file that it should cache any data it reads.
         
@@ -197,6 +165,7 @@ class VersionedFile(object):
         """
         pass
     
+    @deprecated_method(one_four)
     def clear_cache(self):
         """Remove any data cached in the versioned file object.
 
@@ -204,6 +173,7 @@ class VersionedFile(object):
         """
         pass
 
+    @deprecated_method(one_four)
     def clone_text(self, new_version_id, old_version_id, parents):
         """Add an identical text to old_version_id as new_version_id.
 
@@ -212,73 +182,39 @@ class VersionedFile(object):
 
         Must raise RevisionAlreadyPresent if the new version is
         already present in file history."""
-        new_version_id = osutils.safe_revision_id(new_version_id)
-        old_version_id = osutils.safe_revision_id(old_version_id)
         self._check_write_ok()
-        return self._clone_text(new_version_id, old_version_id, parents)
+        return self.add_lines(new_version_id, parents,
+            self.get_lines(old_version_id))
 
-    def _clone_text(self, new_version_id, old_version_id, parents):
-        """Helper function to do the _clone_text work."""
-        raise NotImplementedError(self.clone_text)
-
-    def create_empty(self, name, transport, mode=None):
-        """Create a new versioned file of this exact type.
-
-        :param name: the file name
-        :param transport: the transport
-        :param mode: optional file mode.
-        """
-        raise NotImplementedError(self.create_empty)
-
-    def fix_parents(self, version_id, new_parents):
-        """Fix the parents list for version.
+    def get_format_signature(self):
+        """Get a text description of the data encoding in this file.
         
-        This is done by appending a new version to the index
-        with identical data except for the parents list.
-        the parents list must be a superset of the current
-        list.
+        :since: 0.90
         """
-        version_id = osutils.safe_revision_id(version_id)
-        new_parents = [osutils.safe_revision_id(p) for p in new_parents]
-        self._check_write_ok()
-        return self._fix_parents(version_id, new_parents)
-
-    def _fix_parents(self, version_id, new_parents):
-        """Helper for fix_parents."""
-        raise NotImplementedError(self.fix_parents)
-
-    def get_delta(self, version):
-        """Get a delta for constructing version from some other version.
-        
-        :return: (delta_parent, sha1, noeol, delta)
-        Where delta_parent is a version id or None to indicate no parent.
-        """
-        raise NotImplementedError(self.get_delta)
-
-    def get_deltas(self, version_ids):
-        """Get multiple deltas at once for constructing versions.
-        
-        :return: dict(version_id:(delta_parent, sha1, noeol, delta))
-        Where delta_parent is a version id or None to indicate no parent, and
-        version_id is the version_id created by that delta.
-        """
-        result = {}
-        for version_id in version_ids:
-            result[version_id] = self.get_delta(version_id)
-        return result
+        raise NotImplementedError(self.get_format_signature)
 
     def make_mpdiffs(self, version_ids):
-        """Create multiparent diffs for specified versions"""
+        """Create multiparent diffs for specified versions."""
         knit_versions = set()
+        knit_versions.update(version_ids)
+        parent_map = self.get_parent_map(version_ids)
         for version_id in version_ids:
-            knit_versions.add(version_id)
-            knit_versions.update(self.get_parents(version_id))
+            try:
+                knit_versions.update(parent_map[version_id])
+            except KeyError:
+                raise RevisionNotPresent(version_id, self)
+        # We need to filter out ghosts, because we can't diff against them.
+        knit_versions = set(self.get_parent_map(knit_versions).keys())
         lines = dict(zip(knit_versions,
             self._get_lf_split_line_list(knit_versions)))
         diffs = []
         for version_id in version_ids:
             target = lines[version_id]
-            parents = [lines[p] for p in self.get_parents(version_id)]
+            try:
+                parents = [lines[p] for p in parent_map[version_id] if p in
+                    knit_versions]
+            except KeyError:
+                raise RevisionNotPresent(version_id, self)
             if len(parents) > 0:
                 left_parent_blocks = self._extract_blocks(version_id,
                                                           parents[0], target)
@@ -292,31 +228,56 @@ class VersionedFile(object):
         return None
 
     def add_mpdiffs(self, records):
-        """Add mpdiffs to this versionedfile
+        """Add mpdiffs to this VersionedFile.
 
         Records should be iterables of version, parents, expected_sha1,
-        mpdiff.  mpdiff should be a MultiParent instance.
+        mpdiff. mpdiff should be a MultiParent instance.
         """
+        # Does this need to call self._check_write_ok()? (IanC 20070919)
         vf_parents = {}
-        for version, parents, expected_sha1, mpdiff in records:
-            mpvf = multiparent.MultiMemoryVersionedFile()
-            needed_parents = [p for p in parents if not mpvf.has_version(p)]
-            parent_lines = self._get_lf_split_line_list(needed_parents)
-            for parent_id, lines in zip(needed_parents, parent_lines):
-                mpvf.add_version(lines, parent_id, [])
-            mpvf.add_diff(mpdiff, version, parents)
-            lines = mpvf.get_line_list([version])[0]
-            version_text = self.add_lines(version, parents, lines, vf_parents)
+        mpvf = multiparent.MultiMemoryVersionedFile()
+        versions = []
+        for version, parent_ids, expected_sha1, mpdiff in records:
+            versions.append(version)
+            mpvf.add_diff(mpdiff, version, parent_ids)
+        needed_parents = set()
+        for version, parent_ids, expected_sha1, mpdiff in records:
+            needed_parents.update(p for p in parent_ids
+                                  if not mpvf.has_version(p))
+        present_parents = set(self.get_parent_map(needed_parents).keys())
+        for parent_id, lines in zip(present_parents,
+                                 self._get_lf_split_line_list(present_parents)):
+            mpvf.add_version(lines, parent_id, [])
+        for (version, parent_ids, expected_sha1, mpdiff), lines in\
+            zip(records, mpvf.get_line_list(versions)):
+            if len(parent_ids) == 1:
+                left_matching_blocks = list(mpdiff.get_matching_blocks(0,
+                    mpvf.get_diff(parent_ids[0]).num_lines()))
+            else:
+                left_matching_blocks = None
+            try:
+                _, _, version_text = self.add_lines_with_ghosts(version,
+                    parent_ids, lines, vf_parents,
+                    left_matching_blocks=left_matching_blocks)
+            except NotImplementedError:
+                # The vf can't handle ghosts, so add lines normally, which will
+                # (reasonably) fail if there are ghosts in the data.
+                _, _, version_text = self.add_lines(version,
+                    parent_ids, lines, vf_parents,
+                    left_matching_blocks=left_matching_blocks)
             vf_parents[version] = version_text
-            if expected_sha1 != self.get_sha1(version):
+        for (version, parent_ids, expected_sha1, mpdiff), sha1 in\
+             zip(records, self.get_sha1s(versions)):
+            if expected_sha1 != sha1:
                 raise errors.VersionedFileInvalidChecksum(version)
 
+    @deprecated_method(one_four)
     def get_sha1(self, version_id):
         """Get the stored sha1 sum for the given revision.
         
-        :param name: The name of the version to lookup
+        :param version_id: The name of the version to lookup
         """
-        raise NotImplementedError(self.get_sha1)
+        return self.get_sha1s([version_id])[0]
 
     def get_sha1s(self, version_ids):
         """Get the stored sha1 sums for the given revisions.
@@ -324,12 +285,8 @@ class VersionedFile(object):
         :param version_ids: The names of the versions to lookup
         :return: a list of sha1s in order according to the version_ids
         """
-        raise NotImplementedError(self.get_sha1)
+        raise NotImplementedError(self.get_sha1s)
 
-    def get_suffixes(self):
-        """Return the file suffixes associated with this versioned file."""
-        raise NotImplementedError(self.get_suffixes)
-    
     def get_text(self, version_id):
         """Return version contents as a text string.
 
@@ -382,7 +339,8 @@ class VersionedFile(object):
         but are not explicitly marked.
         """
         raise NotImplementedError(self.get_ancestry_with_ghosts)
-        
+    
+    @deprecated_method(one_four)
     def get_graph(self, version_ids=None):
         """Return a graph from the versioned file. 
         
@@ -391,20 +349,31 @@ class VersionedFile(object):
                             None means retrieve all versions.
         """
         if version_ids is None:
-            return dict(self.iter_parents(self.versions()))
-        result = {}
-        pending = set(osutils.safe_revision_id(v) for v in version_ids)
-        while pending:
-            this_iteration = pending
-            pending = set()
-            for version, parents in self.iter_parents(this_iteration):
-                result[version] = parents
-                for parent in parents:
-                    if parent in result:
-                        continue
-                    pending.add(parent)
+            result = self.get_parent_map(self.versions())
+        else:
+            result = {}
+            pending = set(version_ids)
+            while pending:
+                this_iteration = pending
+                pending = set()
+                parents = self.get_parent_map(this_iteration)
+                for version, parents in parents.iteritems():
+                    result[version] = parents
+                    for parent in parents:
+                        if parent in result:
+                            continue
+                        pending.add(parent)
+        references = set()
+        for parents in result.itervalues():
+            references.update(parents)
+        existing_parents = self.get_parent_map(references)
+        for key, parents in result.iteritems():
+            present_parents = [parent for parent in parents if parent in
+                existing_parents]
+            result[key] = tuple(present_parents)
         return result
 
+    @deprecated_method(one_four)
     def get_graph_with_ghosts(self):
         """Return a graph for the entire versioned file.
         
@@ -413,21 +382,31 @@ class VersionedFile(object):
         """
         raise NotImplementedError(self.get_graph_with_ghosts)
 
-    @deprecated_method(zero_eight)
-    def parent_names(self, version):
-        """Return version names for parents of a version.
-        
-        See get_parents for the current api.
-        """
-        return self.get_parents(version)
+    def get_parent_map(self, version_ids):
+        """Get a map of the parents of version_ids.
 
+        :param version_ids: The version ids to look up parents for.
+        :return: A mapping from version id to parents.
+        """
+        raise NotImplementedError(self.get_parent_map)
+
+    @deprecated_method(one_four)
     def get_parents(self, version_id):
         """Return version names for parents of a version.
 
         Must raise RevisionNotPresent if version is not present in
         file history.
         """
-        raise NotImplementedError(self.get_parents)
+        try:
+            all = self.get_parent_map([version_id])[version_id]
+        except KeyError:
+            raise errors.RevisionNotPresent(version_id, self)
+        result = []
+        parent_parents = self.get_parent_map(all)
+        for version_id in all:
+            if version_id in parent_parents:
+                result.append(version_id)
+        return result
 
     def get_parents_with_ghosts(self, version_id):
         """Return version names for parents of version_id.
@@ -438,28 +417,28 @@ class VersionedFile(object):
         Ghosts that are known about will be included in the parent list,
         but are not explicitly marked.
         """
-        raise NotImplementedError(self.get_parents_with_ghosts)
+        try:
+            return list(self.get_parent_map([version_id])[version_id])
+        except KeyError:
+            raise errors.RevisionNotPresent(version_id, self)
 
+    @deprecated_method(one_four)
     def annotate_iter(self, version_id):
         """Yield list of (version-id, line) pairs for the specified
         version.
 
-        Must raise RevisionNotPresent if any of the given versions are
+        Must raise RevisionNotPresent if the given version is
         not present in file history.
         """
-        raise NotImplementedError(self.annotate_iter)
+        return iter(self.annotate(version_id))
 
     def annotate(self, version_id):
-        return list(self.annotate_iter(version_id))
+        """Return a list of (version-id, line) tuples for version_id.
 
-    def _apply_delta(self, lines, delta):
-        """Apply delta to lines."""
-        lines = list(lines)
-        offset = 0
-        for start, end, count, delta_lines in delta:
-            lines[offset+start:offset+end] = delta_lines
-            offset = offset + (start - end) + count
-        return lines
+        :raise RevisionNotPresent: If the given version is
+        not present in file history.
+        """
+        raise NotImplementedError(self.annotate)
 
     def join(self, other, pb=None, msg=None, version_ids=None,
              ignore_missing=False):
@@ -469,8 +448,8 @@ class VersionedFile(object):
         incorporated into this versioned file.
 
         Must raise RevisionNotPresent if any of the specified versions
-        are not present in the other files history unless ignore_missing
-        is supplied when they are silently skipped.
+        are not present in the other file's history unless ignore_missing
+        is supplied in which case they are silently skipped.
         """
         self._check_write_ok()
         return InterVersionedFile.get(other, self).join(
@@ -479,15 +458,16 @@ class VersionedFile(object):
             version_ids,
             ignore_missing)
 
-    def iter_lines_added_or_present_in_versions(self, version_ids=None, 
+    def iter_lines_added_or_present_in_versions(self, version_ids=None,
                                                 pb=None):
         """Iterate over the lines in the versioned file from version_ids.
 
-        This may return lines from other versions, and does not return the
-        specific version marker at this point. The api may be changed
-        during development to include the version that the versioned file
-        thinks is relevant, but given that such hints are just guesses,
-        its better not to have it if we don't need it.
+        This may return lines from other versions. Each item the returned
+        iterator yields is a tuple of a line and a text version that that line
+        is present in (not introduced in).
+
+        Ordering of results is in whatever order is most suitable for the
+        underlying storage format.
 
         If a progress bar is supplied, it may be used to indicate progress.
         The caller is responsible for cleaning up progress bars (because this
@@ -495,9 +475,12 @@ class VersionedFile(object):
 
         NOTES: Lines are normalised: they will all have \n terminators.
                Lines are returned in arbitrary order.
+
+        :return: An iterator over (line, version_id).
         """
         raise NotImplementedError(self.iter_lines_added_or_present_in_versions)
 
+    @deprecated_method(one_four)
     def iter_parents(self, version_ids):
         """Iterate through the parents for many version ids.
 
@@ -507,40 +490,7 @@ class VersionedFile(object):
             The order is undefined, allowing for different optimisations in
             the underlying implementation.
         """
-        for version_id in version_ids:
-            try:
-                yield version_id, tuple(self.get_parents(version_id))
-            except errors.RevisionNotPresent:
-                pass
-
-    def transaction_finished(self):
-        """The transaction that this file was opened in has finished.
-
-        This records self.finished = True and should cause all mutating
-        operations to error.
-        """
-        self.finished = True
-
-    @deprecated_method(zero_eight)
-    def walk(self, version_ids=None):
-        """Walk the versioned file as a weave-like structure, for
-        versions relative to version_ids.  Yields sequence of (lineno,
-        insert, deletes, text) for each relevant line.
-
-        Must raise RevisionNotPresent if any of the specified versions
-        are not present in the file history.
-
-        :param version_ids: the version_ids to walk with respect to. If not
-                            supplied the entire weave-like structure is walked.
-
-        walk is deprecated in favour of iter_lines_added_or_present_in_versions
-        """
-        raise NotImplementedError(self.walk)
-
-    @deprecated_method(zero_eight)
-    def iter_names(self):
-        """Walk the names list."""
-        return iter(self.versions())
+        return self.get_parent_map(version_ids).iteritems()
 
     def plan_merge(self, ver_a, ver_b):
         """Return pseudo-annotation indicating how the two versions merge.
@@ -569,6 +519,135 @@ class VersionedFile(object):
         return PlanWeaveMerge(plan, a_marker, b_marker).merge_lines()[0]
 
 
+class _PlanMergeVersionedFile(object):
+    """A VersionedFile for uncommitted and committed texts.
+
+    It is intended to allow merges to be planned with working tree texts.
+    It implements only the small part of the VersionedFile interface used by
+    PlanMerge.  It falls back to multiple versionedfiles for data not stored in
+    _PlanMergeVersionedFile itself.
+    """
+
+    def __init__(self, file_id, fallback_versionedfiles=None):
+        """Constuctor
+
+        :param file_id: Used when raising exceptions.
+        :param fallback_versionedfiles: If supplied, the set of fallbacks to
+            use.  Otherwise, _PlanMergeVersionedFile.fallback_versionedfiles
+            can be appended to later.
+        """
+        self._file_id = file_id
+        if fallback_versionedfiles is None:
+            self.fallback_versionedfiles = []
+        else:
+            self.fallback_versionedfiles = fallback_versionedfiles
+        self._parents = {}
+        self._lines = {}
+
+    def plan_merge(self, ver_a, ver_b, base=None):
+        """See VersionedFile.plan_merge"""
+        from bzrlib.merge import _PlanMerge
+        if base is None:
+            return _PlanMerge(ver_a, ver_b, self).plan_merge()
+        old_plan = list(_PlanMerge(ver_a, base, self).plan_merge())
+        new_plan = list(_PlanMerge(ver_a, ver_b, self).plan_merge())
+        return _PlanMerge._subtract_plans(old_plan, new_plan)
+
+    def plan_lca_merge(self, ver_a, ver_b, base=None):
+        from bzrlib.merge import _PlanLCAMerge
+        graph = self._get_graph()
+        new_plan = _PlanLCAMerge(ver_a, ver_b, self, graph).plan_merge()
+        if base is None:
+            return new_plan
+        old_plan = _PlanLCAMerge(ver_a, base, self, graph).plan_merge()
+        return _PlanLCAMerge._subtract_plans(list(old_plan), list(new_plan))
+
+    def add_lines(self, version_id, parents, lines):
+        """See VersionedFile.add_lines
+
+        Lines are added locally, not fallback versionedfiles.  Also, ghosts are
+        permitted.  Only reserved ids are permitted.
+        """
+        if not revision.is_reserved_id(version_id):
+            raise ValueError('Only reserved ids may be used')
+        if parents is None:
+            raise ValueError('Parents may not be None')
+        if lines is None:
+            raise ValueError('Lines may not be None')
+        self._parents[version_id] = tuple(parents)
+        self._lines[version_id] = lines
+
+    def get_lines(self, version_id):
+        """See VersionedFile.get_ancestry"""
+        lines = self._lines.get(version_id)
+        if lines is not None:
+            return lines
+        for versionedfile in self.fallback_versionedfiles:
+            try:
+                return versionedfile.get_lines(version_id)
+            except errors.RevisionNotPresent:
+                continue
+        else:
+            raise errors.RevisionNotPresent(version_id, self._file_id)
+
+    def get_ancestry(self, version_id, topo_sorted=False):
+        """See VersionedFile.get_ancestry.
+
+        Note that this implementation assumes that if a VersionedFile can
+        answer get_ancestry at all, it can give an authoritative answer.  In
+        fact, ghosts can invalidate this assumption.  But it's good enough
+        99% of the time, and far cheaper/simpler.
+
+        Also note that the results of this version are never topologically
+        sorted, and are a set.
+        """
+        if topo_sorted:
+            raise ValueError('This implementation does not provide sorting')
+        parents = self._parents.get(version_id)
+        if parents is None:
+            for vf in self.fallback_versionedfiles:
+                try:
+                    return vf.get_ancestry(version_id, topo_sorted=False)
+                except errors.RevisionNotPresent:
+                    continue
+            else:
+                raise errors.RevisionNotPresent(version_id, self._file_id)
+        ancestry = set([version_id])
+        for parent in parents:
+            ancestry.update(self.get_ancestry(parent, topo_sorted=False))
+        return ancestry
+
+    def get_parent_map(self, version_ids):
+        """See VersionedFile.get_parent_map"""
+        result = {}
+        pending = set(version_ids)
+        for key in version_ids:
+            try:
+                result[key] = self._parents[key]
+            except KeyError:
+                pass
+        pending = pending - set(result.keys())
+        for versionedfile in self.fallback_versionedfiles:
+            parents = versionedfile.get_parent_map(pending)
+            result.update(parents)
+            pending = pending - set(parents.keys())
+            if not pending:
+                return result
+        return result
+
+    def _get_graph(self):
+        from bzrlib.graph import (
+            DictParentsProvider,
+            Graph,
+            _StackedParentsProvider,
+            )
+        from bzrlib.repofmt.knitrepo import _KnitParentsProvider
+        parent_providers = [DictParentsProvider(self._parents)]
+        for vf in self.fallback_versionedfiles:
+            parent_providers.append(_KnitParentsProvider(vf))
+        return Graph(_StackedParentsProvider(parent_providers))
+
+
 class PlanWeaveMerge(TextMerge):
     """Weave merge that takes a plan as its input.
     
@@ -626,15 +705,22 @@ class PlanWeaveMerge(TextMerge):
             elif state == 'new-b':
                 ch_b = True
                 lines_b.append(line)
+            elif state == 'conflicted-a':
+                ch_b = ch_a = True
+                lines_a.append(line)
+            elif state == 'conflicted-b':
+                ch_b = ch_a = True
+                lines_b.append(line)
             else:
-                assert state in ('irrelevant', 'ghost-a', 'ghost-b', 
-                                 'killed-base', 'killed-both'), state
+                if state not in ('irrelevant', 'ghost-a', 'ghost-b',
+                        'killed-base', 'killed-both'):
+                    raise AssertionError(state)
         for struct in outstanding_struct():
             yield struct
 
 
 class WeaveMerge(PlanWeaveMerge):
-    """Weave merge that takes a VersionedFile and two versions as its input"""
+    """Weave merge that takes a VersionedFile and two versions as its input."""
 
     def __init__(self, versionedfile, ver_a, ver_b, 
         a_marker=PlanWeaveMerge.A_MARKER, b_marker=PlanWeaveMerge.B_MARKER):
@@ -643,7 +729,7 @@ class WeaveMerge(PlanWeaveMerge):
 
 
 class InterVersionedFile(InterObject):
-    """This class represents operations taking place between two versionedfiles..
+    """This class represents operations taking place between two VersionedFiles.
 
     Its instances have methods like join, and contain
     references to the source and target versionedfiles these operations can be 
@@ -664,24 +750,17 @@ class InterVersionedFile(InterObject):
         incorporated into this versioned file.
 
         Must raise RevisionNotPresent if any of the specified versions
-        are not present in the other files history unless ignore_missing is 
-        supplied when they are silently skipped.
+        are not present in the other file's history unless ignore_missing is 
+        supplied in which case they are silently skipped.
         """
-        # the default join: 
-        # - if the target is empty, just add all the versions from 
-        #   source to target, otherwise:
-        # - make a temporary versioned file of type target
-        # - insert the source content into it one at a time
-        # - join them
-        if not self.target.versions():
-            target = self.target
-        else:
-            # Make a new target-format versioned file. 
-            temp_source = self.target.create_empty("temp", MemoryTransport())
-            target = temp_source
+        target = self.target
         version_ids = self._get_source_version_ids(version_ids, ignore_missing)
-        graph = self.source.get_graph(version_ids)
-        order = tsort.topo_sort(graph.items())
+        graph = Graph(self.source)
+        search = graph._make_breadth_first_searcher(version_ids)
+        transitive_ids = set()
+        map(transitive_ids.update, list(search))
+        parent_map = self.source.get_parent_map(transitive_ids)
+        order = tsort.topo_sort(parent_map.items())
         pb = ui.ui_factory.nested_progress_bar()
         parent_texts = {}
         try:
@@ -698,30 +777,17 @@ class InterVersionedFile(InterObject):
             # TODO: remove parent texts when they are not relevant any more for 
             # memory pressure reduction. RBC 20060313
             # pb.update('Converting versioned data', 0, len(order))
-            # deltas = self.source.get_deltas(order)
+            total = len(order)
             for index, version in enumerate(order):
-                pb.update('Converting versioned data', index, len(order))
-                parent_text = target.add_lines(version,
-                                               self.source.get_parents(version),
+                pb.update('Converting versioned data', index, total)
+                if version in target:
+                    continue
+                _, _, parent_text = target.add_lines(version,
+                                               parent_map[version],
                                                self.source.get_lines(version),
                                                parent_texts=parent_texts)
                 parent_texts[version] = parent_text
-                #delta_parent, sha1, noeol, delta = deltas[version]
-                #target.add_delta(version,
-                #                 self.source.get_parents(version),
-                #                 delta_parent,
-                #                 sha1,
-                #                 noeol,
-                #                 delta)
-                #target.get_lines(version)
-            
-            # this should hit the native code path for target
-            if target is not self.target:
-                return self.target.join(temp_source,
-                                        pb,
-                                        msg,
-                                        version_ids,
-                                        ignore_missing)
+            return total
         finally:
             pb.finished()
 
@@ -739,7 +805,6 @@ class InterVersionedFile(InterObject):
             # None cannot be in source.versions
             return set(self.source.versions())
         else:
-            version_ids = [osutils.safe_revision_id(v) for v in version_ids]
             if ignore_missing:
                 return set(self.source.versions()).intersection(set(version_ids))
             else:
