@@ -88,10 +88,6 @@ from bzrlib.errors import (WeaveError, WeaveFormatError, WeaveParentMismatch,
 import bzrlib.errors as errors
 from bzrlib.osutils import sha_strings
 import bzrlib.patiencediff
-from bzrlib.symbol_versioning import (deprecated_method,
-        deprecated_function,
-        zero_eight,
-        )
 from bzrlib.tsort import topo_sort
 from bzrlib.versionedfile import VersionedFile, InterVersionedFile
 from bzrlib.weavefile import _read_weave_v5, write_weave_v5
@@ -189,7 +185,13 @@ class Weave(VersionedFile):
     __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
                  '_weave_name', '_matcher']
     
-    def __init__(self, weave_name=None, access_mode='w', matcher=None):
+    def __init__(self, weave_name=None, access_mode='w', matcher=None, get_scope=None):
+        """Create a weave.
+
+        :param get_scope: A callable that returns an opaque object to be used
+            for detecting when this weave goes out of scope (should stop
+            answering requests or allowing mutation).
+        """
         super(Weave, self).__init__(access_mode)
         self._weave = []
         self._parents = []
@@ -201,10 +203,22 @@ class Weave(VersionedFile):
             self._matcher = bzrlib.patiencediff.PatienceSequenceMatcher
         else:
             self._matcher = matcher
+        if get_scope is None:
+            get_scope = lambda:None
+        self._get_scope = get_scope
+        self._scope = get_scope()
+        self._access_mode = access_mode
 
     def __repr__(self):
         return "Weave(%r)" % self._weave_name
 
+    def _check_write_ok(self):
+        """Is the versioned file marked as 'finished' ? Raise if it is."""
+        if self._get_scope() != self._scope:
+            raise errors.OutSideTransaction()
+        if self._access_mode != 'w':
+            raise errors.ReadOnlyObjectDirtiedError(self)
+
     def copy(self):
         """Return a deep copy of self.
         
@@ -228,23 +242,9 @@ class Weave(VersionedFile):
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    @deprecated_method(zero_eight)
-    def idx_to_name(self, index):
-        """Old public interface, the public interface is all names now."""
-        return index
-
     def _idx_to_name(self, version):
         return self._names[version]
 
-    @deprecated_method(zero_eight)
-    def lookup(self, name):
-        """Backwards compatibility thunk:
-
-        Return name, as name is valid in the api now, and spew deprecation
-        warnings everywhere.
-        """
-        return name
-
     def _lookup(self, name):
         """Convert symbolic version name to index."""
         self.check_not_reserved_id(name)
@@ -253,16 +253,6 @@ class Weave(VersionedFile):
         except KeyError:
             raise RevisionNotPresent(name, self._weave_name)
 
-    @deprecated_method(zero_eight)
-    def iter_names(self):
-        """Deprecated convenience function, please see VersionedFile.names()."""
-        return iter(self.names())
-
-    @deprecated_method(zero_eight)
-    def names(self):
-        """See Weave.versions for the current api."""
-        return self.versions()
-
     def versions(self):
         """See VersionedFile.versions."""
         return self._names[:]
@@ -273,165 +263,19 @@ class Weave(VersionedFile):
 
     __contains__ = has_version
 
-    def get_delta(self, version_id):
-        """See VersionedFile.get_delta."""
-        return self.get_deltas([version_id])[version_id]
-
-    def get_deltas(self, version_ids):
-        """See VersionedFile.get_deltas."""
-        version_ids = self.get_ancestry(version_ids)
-        for version_id in version_ids:
-            if not self.has_version(version_id):
-                raise RevisionNotPresent(version_id, self)
-        # try extracting all versions; parallel extraction is used
-        nv = self.num_versions()
-        sha1s = {}
-        deltas = {}
-        texts = {}
-        inclusions = {}
-        noeols = {}
-        last_parent_lines = {}
-        parents = {}
-        parent_inclusions = {}
-        parent_linenums = {}
-        parent_noeols = {}
-        current_hunks = {}
-        diff_hunks = {}
-        # its simplest to generate a full set of prepared variables.
-        for i in range(nv):
-            name = self._names[i]
-            sha1s[name] = self.get_sha1(name)
-            parents_list = self.get_parents(name)
-            try:
-                parent = parents_list[0]
-                parents[name] = parent
-                parent_inclusions[name] = inclusions[parent]
-            except IndexError:
-                parents[name] = None
-                parent_inclusions[name] = set()
-            # we want to emit start, finish, replacement_length, replacement_lines tuples.
-            diff_hunks[name] = []
-            current_hunks[name] = [0, 0, 0, []] # #start, finish, repl_length, repl_tuples
-            parent_linenums[name] = 0
-            noeols[name] = False
-            parent_noeols[name] = False
-            last_parent_lines[name] = None
-            new_inc = set([name])
-            for p in self._parents[i]:
-                new_inc.update(inclusions[self._idx_to_name(p)])
-            # debug only, known good so far.
-            #assert set(new_inc) == set(self.get_ancestry(name)), \
-            #    'failed %s != %s' % (set(new_inc), set(self.get_ancestry(name)))
-            inclusions[name] = new_inc
-
-        nlines = len(self._weave)
-
-        for lineno, inserted, deletes, line in self._walk_internal():
-            # a line is active in a version if:
-            # insert is in the versions inclusions
-            # and
-            # deleteset & the versions inclusions is an empty set.
-            # so - if we have a included by mapping - version is included by
-            # children, we get a list of children to examine for deletes affect
-            # ing them, which is less than the entire set of children.
-            for version_id in version_ids:  
-                # The active inclusion must be an ancestor,
-                # and no ancestors must have deleted this line,
-                # because we don't support resurrection.
-                parent_inclusion = parent_inclusions[version_id]
-                inclusion = inclusions[version_id]
-                parent_active = inserted in parent_inclusion and not (deletes & parent_inclusion)
-                version_active = inserted in inclusion and not (deletes & inclusion)
-                if not parent_active and not version_active:
-                    # unrelated line of ancestry
-                    continue
-                elif parent_active and version_active:
-                    # shared line
-                    parent_linenum = parent_linenums[version_id]
-                    if current_hunks[version_id] != [parent_linenum, parent_linenum, 0, []]:
-                        diff_hunks[version_id].append(tuple(current_hunks[version_id]))
-                    parent_linenum += 1
-                    current_hunks[version_id] = [parent_linenum, parent_linenum, 0, []]
-                    parent_linenums[version_id] = parent_linenum
-                    try:
-                        if line[-1] != '\n':
-                            noeols[version_id] = True
-                    except IndexError:
-                        pass
-                elif parent_active and not version_active:
-                    # deleted line
-                    current_hunks[version_id][1] += 1
-                    parent_linenums[version_id] += 1
-                    last_parent_lines[version_id] = line
-                elif not parent_active and version_active:
-                    # replacement line
-                    # noeol only occurs at the end of a file because we 
-                    # diff linewise. We want to show noeol changes as a
-                    # empty diff unless the actual eol-less content changed.
-                    theline = line
-                    try:
-                        if last_parent_lines[version_id][-1] != '\n':
-                            parent_noeols[version_id] = True
-                    except (TypeError, IndexError):
-                        pass
-                    try:
-                        if theline[-1] != '\n':
-                            noeols[version_id] = True
-                    except IndexError:
-                        pass
-                    new_line = False
-                    parent_should_go = False
-
-                    if parent_noeols[version_id] == noeols[version_id]:
-                        # no noeol toggle, so trust the weaves statement
-                        # that this line is changed.
-                        new_line = True
-                        if parent_noeols[version_id]:
-                            theline = theline + '\n'
-                    elif parent_noeols[version_id]:
-                        # parent has no eol, we do:
-                        # our line is new, report as such..
-                        new_line = True
-                    elif noeols[version_id]:
-                        # append a eol so that it looks like
-                        # a normalised delta
-                        theline = theline + '\n'
-                        if parents[version_id] is not None:
-                        #if last_parent_lines[version_id] is not None:
-                            parent_should_go = True
-                        if last_parent_lines[version_id] != theline:
-                            # but changed anyway
-                            new_line = True
-                            #parent_should_go = False
-                    if new_line:
-                        current_hunks[version_id][2] += 1
-                        current_hunks[version_id][3].append((inserted, theline))
-                    if parent_should_go:
-                        # last hunk last parent line is not eaten
-                        current_hunks[version_id][1] -= 1
-                    if current_hunks[version_id][1] < 0:
-                        current_hunks[version_id][1] = 0
-                        # import pdb;pdb.set_trace()
-                    # assert current_hunks[version_id][1] >= 0
-
-        # flush last hunk
-        for i in range(nv):
-            version = self._idx_to_name(i)
-            if current_hunks[version] != [0, 0, 0, []]:
-                diff_hunks[version].append(tuple(current_hunks[version]))
+    def get_parent_map(self, version_ids):
+        """See VersionedFile.get_parent_map."""
         result = {}
         for version_id in version_ids:
-            result[version_id] = (
-                                  parents[version_id],
-                                  sha1s[version_id],
-                                  noeols[version_id],
-                                  diff_hunks[version_id],
-                                  )
+            try:
+                result[version_id] = tuple(
+                    map(self._idx_to_name, self._parents[self._lookup(version_id)]))
+            except RevisionNotPresent:
+                pass
         return result
 
-    def get_parents(self, version_id):
-        """See VersionedFile.get_parent."""
-        return map(self._idx_to_name, self._parents[self._lookup(version_id)])
+    def get_parents_with_ghosts(self, version_id):
+        raise NotImplementedError(self.get_parents_with_ghosts)
 
     def _check_repeated_add(self, name, parents, text, sha1):
         """Check that a duplicated add is OK.
@@ -444,21 +288,14 @@ class Weave(VersionedFile):
             raise RevisionAlreadyPresent(name, self._weave_name)
         return idx
 
-    @deprecated_method(zero_eight)
-    def add_identical(self, old_rev_id, new_rev_id, parents):
-        """Please use Weave.clone_text now."""
-        return self.clone_text(new_rev_id, old_rev_id, parents)
-
-    def _add_lines(self, version_id, parents, lines, parent_texts):
+    def _add_lines(self, version_id, parents, lines, parent_texts,
+       left_matching_blocks, nostore_sha, random_id, check_content):
         """See VersionedFile.add_lines."""
-        return self._add(version_id, lines, map(self._lookup, parents))
+        idx = self._add(version_id, lines, map(self._lookup, parents),
+            nostore_sha=nostore_sha)
+        return sha_strings(lines), sum(map(len, lines)), idx
 
-    @deprecated_method(zero_eight)
-    def add(self, name, parents, text, sha1=None):
-        """See VersionedFile.add_lines for the non deprecated api."""
-        return self._add(name, text, map(self._maybe_lookup, parents), sha1)
-
-    def _add(self, version_id, lines, parents, sha1=None):
+    def _add(self, version_id, lines, parents, sha1=None, nostore_sha=None):
         """Add a single text on top of the weave.
   
         Returns the index number of the newly added version.
@@ -472,13 +309,15 @@ class Weave(VersionedFile):
             
         lines
             Sequence of lines to be added in the new version.
-        """
 
-        assert isinstance(version_id, basestring)
+        :param nostore_sha: See VersionedFile.add_lines.
+        """
         self._check_lines_not_unicode(lines)
         self._check_lines_are_lines(lines)
         if not sha1:
             sha1 = sha_strings(lines)
+        if sha1 == nostore_sha:
+            raise errors.ExistingContent
         if version_id in self._name_map:
             return self._check_repeated_add(version_id, parents, lines, sha1)
 
@@ -528,7 +367,7 @@ class Weave(VersionedFile):
         # another small special case: a merge, producing the same text
         # as auto-merge
         if lines == basis_lines:
-            return new_version            
+            return new_version
 
         # add a sentinel, because we can also match against the final line
         basis_lineno.append(len(self._weave))
@@ -553,14 +392,8 @@ class Weave(VersionedFile):
             #print 'raw match', tag, i1, i2, j1, j2
             if tag == 'equal':
                 continue
-
             i1 = basis_lineno[i1]
             i2 = basis_lineno[i2]
-
-            assert 0 <= j1 <= j2 <= len(lines)
-
-            #print tag, i1, i2, j1, j2
-
             # the deletion and insertion are handled separately.
             # first delete the region.
             if i1 != i2:
@@ -579,11 +412,6 @@ class Weave(VersionedFile):
                 offset += 2 + (j2 - j1)
         return new_version
 
-    def _clone_text(self, new_version_id, old_version_id, parents):
-        """See VersionedFile.clone_text."""
-        old_lines = self.get_text(old_version_id)
-        self.add_lines(new_version_id, parents, old_lines)
-
     def _inclusions(self, versions):
         """Return set of all ancestors of given version(s)."""
         if not len(versions):
@@ -597,16 +425,6 @@ class Weave(VersionedFile):
         ## except IndexError:
         ##     raise ValueError("version %d not present in weave" % v)
 
-    @deprecated_method(zero_eight)
-    def inclusions(self, version_ids):
-        """Deprecated - see VersionedFile.get_ancestry for the replacement."""
-        if not version_ids:
-            return []
-        if isinstance(version_ids[0], int):
-            return [self._idx_to_name(v) for v in self._inclusions(version_ids)]
-        else:
-            return self.get_ancestry(version_ids)
-
     def get_ancestry(self, version_ids, topo_sorted=True):
         """See VersionedFile.get_ancestry."""
         if isinstance(version_ids, basestring):
@@ -641,18 +459,13 @@ class Weave(VersionedFile):
         """
         return len(other_parents.difference(my_parents)) == 0
 
-    def annotate_iter(self, version_id):
-        """Yield list of (version-id, line) pairs for the specified version.
+    def annotate(self, version_id):
+        """Return a list of (version-id, line) tuples for version_id.
 
         The index indicates when the line originated in the weave."""
         incls = [self._lookup(version_id)]
-        for origin, lineno, text in self._extract(incls):
-            yield self._idx_to_name(origin), text
-
-    @deprecated_method(zero_eight)
-    def _walk(self):
-        """_walk has become visit, a supported api."""
-        return self._walk_internal()
+        return [(self._idx_to_name(origin), text) for origin, lineno, text in
+            self._extract(incls)]
 
     def iter_lines_added_or_present_in_versions(self, version_ids=None,
                                                 pb=None):
@@ -666,14 +479,9 @@ class Weave(VersionedFile):
             # properly, we do not filter down to that
             # if inserted not in version_ids: continue
             if line[-1] != '\n':
-                yield line + '\n'
+                yield line + '\n', inserted
             else:
-                yield line
-
-    #@deprecated_method(zero_eight)
-    def walk(self, version_ids=None):
-        """See VersionedFile.walk."""
-        return self._walk_internal(version_ids)
+                yield line, inserted
 
     def _walk_internal(self, version_ids=None):
         """Helper method for weave actions."""
@@ -692,15 +500,12 @@ class Weave(VersionedFile):
                 elif c == '}':
                     istack.pop()
                 elif c == '[':
-                    assert self._names[v] not in dset
                     dset.add(self._names[v])
                 elif c == ']':
                     dset.remove(self._names[v])
                 else:
                     raise WeaveFormatError('unexpected instruction %r' % v)
             else:
-                assert l.__class__ in (str, unicode)
-                assert istack
                 yield lineno, istack[-1], frozenset(dset), l
             lineno += 1
 
@@ -723,8 +528,7 @@ class Weave(VersionedFile):
         inc_b = set(self.get_ancestry([ver_b]))
         inc_c = inc_a & inc_b
 
-        for lineno, insert, deleteset, line in\
-            self.walk([ver_a, ver_b]):
+        for lineno, insert, deleteset, line in self._walk_internal([ver_a, ver_b]):
             if deleteset & inc_c:
                 # killed in parent; can't be in either a or b
                 # not relevant to our work
@@ -817,22 +621,19 @@ class Weave(VersionedFile):
                 c, v = l
                 isactive = None
                 if c == '{':
-                    assert v not in iset
                     istack.append(v)
                     iset.add(v)
                 elif c == '}':
                     iset.remove(istack.pop())
                 elif c == '[':
                     if v in included:
-                        assert v not in dset
                         dset.add(v)
-                else:
-                    assert c == ']'
+                elif c == ']':
                     if v in included:
-                        assert v in dset
                         dset.remove(v)
+                else:
+                    raise AssertionError()
             else:
-                assert l.__class__ in (str, unicode)
                 if isactive is None:
                     isactive = (not dset) and istack and (istack[-1] in included)
                 if isactive:
@@ -846,19 +647,6 @@ class Weave(VersionedFile):
                                    % dset)
         return result
 
-    @deprecated_method(zero_eight)
-    def get_iter(self, name_or_index):
-        """Deprecated, please do not use. Lookups are not not needed.
-        
-        Please use get_lines now.
-        """
-        return iter(self.get_lines(self._maybe_lookup(name_or_index)))
-
-    @deprecated_method(zero_eight)
-    def maybe_lookup(self, name_or_index):
-        """Deprecated, please do not use. Lookups are not not needed."""
-        return self._maybe_lookup(name_or_index)
-
     def _maybe_lookup(self, name_or_index):
         """Convert possible symbolic name to index, or pass through indexes.
         
@@ -869,11 +657,6 @@ class Weave(VersionedFile):
         else:
             return self._lookup(name_or_index)
 
-    @deprecated_method(zero_eight)
-    def get(self, version_id):
-        """Please use either Weave.get_text or Weave.get_lines as desired."""
-        return self.get_lines(version_id)
-
     def get_lines(self, version_id):
         """See VersionedFile.get_lines()."""
         int_index = self._maybe_lookup(version_id)
@@ -887,26 +670,13 @@ class Weave(VersionedFile):
                        expected_sha1, measured_sha1))
         return result
 
-    def get_sha1(self, version_id):
-        """See VersionedFile.get_sha1()."""
-        return self._sha1s[self._lookup(version_id)]
-
     def get_sha1s(self, version_ids):
         """See VersionedFile.get_sha1s()."""
         return [self._sha1s[self._lookup(v)] for v in version_ids]
 
-    @deprecated_method(zero_eight)
-    def numversions(self):
-        """How many versions are in this weave?
-
-        Deprecated in favour of num_versions.
-        """
-        return self.num_versions()
-
     def num_versions(self):
         """How many versions are in this weave?"""
         l = len(self._parents)
-        assert l == len(self._sha1s)
         return l
 
     __len__ = num_versions
@@ -938,8 +708,10 @@ class Weave(VersionedFile):
             for p in self._parents[i]:
                 new_inc.update(inclusions[self._idx_to_name(p)])
 
-            assert set(new_inc) == set(self.get_ancestry(name)), \
-                'failed %s != %s' % (set(new_inc), set(self.get_ancestry(name)))
+            if set(new_inc) != set(self.get_ancestry(name)):
+                raise AssertionError(
+                    'failed %s != %s' 
+                    % (set(new_inc), set(self.get_ancestry(name))))
             inclusions[name] = new_inc
 
         nlines = len(self._weave)
@@ -993,8 +765,11 @@ class Weave(VersionedFile):
         version_ids = set(other.versions()).intersection(set(version_ids))
         # pull in the referenced graph.
         version_ids = other.get_ancestry(version_ids)
-        pending_graph = [(version, other.get_parents(version)) for
-                         version in version_ids]
+        pending_parents = other.get_parent_map(version_ids)
+        pending_graph = pending_parents.items()
+        if len(pending_graph) != len(version_ids):
+            raise RevisionNotPresent(
+                set(version_ids) - set(pending_parents.keys()), self)
         for name in topo_sort(pending_graph):
             other_idx = other._name_map[name]
             # returns True if we have it, False if we need it.
@@ -1002,7 +777,6 @@ class Weave(VersionedFile):
                 names_to_join.append((other_idx, name))
             processed += 1
 
-
         if pb and not msg:
             msg = 'weave join'
 
@@ -1066,11 +840,6 @@ class Weave(VersionedFile):
         else:
             return False
 
-    @deprecated_method(zero_eight)
-    def reweave(self, other, pb=None, msg=None):
-        """reweave has been superseded by plain use of join."""
-        return self.join(other, pb, msg)
-
     def _reweave(self, other, pb, msg):
         """Reweave self with other - internal helper for join().
 
@@ -1093,12 +862,12 @@ class WeaveFile(Weave):
 
     WEAVE_SUFFIX = '.weave'
     
-    def __init__(self, name, transport, filemode=None, create=False, access_mode='w'):
+    def __init__(self, name, transport, filemode=None, create=False, access_mode='w', get_scope=None):
         """Create a WeaveFile.
         
         :param create: If not True, only open an existing knit.
         """
-        super(WeaveFile, self).__init__(name, access_mode)
+        super(WeaveFile, self).__init__(name, access_mode, get_scope=get_scope)
         self._transport = transport
         self._filemode = filemode
         try:
@@ -1109,19 +878,16 @@ class WeaveFile(Weave):
             # new file, save it
             self._save()
 
-    def _add_lines(self, version_id, parents, lines, parent_texts):
+    def _add_lines(self, version_id, parents, lines, parent_texts,
+        left_matching_blocks, nostore_sha, random_id, check_content):
         """Add a version and save the weave."""
         self.check_not_reserved_id(version_id)
         result = super(WeaveFile, self)._add_lines(version_id, parents, lines,
-                                                   parent_texts)
+            parent_texts, left_matching_blocks, nostore_sha, random_id,
+            check_content)
         self._save()
         return result
 
-    def _clone_text(self, new_version_id, old_version_id, parents):
-        """See VersionedFile.clone_text."""
-        super(WeaveFile, self)._clone_text(new_version_id, old_version_id, parents)
-        self._save
-
     def copy_to(self, name, transport):
         """See VersionedFile.copy_to()."""
         # as we are all in memory always, just serialise to the new place.
@@ -1130,9 +896,6 @@ class WeaveFile(Weave):
         sio.seek(0)
         transport.put_file(name + WeaveFile.WEAVE_SUFFIX, sio, self._filemode)
 
-    def create_empty(self, name, transport, filemode=None):
-        return WeaveFile(name, transport, filemode, create=True)
-
     def _save(self):
         """Save the weave."""
         self._check_write_ok()
@@ -1155,11 +918,6 @@ class WeaveFile(Weave):
         self._save()
 
 
-@deprecated_function(zero_eight)
-def reweave(wa, wb, pb=None, msg=None):
-    """reweaving is deprecation, please just use weave.join()."""
-    _reweave(wa, wb, pb, msg)
-
 def _reweave(wa, wb, pb=None, msg=None):
     """Combine two weaves and return the result.
 
@@ -1445,10 +1203,7 @@ class InterWeave(InterVersionedFile):
         if self.target.versions() == [] and version_ids is None:
             self.target._copy_weave_content(self.source)
             return
-        try:
-            self.target._join(self.source, pb, msg, version_ids, ignore_missing)
-        except errors.WeaveParentMismatch:
-            self.target._reweave(self.source, pb, msg)
+        self.target._join(self.source, pb, msg, version_ids, ignore_missing)
 
 
 InterVersionedFile.register_optimiser(InterWeave)
