@@ -51,23 +51,35 @@ from bzrlib import errors
 
 class Serializer(object):
     """Abstract object serialize/deserialize"""
+
     def write_inventory(self, inv, f):
         """Write inventory to a file"""
-        elt = self._pack_inventory(inv)
-        self._write_element(elt, f)
+        raise NotImplementedError(self.write_inventory)
 
     def write_inventory_to_string(self, inv):
-        return tostring(self._pack_inventory(inv)) + '\n'
+        raise NotImplementedError(self.write_inventory_to_string)
 
-    def read_inventory_from_string(self, xml_string):
+    def read_inventory_from_string(self, xml_string, revision_id=None):
+        """Read xml_string into an inventory object.
+
+        :param xml_string: The xml to read.
+        :param revision_id: If not-None, the expected revision id of the
+            inventory. Some serialisers use this to set the results' root
+            revision. This should be supplied for deserialising all
+            from-repository inventories so that xml5 inventories that were
+            serialised without a revision identifier can be given the right
+            revision id (but not for working tree inventories where users can
+            edit the data without triggering checksum errors or anything).
+        """
         try:
-            return self._unpack_inventory(fromstring(xml_string))
+            return self._unpack_inventory(fromstring(xml_string), revision_id)
         except ParseError, e:
             raise errors.UnexpectedInventoryFormat(e)
 
-    def read_inventory(self, f):
+    def read_inventory(self, f, revision_id=None):
         try:
-            return self._unpack_inventory(self._read_element(f))
+            return self._unpack_inventory(self._read_element(f),
+                revision_id=None)
         except ParseError, e:
             raise errors.UnexpectedInventoryFormat(e)
 
@@ -167,3 +179,4 @@ format_registry.register_lazy('4', 'bzrlib.xml4', 'serializer_v4')
 format_registry.register_lazy('5', 'bzrlib.xml5', 'serializer_v5')
 format_registry.register_lazy('6', 'bzrlib.xml6', 'serializer_v6')
 format_registry.register_lazy('7', 'bzrlib.xml7', 'serializer_v7')
+format_registry.register_lazy('8', 'bzrlib.xml8', 'serializer_v8')
